@@ -1,7 +1,8 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2008 Jonathan Kleinehellefort
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -11,11 +12,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -125,12 +124,12 @@ static int op_alsa_exit(void)
 /* randomize hw params */
 static int alsa_set_hw_params(void)
 {
-	snd_pcm_hw_params_t *hwparams;
+	snd_pcm_hw_params_t *hwparams = NULL;
 	const char *cmd;
 	unsigned int rate;
 	int rc, dir;
 
-	snd_pcm_hw_params_alloca(&hwparams);
+	snd_pcm_hw_params_malloc(&hwparams);
 
 	cmd = "snd_pcm_hw_params_any";
 	rc = snd_pcm_hw_params_any(alsa_handle, hwparams);
@@ -171,13 +170,15 @@ static int alsa_set_hw_params(void)
 	rc = snd_pcm_hw_params(alsa_handle, hwparams);
 	if (rc < 0)
 		goto error;
-	return 0;
+	goto out;
 error:
 	d_print("%s: error: %s\n", cmd, snd_strerror(rc));
+out:
+	snd_pcm_hw_params_free(hwparams);
 	return rc;
 }
 
-static int op_alsa_open(sample_format_t sf)
+static int op_alsa_open(sample_format_t sf, const channel_position_t *channel_map)
 {
 	int rc;
 
