@@ -1,4 +1,5 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2006 Johannes Weißl
  *
  * This program is free software; you can redistribute it and/or
@@ -12,15 +13,14 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "op.h"
 #include "xmalloc.h"
 #include "utils.h"
 #include "misc.h"
+#include "debug.h"
 
 /*
  * <ao/ao.h> uses FILE but doesn't include stdio.h.
@@ -30,6 +30,10 @@
 #include <strings.h>
 #include <ao/ao.h>
 
+#ifdef AO_EBADFORMAT
+#define AO_API_1
+#endif
+
 static ao_device *libao_device;
 static char *wav_dir = NULL;
 static int wav_counter = 1;
@@ -56,9 +60,66 @@ static int op_ao_exit(void)
 	return 0;
 }
 
-static int op_ao_open(sample_format_t sf)
+/* http://www.xiph.org/ao/doc/ao_sample_format.html */
+static const struct {
+	channel_position_t pos;
+	const char *str;
+} ao_channel_mapping[] = {
+	{ CHANNEL_POSITION_LEFT,			"L" },
+	{ CHANNEL_POSITION_RIGHT,			"R" },
+	{ CHANNEL_POSITION_CENTER,			"C" },
+	{ CHANNEL_POSITION_MONO,			"M" },
+	{ CHANNEL_POSITION_FRONT_LEFT_OF_CENTER,	"CL" },
+	{ CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER,	"CR" },
+	{ CHANNEL_POSITION_REAR_LEFT,			"BL" },
+	{ CHANNEL_POSITION_REAR_RIGHT,			"BR" },
+	{ CHANNEL_POSITION_REAR_CENTER,			"BC" },
+	{ CHANNEL_POSITION_SIDE_LEFT,			"SL" },
+	{ CHANNEL_POSITION_SIDE_RIGHT,			"SR" },
+	{ CHANNEL_POSITION_LFE,				"LFE" },
+	{ CHANNEL_POSITION_INVALID,			"X" },
+};
+
+#ifdef AO_API_1
+static char *ao_channel_matrix(int channels, const channel_position_t *map)
 {
-	ao_sample_format format;
+	int i, j;
+	char buf[256] = "";
+
+	if (!map || !channel_map_valid(map))
+		return NULL;
+
+	for (i = 0; i < channels; i++) {
+		const channel_position_t pos = map[i];
+		int found = 0;
+		for (j = 0; j < N_ELEMENTS(ao_channel_mapping); j++) {
+			if (pos == ao_channel_mapping[j].pos) {
+				strcat(buf, ao_channel_mapping[j].str);
+				strcat(buf, ",");
+				found = 1;
+				break;
+			}
+		}
+		if (!found)
+			strcat(buf, "M,");
+	}
+	buf[strlen(buf)-1] = '\0';
+
+	return xstrdup(buf);
+}
+#endif
+
+static int op_ao_open(sample_format_t sf, const channel_position_t *channel_map)
+{
+	ao_sample_format format = {
+		.bits        = sf_get_bits(sf),
+		.rate        = sf_get_rate(sf),
+		.channels    = sf_get_channels(sf),
+		.byte_format = sf_get_bigendian(sf) ? AO_FMT_BIG : AO_FMT_LITTLE,
+#ifdef AO_API_1
+		.matrix      = ao_channel_matrix(sf_get_channels(sf), channel_map)
+#endif
+	};
 	int driver;
 
 	if (libao_driver == NULL) {
@@ -72,13 +133,6 @@ static int op_ao_open(sample_format_t sf)
 		return -OP_ERROR_ERRNO;
 	}
 
-	memset(&format, 0, sizeof format);
-
-	format.bits = sf_get_bits(sf);
-	format.rate = sf_get_rate(sf);
-	format.channels = sf_get_channels(sf);
-	format.byte_format = sf_get_bigendian(sf) ? AO_FMT_BIG : AO_FMT_LITTLE;
-
 	if (is_wav) {
 		char file[512];
 
@@ -112,6 +166,10 @@ static int op_ao_open(sample_format_t sf)
 			return -OP_ERROR_INTERNAL;
 		}
 	}
+
+#ifdef AO_API_1
+	d_print("channel matrix: %s\n", format.matrix ? format.matrix : "default");
+#endif
 	return 0;
 }
 
@@ -190,7 +248,7 @@ static int op_ao_get_option(int key, char **val)
 	case 3:
 		if (wav_dir == NULL)
 			wav_dir = xstrdup(home_dir);
-		*val = xstrdup(wav_dir);
+		*val = expand_filename(wav_dir);
 		break;
 	default:
 		return -OP_ERROR_NOT_OPTION;
