@@ -1,10 +1,23 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2006 Chun-Yu Shei <cshei AT cs.indiana.edu>
  *
  * Cleaned up by Timo Hirvonen <tihirvon@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "ip.h"
 #include "ape.h"
 #include "file.h"
 #include "xmalloc.h"
@@ -189,7 +202,7 @@ static off_t get_size(int fd)
 {
 	struct stat statbuf;
 
-	if (fstat(fd, &statbuf) || !(statbuf.st_mode & S_IFREG))
+	if (fstat(fd, &statbuf) || !S_ISREG(statbuf.st_mode))
 		return 0;
 	return statbuf.st_size;
 }
@@ -198,7 +211,8 @@ static off_t get_size(int fd)
 int ape_read_tags(struct apetag *ape, int fd, int slow)
 {
 	struct ape_header *h = &ape->header;
-	int old_pos, rc = -1;
+	int rc = -1;
+	off_t old_pos;
 
 	/* save position */
 	old_pos = lseek(fd, 0, SEEK_CUR);
