@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,22 +11,20 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "browser.h"
 #include "load_dir.h"
 #include "cmus.h"
 #include "xmalloc.h"
-#include "xstrjoin.h"
 #include "ui_curses.h"
 #include "file.h"
 #include "misc.h"
 #include "options.h"
+#include "uchar.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -33,6 +32,7 @@
 #include <dirent.h>
 #include <string.h>
 #include <errno.h>
+#include <sys/mman.h>
 
 struct window *browser_win;
 struct searchable *browser_searchable;
@@ -147,6 +147,11 @@ static int do_browser_load(const char *name)
 		free_browser_list();
 
 		if (buf) {
+			struct browser_entry *parent_dir_e = xmalloc(sizeof(struct browser_entry) + 4);
+			strcpy(parent_dir_e->name, "../");
+			parent_dir_e->type = BROWSER_ENTRY_DIR;
+			list_add_tail(&parent_dir_e->node, &browser_head);
+
 			cmus_playlist_for_each(buf, size, 0, add_pl_line, NULL);
 			munmap(buf, size);
 		}
