@@ -1,3 +1,21 @@
+/*
+ * Copyright 2008-2011 Various Authors
+ * Copyright 2004 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "cmus.h"
 #include "job.h"
 #include "lib.h"
@@ -13,12 +31,12 @@
 #include "path.h"
 #include "options.h"
 #include "xmalloc.h"
-#include "xstrjoin.h"
 #include "debug.h"
 #include "load_dir.h"
 #include "ui_curses.h"
 #include "cache.h"
 #include "gbuf.h"
+#include "discid.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -92,7 +110,7 @@ void cmus_play_file(const char *filename)
 	struct track_info *ti;
 
 	cache_lock();
-	ti = cache_get_ti(filename);
+	ti = cache_get_ti(filename, 0);
 	cache_unlock();
 	if (!ti) {
 		error_msg("Couldn't get file information for %s\n", filename);
@@ -107,11 +125,16 @@ enum file_type cmus_detect_ft(const char *name, char **ret)
 	char *absolute;
 	struct stat st;
 
-	if (is_url(name)) {
+	if (is_http_url(name) || is_cue_url(name)) {
 		*ret = xstrdup(name);
 		return FILE_TYPE_URL;
 	}
 
+	if (is_cdda_url(name)) {
+		*ret = complete_cdda_url(cdda_device, name);
+		return FILE_TYPE_CDDA;
+	}
+
 	*ret = NULL;
 	absolute = path_absolute(name);
 	if (absolute == NULL)
@@ -141,13 +164,14 @@ enum file_type cmus_detect_ft(const char *name, char **ret)
 	return FILE_TYPE_FILE;
 }
 
-void cmus_add(add_ti_cb add, const char *name, enum file_type ft, int jt)
+void cmus_add(add_ti_cb add, const char *name, enum file_type ft, int jt, int force)
 {
 	struct add_data *data = xnew(struct add_data, 1);
 
 	data->add = add;
 	data->name = xstrdup(name);
 	data->type = ft;
+	data->force = force;
 	worker_add_job(jt, do_add_job, free_add_job, data);
 }
 
@@ -233,9 +257,14 @@ static int update_cb(void *data, struct track_info *ti)
 	return 0;
 }
 
-void cmus_update_cache(void)
+void cmus_update_cache(int force)
 {
-	worker_add_job(JOB_TYPE_LIB, do_update_cache_job, free_update_cache_job, NULL);
+	struct update_cache_data *data;
+
+	data = xnew(struct update_cache_data, 1);
+	data->force = force;
+
+	worker_add_job(JOB_TYPE_LIB, do_update_cache_job, free_update_cache_job, data);
 }
 
 void cmus_update_lib(void)
@@ -254,7 +283,7 @@ void cmus_update_lib(void)
 	worker_add_job(JOB_TYPE_LIB, do_update_job, free_update_job, data);
 }
 
-void cmus_update_tis(struct track_info **tis, int nr)
+void cmus_update_tis(struct track_info **tis, int nr, int force)
 {
 	struct update_data *data;
 
@@ -262,6 +291,7 @@ void cmus_update_tis(struct track_info **tis, int nr)
 	data->size = nr;
 	data->used = nr;
 	data->ti = tis;
+	data->force = force;
 	worker_add_job(JOB_TYPE_LIB, do_update_job, free_update_job, data);
 }
 
