@@ -1,3 +1,21 @@
+/*
+ * Copyright 2008-2011 Various Authors
+ * Copyright 2004 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ */
+
 #ifndef _CMUS_H
 #define _CMUS_H
 
@@ -20,7 +38,8 @@ enum file_type {
 	FILE_TYPE_URL,
 	FILE_TYPE_PL,
 	FILE_TYPE_DIR,
-	FILE_TYPE_FILE
+	FILE_TYPE_FILE,
+	FILE_TYPE_CDDA
 };
 
 typedef int (*track_info_cb)(void *data, struct track_info *ti);
@@ -53,14 +72,14 @@ enum file_type cmus_detect_ft(const char *name, char **ret);
  *
  * returns immediately, actual work is done in the worker thread.
  */
-void cmus_add(add_ti_cb, const char *name, enum file_type ft, int jt);
+void cmus_add(add_ti_cb, const char *name, enum file_type ft, int jt, int force);
 
 int cmus_save(for_each_ti_cb for_each_ti, const char *filename);
 int cmus_save_ext(for_each_ti_cb for_each_ti, const char *filename);
 
-void cmus_update_cache(void);
+void cmus_update_cache(int force);
 void cmus_update_lib(void);
-void cmus_update_tis(struct track_info **tis, int nr);
+void cmus_update_tis(struct track_info **tis, int nr, int force);
 
 int cmus_is_playlist(const char *filename);
 int cmus_is_playable(const char *filename);
