@@ -1,4 +1,5 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
  *
  * This program is free software; you can redistribute it and/or
@@ -12,9 +13,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "command_mode.h"
@@ -40,21 +39,22 @@
 #include "xstrjoin.h"
 #include "misc.h"
 #include "path.h"
-#include "format_print.h"
 #include "spawn.h"
 #include "utils.h"
 #include "list.h"
 #include "debug.h"
 #include "load_dir.h"
+#ifdef HAVE_CONFIG
 #include "config/datadir.h"
+#endif
 #include "help.h"
+#include "op.h"
 
 #include <stdlib.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <dirent.h>
-#include <pwd.h>
 
 #if defined(__sun__)
 #include <ncurses.h>
@@ -68,63 +68,6 @@ static char *history_search_text = NULL;
 static int arg_expand_cmd = -1;
 static int prev_view = -1;
 
-static char *get_home_dir(const char *username)
-{
-	struct passwd *passwd;
-
-	if (username == NULL)
-		return xstrdup(home_dir);
-	passwd = getpwnam(username);
-	if (passwd == NULL)
-		return NULL;
-	/* don't free passwd */
-	return xstrdup(passwd->pw_dir);
-}
-
-static char *expand_filename(const char *name)
-{
-	if (name[0] == '~') {
-		char *slash;
-
-		slash = strchr(name, '/');
-		if (slash) {
-			char *username, *home;
-
-			if (slash - name - 1 > 0) {
-				/* ~user/... */
-				username = xstrndup(name + 1, slash - name - 1);
-			} else {
-				/* ~/... */
-				username = NULL;
-			}
-			home = get_home_dir(username);
-			free(username);
-			if (home) {
-				char *expanded;
-
-				expanded = xstrjoin(home, slash);
-				free(home);
-				return expanded;
-			} else {
-				return xstrdup(name);
-			}
-		} else {
-			if (name[1] == 0) {
-				return xstrdup(home_dir);
-			} else {
-				char *home;
-
-				home = get_home_dir(name + 1);
-				if (home)
-					return home;
-				return xstrdup(name);
-			}
-		}
-	} else {
-		return xstrdup(name);
-	}
-}
-
 /* view {{{ */
 
 void view_clear(int view)
@@ -175,16 +118,16 @@ void view_add(int view, char *arg, int prepend)
 	switch (view) {
 	case TREE_VIEW:
 	case SORTED_VIEW:
-		cmus_add(lib_add_track, name, ft, JOB_TYPE_LIB);
+		cmus_add(lib_add_track, name, ft, JOB_TYPE_LIB, 0);
 		break;
 	case PLAYLIST_VIEW:
-		cmus_add(pl_add_track, name, ft, JOB_TYPE_PL);
+		cmus_add(pl_add_track, name, ft, JOB_TYPE_PL, 0);
 		break;
 	case QUEUE_VIEW:
 		if (prepend) {
-			cmus_add(play_queue_prepend, name, ft, JOB_TYPE_QUEUE);
+			cmus_add(play_queue_prepend, name, ft, JOB_TYPE_QUEUE, 0);
 		} else {
-			cmus_add(play_queue_append, name, ft, JOB_TYPE_QUEUE);
+			cmus_add(play_queue_append, name, ft, JOB_TYPE_QUEUE, 0);
 		}
 		break;
 	default:
@@ -222,7 +165,7 @@ void view_load(int view, char *arg)
 		editable_lock();
 		editable_clear(&lib_editable);
 		editable_unlock();
-		cmus_add(lib_add_track, name, FILE_TYPE_PL, JOB_TYPE_LIB);
+		cmus_add(lib_add_track, name, FILE_TYPE_PL, JOB_TYPE_LIB, 0);
 		free(lib_filename);
 		lib_filename = name;
 		break;
@@ -231,7 +174,7 @@ void view_load(int view, char *arg)
 		editable_lock();
 		editable_clear(&pl_editable);
 		editable_unlock();
-		cmus_add(pl_add_track, name, FILE_TYPE_PL, JOB_TYPE_PL);
+		cmus_add(pl_add_track, name, FILE_TYPE_PL, JOB_TYPE_PL, 0);
 		free(pl_filename);
 		pl_filename = name;
 		break;
@@ -672,7 +615,8 @@ static void cmd_unmark(char *arg)
 
 static void cmd_update_cache(char *arg)
 {
-	cmus_update_cache();
+	int flag = parse_flags((const char **)&arg, "f");
+	cmus_update_cache(flag == 'f');
 }
 
 static void cmd_cd(char *arg)
@@ -1005,18 +949,21 @@ error:
 	return NULL;
 }
 
-static struct track_info **sel_tis;
-static int sel_tis_alloc;
-static int sel_tis_nr;
+struct track_info_selection {
+	struct track_info **tis;
+	int tis_alloc;
+	int tis_nr;
+};
 
 static int add_ti(void *data, struct track_info *ti)
 {
-	if (sel_tis_nr == sel_tis_alloc) {
-		sel_tis_alloc = sel_tis_alloc ? sel_tis_alloc * 2 : 8;
-		sel_tis = xrenew(struct track_info *, sel_tis, sel_tis_alloc);
+	struct track_info_selection *sel = data;
+	if (sel->tis_nr == sel->tis_alloc) {
+		sel->tis_alloc = sel->tis_alloc ? sel->tis_alloc * 2 : 8;
+		sel->tis = xrenew(struct track_info *, sel->tis, sel->tis_alloc);
 	}
 	track_info_ref(ti);
-	sel_tis[sel_tis_nr++] = ti;
+	sel->tis[sel->tis_nr++] = ti;
 	return 0;
 }
 
@@ -1024,6 +971,7 @@ static void cmd_run(char *arg)
 {
 	char **av, **argv;
 	int ac, argc, i, run, files_idx = -1;
+	struct track_info_selection sel = { .tis = NULL };
 
 	if (cur_view > QUEUE_VIEW) {
 		info_msg("Command execution is supported only in views 1-4");
@@ -1036,48 +984,44 @@ static void cmd_run(char *arg)
 	}
 
 	/* collect selected files (struct track_info) */
-	sel_tis = NULL;
-	sel_tis_alloc = 0;
-	sel_tis_nr = 0;
-
 	editable_lock();
 	switch (cur_view) {
 	case TREE_VIEW:
 		__tree_for_each_sel(add_ti, NULL, 0);
 		break;
 	case SORTED_VIEW:
-		__editable_for_each_sel(&lib_editable, add_ti, NULL, 0);
+		__editable_for_each_sel(&lib_editable, add_ti, &sel, 0);
 		break;
 	case PLAYLIST_VIEW:
-		__editable_for_each_sel(&pl_editable, add_ti, NULL, 0);
+		__editable_for_each_sel(&pl_editable, add_ti, &sel, 0);
 		break;
 	case QUEUE_VIEW:
-		__editable_for_each_sel(&pq_editable, add_ti, NULL, 0);
+		__editable_for_each_sel(&pq_editable, add_ti, &sel, 0);
 		break;
 	}
 	editable_unlock();
 
-	if (sel_tis_nr == 0) {
+	if (sel.tis_nr == 0) {
 		/* no files selected, do nothing */
 		free_str_array(av);
 		return;
 	}
-	sel_tis[sel_tis_nr] = NULL;
+	sel.tis[sel.tis_nr] = NULL;
 
 	/* build argv */
-	argv = xnew(char *, ac + sel_tis_nr + 1);
+	argv = xnew(char *, ac + sel.tis_nr + 1);
 	argc = 0;
 	if (files_idx == -1) {
 		/* add selected files after rest of the args */
 		for (i = 0; i < ac; i++)
 			argv[argc++] = av[i];
-		for (i = 0; i < sel_tis_nr; i++)
-			argv[argc++] = sel_tis[i]->filename;
+		for (i = 0; i < sel.tis_nr; i++)
+			argv[argc++] = sel.tis[i]->filename;
 	} else {
 		for (i = 0; i < files_idx; i++)
 			argv[argc++] = av[i];
-		for (i = 0; i < sel_tis_nr; i++)
-			argv[argc++] = sel_tis[i]->filename;
+		for (i = 0; i < sel.tis_nr; i++)
+			argv[argc++] = sel.tis[i]->filename;
 		for (i = files_idx; i < ac; i++)
 			argv[argc++] = av[i];
 	}
@@ -1087,8 +1031,8 @@ static void cmd_run(char *arg)
 		d_print("ARG: '%s'\n", argv[i]);
 
 	run = 1;
-	if (confirm_run && (sel_tis_nr > 1 || strcmp(argv[0], "rm") == 0)) {
-		if (!yes_no_query("Execute %s for the %d selected files? [y/N]", arg, sel_tis_nr)) {
+	if (confirm_run && (sel.tis_nr > 1 || strcmp(argv[0], "rm") == 0)) {
+		if (!yes_no_query("Execute %s for the %d selected files? [y/N]", arg, sel.tis_nr)) {
 			info_msg("Aborted");
 			run = 0;
 		}
@@ -1111,23 +1055,23 @@ static void cmd_run(char *arg)
 			switch (cur_view) {
 			case TREE_VIEW:
 			case SORTED_VIEW:
-				/* this must be done before sel_tis are unreffed */
+				/* this must be done before sel.tis are unreffed */
 				free_str_array(av);
 				free(argv);
 
 				/* remove non-existed files, update tags for changed files */
-				cmus_update_tis(sel_tis, sel_tis_nr);
+				cmus_update_tis(sel.tis, sel.tis_nr, 0);
 
-				/* we don't own sel_tis anymore! */
+				/* we don't own sel.tis anymore! */
 				return;
 			}
 		}
 	}
 	free_str_array(av);
 	free(argv);
-	for (i = 0; sel_tis[i]; i++)
-		track_info_unref(sel_tis[i]);
-	free(sel_tis);
+	for (i = 0; sel.tis[i]; i++)
+		track_info_unref(sel.tis[i]);
+	free(sel.tis);
 }
 
 static void cmd_shell(char *arg)
@@ -1419,9 +1363,14 @@ static void add_from_browser(add_ti_cb add, int job_type)
 		enum file_type ft;
 		char *ret;
 
+		if (ends_with(sel, "/../") || ends_with(sel, "/..")) {
+			info_msg("For convenience, you can not add \"..\" directory from the browser view");
+			return;
+		}
+
 		ft = cmus_detect_ft(sel, &ret);
 		if (ft != FILE_TYPE_INVALID) {
-			cmus_add(add, ret, ft, job_type);
+			cmus_add(add, ret, ft, job_type, 0);
 			window_down(browser_win, 1);
 		}
 		free(ret);
@@ -1726,6 +1675,44 @@ static void cmd_win_pg_up(char *arg)
 	editable_unlock();
 }
 
+static void cmd_win_pg_top(char *arg)
+{
+	editable_lock();
+	window_page_top(current_win());
+	editable_unlock();
+}
+
+static void cmd_win_pg_bottom(char *arg)
+{
+	editable_lock();
+	window_page_bottom(current_win());
+	editable_unlock();
+}
+
+static void cmd_win_pg_middle(char *arg)
+{
+	editable_lock();
+	window_page_middle(current_win());
+	editable_unlock();
+}
+
+static void cmd_win_update_cache(char *arg)
+{
+	struct track_info_selection sel = { .tis = NULL };
+	int flag = parse_flags((const char **)&arg, "f");
+
+	if (cur_view != TREE_VIEW && cur_view != SORTED_VIEW)
+		return;
+
+	editable_lock();
+	view_for_each_sel[cur_view](add_ti, &sel, 0);
+	editable_unlock();
+	if (sel.tis_nr == 0)
+		return;
+	sel.tis[sel.tis_nr] = NULL;
+	cmus_update_tis(sel.tis, sel.tis_nr, flag == 'f');
+}
+
 static void cmd_win_top(char *arg)
 {
 	editable_lock();
@@ -1976,6 +1963,11 @@ static int filter_directories(const char *name, const struct stat *s)
 	return S_ISDIR(s->st_mode);
 }
 
+static int filter_executable_files(const char *name, const struct stat *s)
+{
+	return S_ISREG(s->st_mode) && (s->st_mode & 0111);
+}
+
 static int filter_any(const char *name, const struct stat *s)
 {
 	return 1;
@@ -2038,6 +2030,27 @@ static void expand_add(const char *str)
 	}
 }
 
+static void expand_program_paths(const char *str)
+{
+	if (str == NULL)
+		str = "";
+	if (str[0] == '~' || strchr(str, '/'))
+		expand_files(str);
+	else
+		expand_env_path(str, filter_executable_files);
+}
+
+static void expand_program_paths_option(const char *str, const char *opt)
+{
+	expand_program_paths(str);
+
+	if (tabexp.head && opt) {
+		snprintf(expbuf, sizeof(expbuf), "%s=%s", opt, tabexp.head);
+		free(tabexp.head);
+		tabexp.head = xstrdup(expbuf);
+	}
+}
+
 static void expand_load_save(const char *str)
 {
 	int flag = parse_flags(&str, "lp");
@@ -2304,33 +2317,57 @@ static void expand_factivate(const char *str)
 	tabexp.count = array.count;
 }
 
+static void expand_fset(const char *str)
+{
+	struct filter_entry *e;
+	PTR_ARRAY(array);
+
+	list_for_each_entry(e, &filters_head, node) {
+		char *line = xnew(char, strlen(e->name) + strlen(e->filter) + 2);
+		sprintf(line, "%s=%s", e->name, e->filter);
+		if (!strncmp(str, line, strlen(str)))
+			ptr_array_add(&array, xstrdup(line + strlen(str)));
+		free(line);
+	}
+	if (!array.count)
+		return;
+
+	tabexp.head = xstrdup(str);
+	tabexp.tails = array.ptrs;
+	tabexp.count = array.count;
+}
+
 static void expand_options(const char *str)
 {
 	struct cmus_opt *opt;
 	int len;
-	char **tails;
+	char **tails, *sep;
 
 	/* tabexp is resetted */
 	len = strlen(str);
-	if (len > 1 && str[len - 1] == '=') {
+	sep = strchr(str, '=');
+	if (len > 1 && sep) {
 		/* expand value */
-		char *var = xstrndup(str, len - 1);
+		char *var = xstrndup(str, sep - str);
 
 		list_for_each_entry(opt, &option_head, node) {
 			if (strcmp(var, opt->name) == 0) {
-				char buf[OPTION_MAX_SIZE];
+				if (str[len - 1] == '=') {
+					char buf[OPTION_MAX_SIZE];
 
-				tails = xnew(char *, 1);
+					tails = xnew(char *, 1);
 
-				buf[0] = 0;
-				opt->get(opt->id, buf);
-				tails[0] = xstrdup(buf);
+					buf[0] = 0;
+					opt->get(opt->id, buf);
+					tails[0] = xstrdup(buf);
 
-				tabexp.head = xstrdup(str);
-				tabexp.tails = tails;
-				tabexp.count = 1;
-				free(var);
-				return;
+					tabexp.head = xstrdup(str);
+					tabexp.tails = tails;
+					tabexp.count = 1;
+				} else if (opt->flags & OPT_PROGRAM_PATH) {
+					expand_program_paths_option(sep + 1, var);
+				}
+				break;
 			}
 		}
 		free(var);
@@ -2445,7 +2482,7 @@ struct command commands[] = {
 	{ "echo",		cmd_echo,	1,-1, NULL,		  0, 0 },
 	{ "factivate",		cmd_factivate,	0, 1, expand_factivate,	  0, 0 },
 	{ "filter",		cmd_filter,	0, 1, NULL,		  0, 0 },
-	{ "fset",		cmd_fset,	1, 1, NULL,		  0, 0 },
+	{ "fset",		cmd_fset,	1, 1, expand_fset,	  0, 0 },
 	{ "help",		cmd_help,	0, 0, NULL,		  0, 0 },
 	{ "invert",		cmd_invert,	0, 0, NULL,		  0, 0 },
 	{ "live-filter",	cmd_live_filter,0, 1, NULL,		  0, CMD_LIVE },
@@ -2461,13 +2498,13 @@ struct command commands[] = {
 	{ "push",		cmd_push,	1,-1, expand_commands,	  0, 0 },
 	{ "quit",		cmd_quit,	0, 1, NULL,		  0, 0 },
 	{ "refresh",		cmd_refresh,	0, 0, NULL,		  0, 0 },
-	{ "run",		cmd_run,	1,-1, NULL,		  0, CMD_UNSAFE },
+	{ "run",		cmd_run,	1,-1, expand_program_paths, 0, CMD_UNSAFE },
 	{ "save",		cmd_save,	0, 1, expand_load_save,	  0, CMD_UNSAFE },
 	{ "search-next",	cmd_search_next,0, 0, NULL,		  0, 0 },
 	{ "search-prev",	cmd_search_prev,0, 0, NULL,		  0, 0 },
 	{ "seek",		cmd_seek,	1, 1, NULL,		  0, 0 },
 	{ "set",		cmd_set,	1, 1, expand_options,	  0, 0 },
-	{ "shell",		cmd_shell,	1,-1, NULL,		  0, CMD_UNSAFE },
+	{ "shell",		cmd_shell,	1,-1, expand_program_paths, 0, CMD_UNSAFE },
 	{ "showbind",		cmd_showbind,	1, 1, expand_unbind_args, 0, 0 },
 	{ "shuffle",		cmd_reshuffle,	0, 0, NULL,		  0, 0 },
 	{ "source",		cmd_source,	1, 1, expand_files,	  0, CMD_UNSAFE },
@@ -2475,7 +2512,7 @@ struct command commands[] = {
 	{ "tqueue",		cmd_tqueue,	0, 1, NULL,		  0, 0 },
 	{ "unbind",		cmd_unbind,	1, 1, expand_unbind_args, 0, 0 },
 	{ "unmark",		cmd_unmark,	0, 0, NULL,		  0, 0 },
-	{ "update-cache",	cmd_update_cache,0, 0, NULL,		  0, 0 },
+	{ "update-cache",	cmd_update_cache,0, 1, NULL,		  0, 0 },
 	{ "view",		cmd_view,	1, 1, NULL,		  0, 0 },
 	{ "vol",		cmd_vol,	1, 2, NULL,		  0, 0 },
 	{ "w",			cmd_save,	0, 1, expand_load_save,	  0, CMD_UNSAFE },
@@ -2489,7 +2526,10 @@ struct command commands[] = {
 	{ "win-mv-after",	cmd_win_mv_after,0, 0, NULL,		  0, 0 },
 	{ "win-mv-before",	cmd_win_mv_before,0, 0, NULL,		  0, 0 },
 	{ "win-next",		cmd_win_next,	0, 0, NULL,		  0, 0 },
+	{ "win-page-bottom",	cmd_win_pg_bottom,0, 0, NULL,		  0, 0 },
 	{ "win-page-down",	cmd_win_pg_down,0, 0, NULL,		  0, 0 },
+	{ "win-page-middle",	cmd_win_pg_middle,0, 0, NULL,		  0, 0 },
+	{ "win-page-top",	cmd_win_pg_top,	0, 0, NULL,		  0, 0 },
 	{ "win-page-up",	cmd_win_pg_up,	0, 0, NULL,		  0, 0 },
 	{ "win-remove",		cmd_win_remove,	0, 0, NULL,		  0, CMD_UNSAFE },
 	{ "win-sel-cur",	cmd_win_sel_cur,0, 0, NULL,		  0, 0 },
@@ -2497,6 +2537,7 @@ struct command commands[] = {
 	{ "win-top",		cmd_win_top,	0, 0, NULL,		  0, 0 },
 	{ "win-up",		cmd_win_up,	0, 0, NULL,		  0, 0 },
 	{ "win-update",		cmd_win_update,	0, 0, NULL,		  0, 0 },
+	{ "win-update-cache",	cmd_win_update_cache,0, 1, NULL,	  0, 0 },
 	{ "wq",			cmd_quit,	0, 1, NULL,		  0, 0 },
 	{ NULL,			NULL,		0, 0, 0,		  0, 0 }
 };
@@ -2744,9 +2785,11 @@ void run_parsed_command(char *cmd, char *arg)
 				error_msg("too many arguments\n");
 				break;
 			}
-			if (run_only_safe_commands && c->flags & CMD_UNSAFE) {
-				d_print("trying to execute unsafe command over net\n");
-				break;
+			if (run_only_safe_commands && (c->flags & CMD_UNSAFE)) {
+				if (c->func != cmd_save || !is_stdout_filename(arg)) {
+					d_print("trying to execute unsafe command over net\n");
+					break;
+				}
 			}
 			c->func(arg);
 			break;
