@@ -1,5 +1,19 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2006 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "editable.h"
@@ -35,7 +49,7 @@ void editable_init(struct editable *e, void (*free_track)(struct list_head *item
 	struct iter iter;
 
 	list_init(&e->head);
-	e->tree_root.rb_node = NULL;
+	e->tree_root = RB_ROOT;
 	e->nr_tracks = 0;
 	e->nr_marked = 0;
 	e->total_time = 0;
@@ -53,15 +67,25 @@ void editable_init(struct editable *e, void (*free_track)(struct list_head *item
 	e->searchable = searchable_new(e->win, &iter, &simple_search_ops);
 }
 
-void editable_add(struct editable *e, struct simple_track *track)
+static void do_editable_add(struct editable *e, struct simple_track *track, int tiebreak)
 {
-	sorted_list_add_track(&e->head, &e->tree_root, track, e->sort_keys);
+	sorted_list_add_track(&e->head, &e->tree_root, track, e->sort_keys, tiebreak);
 	e->nr_tracks++;
 	if (track->info->duration != -1)
 		e->total_time += track->info->duration;
 	window_changed(e->win);
 }
 
+void editable_add(struct editable *e, struct simple_track *track)
+{
+	do_editable_add(e, track, +1);
+}
+
+void editable_add_before(struct editable *e, struct simple_track *track)
+{
+	do_editable_add(e, track, -1);
+}
+
 void editable_remove_track(struct editable *e, struct simple_track *track)
 {
 	struct track_info *ti = track->info;
@@ -75,10 +99,7 @@ void editable_remove_track(struct editable *e, struct simple_track *track)
 	if (ti->duration != -1)
 		e->total_time -= ti->duration;
 
-	list_del(&track->node);
-	if (!RB_EMPTY_NODE(&track->tree_node))
-		rb_erase(&track->tree_node, &e->tree_root);
-
+	sorted_list_remove_track(&e->head, &e->tree_root, track);
 	e->free_track(&track->node);
 }
 
@@ -106,16 +127,9 @@ void editable_remove_sel(struct editable *e)
 
 void editable_sort(struct editable *e)
 {
-	struct rb_node *node, *tmp;
-	struct rb_root tmptree = RB_ROOT;
-
-	list_init(&e->head);
-	rb_for_each_safe(node, tmp, &e->tree_root) {
-		struct simple_track *track = tree_node_to_simple_track(node);
-		rb_erase(node, &e->tree_root);
-		sorted_list_add_track(&e->head, &tmptree, track, e->sort_keys);
-	}
-	e->tree_root.rb_node = tmptree.rb_node;
+	if (e->nr_tracks <= 1)
+		return;
+	sorted_list_rebuild(&e->head, &e->tree_root, e->sort_keys);
 
 	window_changed(e->win);
 	window_goto_top(e->win);
@@ -154,6 +168,18 @@ static void move_item(struct editable *e, struct list_head *head, struct list_he
 	list_add(item, head);
 }
 
+static void reset_tree(struct editable *e)
+{
+	struct simple_track *old, *first_track;
+
+	old = tree_node_to_simple_track(rb_first(&e->tree_root));
+	first_track = to_simple_track(e->head.next);
+	if (old != first_track) {
+		rb_replace_node(&old->tree_node, &first_track->tree_node, &e->tree_root);
+		RB_CLEAR_NODE(&old->tree_node);
+	}
+}
+
 static void move_sel(struct editable *e, struct list_head *after)
 {
 	struct simple_track *t;
@@ -184,6 +210,7 @@ static void move_sel(struct editable *e, struct list_head *after)
 		list_add(item, after);
 		item = next;
 	}
+	reset_tree(e);
 
 	/* select top-most of the moved tracks */
 	editable_track_to_iter(e, to_simple_track(after->next), &iter);
@@ -262,14 +289,10 @@ void editable_move_before(struct editable *e)
 
 void editable_clear(struct editable *e)
 {
-	struct list_head *item, *next;
+	struct list_head *item, *tmp;
 
-	item = e->head.next;
-	while (item != &e->head) {
-		next = item->next;
+	list_for_each_safe(item, tmp, &e->head)
 		editable_remove_track(e, to_simple_track(item));
-		item = next;
-	}
 }
 
 void editable_remove_matching_tracks(struct editable *e,
@@ -356,3 +379,25 @@ int editable_for_each_sel(struct editable *e, int (*cb)(void *data, struct track
 		window_down(e->win, 1);
 	return rc;
 }
+
+void editable_update_track(struct editable *e, struct track_info *old, struct track_info *new)
+{
+	struct list_head *item, *tmp;
+	int changed = 0;
+
+	list_for_each_safe(item, tmp, &e->head) {
+		struct simple_track *track = to_simple_track(item);
+		if (track->info == old) {
+			if (new) {
+				track_info_unref(old);
+				track_info_ref(new);
+				track->info = new;
+			} else {
+				editable_remove_track(e, track);
+			}
+			changed = 1;
+		}
+	}
+	if (changed)
+		e->win->changed = changed;
+}
