@@ -1,5 +1,19 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2005 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "expr.h"
@@ -388,6 +402,7 @@ static const struct {
 	char short_key;
 	const char *long_key;
 } map_short2long[] = {
+	{ 'A',	"albumartist"	},
 	{ 'D',	"discnumber"	},
 	{ 'T',	"tag",		},
 	{ 'a',	"artist"	},
@@ -408,15 +423,19 @@ static const struct {
 	enum expr_type type;
 } builtin[] = {
 	{ "album",	EXPR_STR	},
+	{ "albumartist",EXPR_STR	},
 	{ "artist",	EXPR_STR	},
 	{ "bitrate",	EXPR_INT	},
 	{ "codec",	EXPR_STR	},
+	{ "codec_profile",EXPR_STR	},
 	{ "comment",	EXPR_STR	},
 	{ "date",	EXPR_INT	},
 	{ "discnumber",	EXPR_INT	},
 	{ "duration",	EXPR_INT	},
 	{ "filename",	EXPR_STR	},
 	{ "genre",	EXPR_STR	},
+	{ "media",	EXPR_STR	},
+	{ "originaldate",EXPR_INT	},
 	{ "stream",	EXPR_BOOL	},
 	{ "tag",	EXPR_BOOL	},
 	{ "title",	EXPR_STR	},
@@ -815,7 +834,7 @@ unsigned int expr_get_match_type(struct expr *expr)
 	}
 
 	key = expr->key;
-	if (strcmp(key, "artist") == 0)
+	if (strcmp(key, "artist") == 0 || strcmp(key, "albumartist") == 0)
 		return TI_MATCH_ARTIST;
 	if (strcmp(key, "album") == 0 || strcmp(key, "discnumber") == 0)
 		return TI_MATCH_ALBUM;
@@ -877,6 +896,8 @@ int expr_eval(struct expr *expr, struct track_info *ti)
 				val = uval;
 		} else if (strcmp(key, "codec") == 0) {
 			val = ti->codec;
+		} else if (strcmp(key, "codec_profile") == 0) {
+			val = ti->codec_profile;
 		} else {
 			val = keyvals_get_val(ti->comments, key);
 		}
@@ -894,10 +915,12 @@ int expr_eval(struct expr *expr, struct track_info *ti)
 		if (strcmp(key, "duration") == 0) {
 			val = ti->duration;
 			/* duration of a stream is infinite (well, almost) */
-			if (is_url(ti->filename))
+			if (is_http_url(ti->filename))
 				val = INT_MAX;
 		} else if (strcmp(key, "date") == 0) {
 			val = (ti->date >= 0) ? (ti->date / 10000) : -1;
+		} else if (strcmp(key, "originaldate") == 0) {
+			val = (ti->originaldate >= 0) ? (ti->originaldate / 10000) : -1;
 		} else if (strcmp(key, "bitrate") == 0) {
 			val = (ti->bitrate >= 0) ? (int) (ti->bitrate / 1000. + 0.5) : -1;
 		} else {
@@ -934,7 +957,7 @@ int expr_eval(struct expr *expr, struct track_info *ti)
 		}
 	}
 	if (strcmp(key, "stream") == 0)
-		return is_url(ti->filename);
+		return is_http_url(ti->filename);
 	return track_info_has_tag(ti);
 }
 
