@@ -1,4 +1,5 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2007 Kevin Ko <kevin.s.ko@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -12,16 +13,17 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "ip.h"
 #include "xmalloc.h"
 #include "debug.h"
 #include "utils.h"
+#include "comment.h"
+#ifdef HAVE_CONFIG
 #include "config/ffmpeg.h"
+#endif
 
 #include <stdio.h>
 #ifdef HAVE_FFMPEG_AVCODEC_H
@@ -65,6 +67,9 @@ struct ffmpeg_input {
 	AVPacket pkt;
 	int curr_pkt_size;
 	uint8_t *curr_pkt_buf;
+
+	unsigned long curr_size;
+	unsigned long curr_duration;
 };
 
 struct ffmpeg_output {
@@ -173,13 +178,14 @@ static int ffmpeg_open(struct input_plugin_data *ip_data)
 	int err = 0;
 	int i;
 	int stream_index = -1;
+	int64_t channel_layout = 0;
 	AVCodec *codec;
 	AVCodecContext *cc = NULL;
 	AVFormatContext *ic = NULL;
 
 	ffmpeg_init();
 
-#if (LIBAVFORMAT_VERSION_INT < ((53<<16)+(2<<8)+0))
+#if (LIBAVFORMAT_VERSION_INT <= ((53<<16)+(2<<8)+0))
 	err = av_open_input_file(&ic, ip_data->filename, NULL, 0, NULL);
 #else
 	err = avformat_open_input(&ic, ip_data->filename, NULL, NULL);
@@ -190,7 +196,7 @@ static int ffmpeg_open(struct input_plugin_data *ip_data)
 	}
 
 	do {
-#if (LIBAVFORMAT_VERSION_INT < ((53<<16)+(5<<8)+0))
+#if (LIBAVFORMAT_VERSION_INT <= ((53<<16)+(5<<8)+0))
 		err = av_find_stream_info(ic);
 #else
 		err = avformat_find_stream_info(ic, NULL);
@@ -283,6 +289,10 @@ static int ffmpeg_open(struct input_plugin_data *ip_data)
 #ifdef WORDS_BIGENDIAN
 	ip_data->sf |= sf_bigendian(1);
 #endif
+#if (LIBAVCODEC_VERSION_INT > ((52<<16)+(1<<8)+0))
+	channel_layout = cc->channel_layout;
+#endif
+	channel_map_init_waveex(cc->channels, channel_layout, ip_data->channel_map);
 	return 0;
 }
 
@@ -320,6 +330,8 @@ static int ffmpeg_fill_buffer(AVFormatContext *ic, AVCodecContext *cc, struct ff
 			}
 			input->curr_pkt_size = input->pkt.size;
 			input->curr_pkt_buf = input->pkt.data;
+			input->curr_size += input->pkt.size;
+			input->curr_duration += input->pkt.duration;
 			continue;
 		}
 
@@ -430,7 +442,7 @@ static int ffmpeg_read_comments(struct input_plugin_data *ip_data, struct keyval
 	char buff[16];
 	int i = 0;
 
-	*comments = xnew0(struct keyval, NUM_FFMPEG_KEYS + 1);
+	*comments = keyvals_new(NUM_FFMPEG_KEYS);
 
 	i = set_comment(*comments, i, "artist", ic->author);
 	i = set_comment(*comments, i, "album", ic->album);
@@ -475,12 +487,60 @@ static long ffmpeg_bitrate(struct input_plugin_data *ip_data)
 	return bitrate ? bitrate : -IP_ERROR_FUNCTION_NOT_SUPPORTED;
 }
 
+static long ffmpeg_current_bitrate(struct input_plugin_data *ip_data)
+{
+	struct ffmpeg_private *priv = ip_data->private;
+	AVStream *st = priv->input_context->streams[priv->stream_index];
+	long bitrate = -1;
+#if (LIBAVFORMAT_VERSION_INT > ((51<<16)+(43<<8)+0))
+	/* ape codec returns silly numbers */
+	if (priv->codec->id == CODEC_ID_APE)
+		return -1;
+#endif
+	if (priv->input->curr_duration > 0) {
+		double seconds = priv->input->curr_duration * av_q2d(st->time_base);
+		bitrate = (8 * priv->input->curr_size) / seconds;
+		priv->input->curr_size = 0;
+		priv->input->curr_duration = 0;
+	}
+	return bitrate;
+}
+
 static char *ffmpeg_codec(struct input_plugin_data *ip_data)
 {
 	struct ffmpeg_private *priv = ip_data->private;
 	return xstrdup(priv->codec->name);
 }
 
+#if (LIBAVCODEC_VERSION_INT < ((52<<16)+(104<<8)+0))
+static const char *codec_profile_to_str(int profile)
+{
+#if (LIBAVCODEC_VERSION_INT >= ((51<<16)+(41<<8)+0))
+	switch (profile) {
+	case FF_PROFILE_AAC_MAIN:	return "Main";
+	case FF_PROFILE_AAC_LOW:	return "LC";
+	case FF_PROFILE_AAC_SSR:	return "SSR";
+	case FF_PROFILE_AAC_LTP:	return "LTP";
+	}
+#endif
+	return NULL;
+}
+#endif
+
+static char *ffmpeg_codec_profile(struct input_plugin_data *ip_data)
+{
+	struct ffmpeg_private *priv = ip_data->private;
+	const char *profile;
+
+#if (LIBAVCODEC_VERSION_INT < ((52<<16)+(104<<8)+0))
+	profile = codec_profile_to_str(priv->codec_context->profile);
+#else
+	profile = av_get_profile_name(priv->codec, priv->codec_context->profile);
+#endif
+
+	return profile ? xstrdup(profile) : NULL;
+}
+
 const struct input_plugin_ops ip_ops = {
 	.open = ffmpeg_open,
 	.close = ffmpeg_close,
@@ -489,19 +549,21 @@ const struct input_plugin_ops ip_ops = {
 	.read_comments = ffmpeg_read_comments,
 	.duration = ffmpeg_duration,
 	.bitrate = ffmpeg_bitrate,
-	.codec = ffmpeg_codec
+	.bitrate_current = ffmpeg_current_bitrate,
+	.codec = ffmpeg_codec,
+	.codec_profile = ffmpeg_codec_profile
 };
 
 const int ip_priority = 30;
-#ifdef USE_FALLBACK_IP
-const char *const ip_extensions[] = { "any", NULL };
-#else
 const char *const ip_extensions[] = {
 	"ac3", "aif", "aifc", "aiff", "ape", "au", "mka", "shn", "tta", "wma",
 	/* also supported by other plugins */
 	"aac", "fla", "flac", "m4a", "m4b", "mp+", "mp2", "mp3", "mp4", "mpc",
 	"mpp", "ogg", "wav", "wv",
+#ifdef USE_FALLBACK_IP
+	"*",
+#endif
 	NULL
 };
-#endif
 const char *const ip_mime_types[] = { NULL };
+const char * const ip_options[] = { NULL };
