@@ -1,3 +1,21 @@
+/*
+ * Copyright 2008-2011 Various Authors
+ * Copyright 2005 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "ip.h"
 #include "comment.h"
 #include "xmalloc.h"
@@ -18,7 +36,7 @@
 #endif
 
 #include <FLAC/metadata.h>
-#include <inttypes.h>
+#include <stdint.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <errno.h>
@@ -57,6 +75,7 @@ struct flac_private {
 
 	struct keyval *comments;
 	double duration;
+	long bitrate;
 
 	unsigned int ignore_next_write : 1;
 };
@@ -302,8 +321,11 @@ static void metadata_cb(const Dec *dec, const FLAC__StreamMetadata *metadata, vo
 				sf_bits(bits) |
 				sf_signed(1) |
 				sf_channels(si->channels);
-			if (!ip_data->remote && si->total_samples)
+			if (!ip_data->remote && si->total_samples) {
 				priv->duration = (double) si->total_samples / si->sample_rate;
+				if (priv->duration >= 1 && priv->len >= 1)
+					priv->bitrate = priv->len * 8 / priv->duration;
+			}
 		}
 		break;
 	case FLAC__METADATA_TYPE_VORBIS_COMMENT:
@@ -357,18 +379,34 @@ static void free_priv(struct input_plugin_data *ip_data)
 	errno = save;
 }
 
+/* http://flac.sourceforge.net/format.html#frame_header */
+static void channel_map_init_flac(int channels, channel_position_t *map)
+{
+	unsigned int mask = 0;
+	if (channels == 4)
+		mask = 0x33; // 0b110011, without center and lfe
+	else if (channels == 5)
+		mask = 0x37; // 0b110111, without lfe
+	channel_map_init_waveex(channels, mask, map);
+}
+
 static int flac_open(struct input_plugin_data *ip_data)
 {
 	struct flac_private *priv;
-	Dec *dec;
 
-	dec = F(new)();
-	if (dec == NULL)
+	Dec *dec = F(new)();
+
+	const struct flac_private priv_init = {
+		.dec      = dec,
+		.duration = -1,
+		.bitrate  = -1
+	};
+
+	if (!dec)
 		return -IP_ERROR_INTERNAL;
 
-	priv = xnew0(struct flac_private, 1);
-	priv->dec = dec;
-	priv->duration = -1;
+	priv = xnew(struct flac_private, 1);
+	*priv = priv_init;
 	if (ip_data->remote) {
 		priv->len = UINT64_MAX;
 	} else {
@@ -436,6 +474,7 @@ static int flac_open(struct input_plugin_data *ip_data)
 		return -IP_ERROR_SAMPLE_FORMAT;
 	}
 
+	channel_map_init_flac(sf_get_channels(ip_data->sf), ip_data->channel_map);
 	d_print("sr: %d, ch: %d, bits: %d\n",
 			sf_get_rate(ip_data->sf),
 			sf_get_channels(ip_data->sf),
@@ -504,7 +543,7 @@ static int flac_read_comments(struct input_plugin_data *ip_data, struct keyval *
 	if (priv->comments) {
 		*comments = keyvals_dup(priv->comments);
 	} else {
-		*comments = xnew0(struct keyval, 1);
+		*comments = keyvals_new(0);
 	}
 	return 0;
 }
@@ -519,12 +558,7 @@ static int flac_duration(struct input_plugin_data *ip_data)
 static long flac_bitrate(struct input_plugin_data *ip_data)
 {
 	struct flac_private *priv = ip_data->private;
-	off_t file_size = lseek(ip_data->fd, 0, SEEK_END);
-	if (file_size == -1)
-		return -IP_ERROR_FUNCTION_NOT_SUPPORTED;
-	if (priv->duration < 1)
-		return -1;
-	return file_size * 8 / priv->duration;
+	return priv->bitrate;
 }
 
 static char *flac_codec(struct input_plugin_data *ip_data)
@@ -532,6 +566,12 @@ static char *flac_codec(struct input_plugin_data *ip_data)
 	return xstrdup("flac");
 }
 
+static char *flac_codec_profile(struct input_plugin_data *ip_data)
+{
+	/* maybe identify compression-level over min/max blocksize/framesize */
+	return NULL;
+}
+
 const struct input_plugin_ops ip_ops = {
 	.open = flac_open,
 	.close = flac_close,
@@ -540,9 +580,12 @@ const struct input_plugin_ops ip_ops = {
 	.read_comments = flac_read_comments,
 	.duration = flac_duration,
 	.bitrate = flac_bitrate,
-	.codec = flac_codec
+	.bitrate_current = flac_bitrate,
+	.codec = flac_codec,
+	.codec_profile = flac_codec_profile
 };
 
 const int ip_priority = 50;
 const char * const ip_extensions[] = { "flac", "fla", NULL };
 const char * const ip_mime_types[] = { NULL };
+const char * const ip_options[] = { NULL };
