@@ -1,6 +1,21 @@
 /*
- * help.h, heavily based on filters.h
- *  (c) 2006, <ft@bewatermyfriend.org>
+ * Copyright 2008-2011 Various Authors
+ * Copyright 2006 <ft@bewatermyfriend.org>
+ *
+ * heavily based on filters.h
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef HELP_H
@@ -9,7 +24,6 @@
 #include "list.h"
 #include "window.h"
 #include "search.h"
-#include "uchar.h"
 #include "keys.h"
 
 struct help_entry {
