@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "history.h"
@@ -22,10 +21,12 @@
 #include "file.h"
 #include "uchar.h"
 #include "list.h"
+#include "prog.h"
 
 #include <sys/types.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <stdio.h>
 
 struct history_entry {
 	struct list_head node;
@@ -82,16 +83,18 @@ void history_load(struct history *history, char *filename, int max_lines)
 
 void history_save(struct history *history)
 {
+	char filename_tmp[512];
 	struct list_head *item;
 	int fd;
+	ssize_t rc;
 
-	fd = open(history->filename, O_CREAT | O_WRONLY | O_TRUNC, 0666);
+	snprintf(filename_tmp, sizeof(filename_tmp), "%s.tmp", history->filename);
+	fd = open(filename_tmp, O_CREAT | O_WRONLY | O_TRUNC, 0666);
 	if (fd == -1)
 		return;
 	list_for_each(item, &history->head) {
 		struct history_entry *history_entry;
 		const char nl = '\n';
-		ssize_t rc;
 
 		history_entry = list_entry(item, struct history_entry, node);
 
@@ -105,6 +108,10 @@ void history_save(struct history *history)
 	}
 out:
 	close(fd);
+
+	rc = rename(filename_tmp, history->filename);
+	if (rc)
+		warn_errno("renaming %s to %s", filename_tmp, history->filename);
 }
 
 void history_add_line(struct history *history, const char *line)
