@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "http.h"
@@ -23,11 +22,13 @@
 #include "xmalloc.h"
 #include "gbuf.h"
 
+#include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/socket.h>
+#include <sys/select.h>
 #include <netinet/in.h>
 #include <netdb.h>
 #include <arpa/inet.h>
@@ -133,13 +134,18 @@ void http_free_uri(struct http_uri *u)
 
 int http_open(struct http_get *hg, int timeout_ms)
 {
-	struct hostent *hostent;
+	const struct addrinfo hints = {
+		.ai_socktype = SOCK_STREAM
+	};
+	struct addrinfo *result;
 	union {
 		struct sockaddr sa;
-		struct sockaddr_in in;
+		struct sockaddr_storage sas;
 	} addr;
+	size_t addrlen;
 	struct timeval tv;
-	int save, flags;
+	int save, flags, rc;
+	char port[16];
 
 	char *proxy = getenv("http_proxy");
 	if (proxy) {
@@ -152,15 +158,17 @@ int http_open(struct http_get *hg, int timeout_ms)
 		hg->proxy = NULL;
 	}
 
-	hostent = gethostbyname(hg->proxy ? hg->proxy->host : hg->uri.host);
-	if (hostent == NULL)
+	snprintf(port, sizeof(port), "%d", hg->proxy ? hg->proxy->port : hg->uri.port);
+	rc = getaddrinfo(hg->proxy ? hg->proxy->host : hg->uri.host, port, &hints, &result);
+	if (rc != 0) {
+		d_print("getaddrinfo: %s\n", gai_strerror(rc));
 		return -1;
+	}
+	memcpy(&addr.sa, result->ai_addr, result->ai_addrlen);
+	addrlen = result->ai_addrlen;
+	freeaddrinfo(result);
 
-	addr.in.sin_family = AF_INET;
-	addr.in.sin_port = htons(hg->proxy ? hg->proxy->port : hg->uri.port);
-	memcpy(&addr.in.sin_addr, hostent->h_addr_list[0], hostent->h_length);
-
-	hg->fd = socket(PF_INET, SOCK_STREAM, 0);
+	hg->fd = socket(addr.sa.sa_family, SOCK_STREAM, 0);
 	if (hg->fd == -1)
 		return -1;
 
@@ -174,7 +182,7 @@ int http_open(struct http_get *hg, int timeout_ms)
 		fd_set wfds;
 
 		d_print("connecting. timeout=%lld s %lld us\n", (long long)tv.tv_sec, (long long)tv.tv_usec);
-		if (connect(hg->fd, &addr.sa, sizeof(addr.in)) == 0)
+		if (connect(hg->fd, &addr.sa, addrlen) == 0)
 			break;
 		if (errno == EISCONN)
 			break;
@@ -184,8 +192,6 @@ int http_open(struct http_get *hg, int timeout_ms)
 		FD_ZERO(&wfds);
 		FD_SET(hg->fd, &wfds);
 		while (1) {
-			int rc;
-
 			rc = select(hg->fd + 1, NULL, &wfds, NULL, &tv);
 			if (rc == -1) {
 				if (errno != EINTR)
