@@ -1,9 +1,22 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2005 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "id3.h"
-#include "comment.h"
 #include "xmalloc.h"
 #include "convert.h"
 #include "uchar.h"
@@ -13,7 +26,7 @@
 #include "file.h"
 
 #include <unistd.h>
-#include <inttypes.h>
+#include <stdint.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
@@ -228,18 +241,29 @@ const char * const id3_key_names[NUM_ID3_KEYS] = {
 	"album",
 	"title",
 	"date",
+	"originaldate",
 	"genre",
 	"discnumber",
 	"tracknumber",
 	"albumartist",
 	"artistsort",
 	"albumartistsort",
+	"albumsort",
 	"compilation",
 	"replaygain_track_gain",
 	"replaygain_track_peak",
 	"replaygain_album_gain",
 	"replaygain_album_peak",
+	"composer",
+	"conductor",
+	"lyricist",
+	"remixer",
+	"label",
+	"publisher",
+	"subtitle",
 	"comment",
+	"musicbrainz_trackid",
+	"media",
 };
 
 static int utf16_is_lsurrogate(uchar uch)
@@ -522,8 +546,9 @@ static struct {
 	/* 2.4.0 */
 	{ "TDRC", ID3_DATE }, // recording date
 	{ "TDRL", ID3_DATE }, // release date
-	{ "TDOR", ID3_DATE }, // original release date
+	{ "TDOR", ID3_ORIGINALDATE }, // original release date
 	{ "TSOP", ID3_ARTISTSORT },
+	{ "TSOA", ID3_ALBUMSORT },
 
 	/* >= 2.3.0 */
 	{ "TPE1", ID3_ARTIST },
@@ -536,7 +561,16 @@ static struct {
 	{ "TPE2", ID3_ALBUMARTIST },
 	{ "TSO2", ID3_ALBUMARTISTSORT },
 	{ "XSOP", ID3_ARTISTSORT }, // obsolete
+	{ "XSOA", ID3_ALBUMSORT }, // obsolete
 	{ "TCMP", ID3_COMPILATION },
+	{ "TORY", ID3_ORIGINALDATE },
+	{ "TCOM", ID3_COMPOSER },
+	{ "TPE3", ID3_CONDUCTOR },
+	{ "TEXT", ID3_LYRICIST },
+	{ "TPE4", ID3_REMIXER },
+	{ "TPUB", ID3_PUBLISHER }, // TPUB can be both publisher or label
+	{ "TIT3", ID3_SUBTITLE },
+	{ "TMED", ID3_MEDIA },
 
 	/* obsolete frames (2.2.0) */
 	{ "TP1",  ID3_ARTIST },
@@ -549,6 +583,7 @@ static struct {
 	{ "TRK",  ID3_TRACK },
 	{ "TSP",  ID3_ARTISTSORT },
 	{ "TS2",  ID3_ALBUMARTISTSORT },
+	{ "TSA",  ID3_ALBUMSORT },
 	{ "TCP",  ID3_COMPILATION },
 
 	{ "", -1 }
@@ -675,7 +710,7 @@ static void decode_normal(struct id3tag *id3, const char *buf, int len, int enco
 		tmp = parse_genre(out);
 		free(out);
 		out = tmp;
-	} else if (key == ID3_DATE) {
+	} else if (key == ID3_DATE || key == ID3_ORIGINALDATE) {
 		int date_len = check_date_format(out);
 		id3_debug("date before: '%s'\n", out);
 		if (date_len)
@@ -694,6 +729,8 @@ static void decode_normal(struct id3tag *id3, const char *buf, int len, int enco
 		 */
 		if (id3->v2[key])
 			return;
+	} else if (key == ID3_PUBLISHER) {
+		 add_v2(id3, ID3_LABEL, strdup(out));
 	}
 
 	add_v2(id3, key, out);
@@ -750,6 +787,8 @@ static void decode_txxx(struct id3tag *id3, const char *buf, int len, int encodi
 		key = ID3_ALBUMARTIST;
 	else if (!strcasecmp(out, "albumartistsort"))
 		key = ID3_ALBUMARTISTSORT;
+	else if (!strcasecmp(out, "albumsort"))
+		key = ID3_ALBUMSORT;
 	else if (!strcasecmp(out, "compilation"))
 		key = ID3_COMPILATION;
 
@@ -920,6 +959,22 @@ static void decode_rva2(struct id3tag *id3, const char *buf, int len)
 	id3_debug("gain %s, peak %s\n", gain_str, peak_str ? peak_str : "none");
 }
 
+static void decode_ufid(struct id3tag *id3, const char *buf, int len)
+{
+	char *ufid;
+	int ufid_len = len - 22 - 1;
+
+	if (ufid_len < 0 || strcmp(buf, "http://musicbrainz.org") != 0)
+		return;
+
+	ufid = xnew(char, ufid_len + 1);
+	memcpy(ufid, buf + len - ufid_len, ufid_len);
+	ufid[ufid_len] = '\0';
+
+	id3_debug("%s: %s\n", buf, ufid);
+	add_v2(id3, ID3_MUSICBRAINZ_TRACKID, ufid);
+}
+
 
 static void v2_add_frame(struct id3tag *id3, struct v2_frame_header *fh, const char *buf)
 {
@@ -930,6 +985,9 @@ static void v2_add_frame(struct id3tag *id3, struct v2_frame_header *fh, const c
 	if (!strncmp(fh->id, "RVA2", 4)) {
 		decode_rva2(id3, buf, fh->size);
 		return;
+	} else if (!strncmp(fh->id, "UFID", 4)) {
+		decode_ufid(id3, buf, fh->size);
+		return;
 	}
 
 	encoding = *buf++;
@@ -1081,7 +1139,8 @@ int id3_tag_size(const char *buf, int buf_size)
 
 void id3_init(struct id3tag *id3)
 {
-	memset(id3, 0, sizeof(*id3));
+	const struct id3tag t = { .has_v1 = 0, .has_v2 = 0 };
+	*id3 = t;
 }
 
 void id3_free(struct id3tag *id3)
