@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,27 +11,30 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "convert.h"
 #include "input.h"
 #include "ip.h"
 #include "pcm.h"
 #include "http.h"
 #include "xmalloc.h"
 #include "file.h"
+#include "path.h"
 #include "utils.h"
 #include "cmus.h"
+#include "options.h"
 #include "list.h"
 #include "mergesort.h"
 #include "misc.h"
 #include "debug.h"
 #include "ui_curses.h"
+#ifdef HAVE_CONFIG
 #include "config/libdir.h"
+#endif
 
 #include <unistd.h>
 #include <string.h>
@@ -60,6 +64,8 @@ struct input_plugin {
 	long bitrate;
 	/* cached codec, NULL = unset */
 	char *codec;
+	/* cached codec_profile, NULL = unset */
+	char *codec_profile;
 
 	/*
 	 * pcm is converted to 16-bit signed little-endian stereo
@@ -84,6 +90,7 @@ struct ip {
 	const char * const *extensions;
 	const char * const *mime_types;
 	const struct input_plugin_ops *ops;
+	const char * const *options;
 };
 
 static const char * const plugin_dir = LIBDIR "/cmus/ip";
@@ -99,21 +106,6 @@ static const char *pl_mime_types[] = {
 	"audio/x-mpegurl"
 };
 
-static const char *get_extension(const char *filename)
-{
-	const char *ext;
-
-	ext = filename + strlen(filename) - 1;
-	while (ext >= filename && *ext != '/') {
-		if (*ext == '.') {
-			ext++;
-			return ext;
-		}
-		ext--;
-	}
-	return NULL;
-}
-
 static const struct input_plugin_ops *get_ops_by_extension(const char *ext, struct list_head **headp)
 {
 	struct list_head *node = *headp;
@@ -124,7 +116,7 @@ static const struct input_plugin_ops *get_ops_by_extension(const char *ext, stru
 		int i;
 
 		for (i = 0; exts[i]; i++) {
-			if (strcasecmp(ext, exts[i]) == 0 || strcmp("any", exts[i]) == 0) {
+			if (strcasecmp(ext, exts[i]) == 0 || strcmp("*", exts[i]) == 0) {
 				*headp = node;
 				return ip->ops;
 			}
@@ -223,15 +215,17 @@ static int do_http_get(struct http_get *hg, const char *uri, int redirections)
 		val = keyvals_get_val(hg->headers, "location");
 		if (!val)
 			return -IP_ERROR_HTTP_RESPONSE;
-		redirloc = xstrdup(val);
-		http_get_free(hg);
-		close(hg->fd);
 
 		redirections++;
 		if (redirections > 2)
 			return -IP_ERROR_HTTP_REDIRECT_LIMIT;
 
+		redirloc = xstrdup(val);
+		http_get_free(hg);
+		close(hg->fd);
+
 		rc = do_http_get(hg, redirloc, redirections);
+
 		free(redirloc);
 		return rc;
 	default:
@@ -279,15 +273,15 @@ static int setup_remote(struct input_plugin *ip, const struct keyval *headers, i
 
 	val = keyvals_get_val(headers, "icy-name");
 	if (val)
-		ip->data.icy_name = xstrdup(val);
+		ip->data.icy_name = to_utf8(val, icecast_default_charset);
 
 	val = keyvals_get_val(headers, "icy-genre");
 	if (val)
-		ip->data.icy_genre = xstrdup(val);
+		ip->data.icy_genre = to_utf8(val, icecast_default_charset);
 
 	val = keyvals_get_val(headers, "icy-url");
 	if (val)
-		ip->data.icy_url = xstrdup(val);
+		ip->data.icy_url = to_utf8(val, icecast_default_charset);
 
 	return 0;
 }
@@ -377,15 +371,19 @@ static int open_remote(struct input_plugin *ip)
 
 static void ip_init(struct input_plugin *ip, char *filename)
 {
-	memset(ip, 0, sizeof(*ip));
-	ip->http_code = -1;
-	ip->pcm_convert_scale = -1;
-	ip->duration = -1;
-	ip->bitrate = -1;
-	ip->codec = NULL;
-	ip->data.fd = -1;
-	ip->data.filename = filename;
-	ip->data.remote = is_url(filename);
+	const struct input_plugin t = {
+		.http_code          = -1,
+		.pcm_convert_scale  = -1,
+		.duration           = -1,
+		.bitrate            = -1,
+		.data = {
+			.fd         = -1,
+			.filename   = filename,
+			.remote     = is_http_url(filename),
+			.channel_map = CHANNEL_MAP_INIT
+		}
+	};
+	*ip = t;
 }
 
 static void ip_reset(struct input_plugin *ip, int close_fd)
@@ -485,7 +483,8 @@ void ip_load_plugins(void)
 		ip->extensions = dlsym(so, "ip_extensions");
 		ip->mime_types = dlsym(so, "ip_mime_types");
 		ip->ops = dlsym(so, "ip_ops");
-		if (!priority_ptr || !ip->extensions || !ip->mime_types || !ip->ops) {
+		ip->options = dlsym(so, "ip_options");
+		if (!priority_ptr || !ip->extensions || !ip->mime_types || !ip->ops || !ip->options) {
 			error_msg("%s: missing symbol", filename);
 			free(ip);
 			dlclose(so);
@@ -530,7 +529,14 @@ int ip_open(struct input_plugin *ip)
 		if (rc == 0)
 			rc = ip->ops->open(&ip->data);
 	} else {
-		rc = open_file(ip);
+		if (is_cdda_url(ip->data.filename)) {
+			ip->ops = get_ops_by_mime_type("x-content/audio-cdda");
+			rc = ip->ops ? ip->ops->open(&ip->data) : 1;
+		} else if (is_cue_url(ip->data.filename)) {
+			ip->ops = get_ops_by_mime_type("application/x-cue");
+			rc = ip->ops ? ip->ops->open(&ip->data) : 1;
+		} else
+			rc = open_file(ip);
 	}
 
 	if (rc) {
@@ -714,6 +720,11 @@ int ip_bitrate(struct input_plugin *ip)
 	return ip->bitrate;
 }
 
+int ip_current_bitrate(struct input_plugin *ip)
+{
+	return ip->ops->bitrate_current(&ip->data);
+}
+
 char *ip_codec(struct input_plugin *ip)
 {
 	if (ip->data.remote)
@@ -723,12 +734,27 @@ char *ip_codec(struct input_plugin *ip)
 	return ip->codec;
 }
 
+char *ip_codec_profile(struct input_plugin *ip)
+{
+	if (ip->data.remote)
+		return NULL;
+	if (!ip->codec_profile)
+		ip->codec_profile = ip->ops->codec_profile(&ip->data);
+	return ip->codec_profile;
+}
+
 sample_format_t ip_get_sf(struct input_plugin *ip)
 {
 	BUG_ON(!ip->open);
 	return ip->data.sf;
 }
 
+void ip_get_channel_map(struct input_plugin *ip, channel_position_t *channel_map)
+{
+	BUG_ON(!ip->open);
+	channel_map_copy(channel_map, ip->data.channel_map);
+}
+
 const char *ip_get_filename(struct input_plugin *ip)
 {
 	return ip->data.filename;
@@ -760,6 +786,64 @@ int ip_eof(struct input_plugin *ip)
 	return ip->eof;
 }
 
+static struct ip *find_plugin(int idx)
+{
+	struct ip *ip;
+
+	list_for_each_entry(ip, &ip_head, node) {
+		if (idx == 0)
+			return ip;
+		idx--;
+	}
+	return NULL;
+}
+
+static void option_error(int rc)
+{
+	char *msg = ip_get_error_msg(NULL, rc, "setting option");
+	error_msg("%s", msg);
+	free(msg);
+}
+
+static void set_ip_option(unsigned int id, const char *val)
+{
+	const struct ip *ip = find_plugin(id >> 16);
+	int rc;
+
+	rc = ip->ops->set_option(id & 0xffff, val);
+	if (rc)
+		option_error(rc);
+}
+
+static void get_ip_option(unsigned int id, char *buf)
+{
+	const struct ip *ip = find_plugin(id >> 16);
+	char *val = NULL;
+
+	ip->ops->get_option(id & 0xffff, &val);
+	if (val) {
+		strcpy(buf, val);
+		free(val);
+	}
+}
+
+void ip_add_options(void)
+{
+	struct ip *ip;
+	unsigned int iid, pid = 0;
+	char key[64];
+
+	list_for_each_entry(ip, &ip_head, node) {
+		for (iid = 0; ip->options[iid]; iid++) {
+			snprintf(key, sizeof(key), "input.%s.%s",
+					ip->name,
+					ip->options[iid]);
+			option_add(xstrdup(key), (pid << 16) | iid, get_ip_option, set_ip_option, NULL, 0);
+		}
+		pid++;
+	}
+}
+
 char *ip_get_error_msg(struct input_plugin *ip, int rc, const char *arg)
 {
 	char buffer[1024];
@@ -793,6 +877,12 @@ char *ip_get_error_msg(struct input_plugin *ip, int rc, const char *arg)
 				"%s: input plugin doesn't support the sample format",
 				arg);
 		break;
+	case IP_ERROR_WRONG_DISC:
+		snprintf(buffer, sizeof(buffer), "%s: wrong disc inserted, aborting!", arg);
+		break;
+	case IP_ERROR_NO_DISC:
+		snprintf(buffer, sizeof(buffer), "%s: could not read disc", arg);
+		break;
 	case IP_ERROR_HTTP_RESPONSE:
 		snprintf(buffer, sizeof(buffer), "%s: invalid HTTP response", arg);
 		break;
@@ -805,6 +895,10 @@ char *ip_get_error_msg(struct input_plugin *ip, int rc, const char *arg)
 	case IP_ERROR_HTTP_REDIRECT_LIMIT:
 		snprintf(buffer, sizeof(buffer), "%s: too many HTTP redirections", arg);
 		break;
+	case IP_ERROR_NOT_OPTION:
+		snprintf(buffer, sizeof(buffer),
+				"%s: no such option", arg);
+		break;
 	case IP_ERROR_INTERNAL:
 		snprintf(buffer, sizeof(buffer), "%s: internal error", arg);
 		break;
