@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,18 +11,17 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _IP_H
 #define _IP_H
 
-#include "comment.h"
+#include "keyval.h"
 #include "sf.h"
+#include "channelmap.h"
 
 #ifndef __GNUC__
 #include <fcntl.h>
@@ -45,12 +45,18 @@ enum {
 	IP_ERROR_INVALID_URI,
 	/* sample format not supported */
 	IP_ERROR_SAMPLE_FORMAT,
+	/* wrong disc inserted */
+	IP_ERROR_WRONG_DISC,
+	/* could not read disc */
+	IP_ERROR_NO_DISC,
 	/* error parsing response line / headers */
 	IP_ERROR_HTTP_RESPONSE,
 	/* usually 404 */
 	IP_ERROR_HTTP_STATUS,
 	/* too many redirections */
 	IP_ERROR_HTTP_REDIRECT_LIMIT,
+	/* plugin does not have this option */
+	IP_ERROR_NOT_OPTION,
 	/*  */
 	IP_ERROR_INTERNAL
 };
@@ -73,6 +79,7 @@ struct input_plugin_data {
 
 	/* filled by plugin */
 	sample_format_t sf;
+	channel_position_t channel_map[CHANNELS_MAX];
 	void *private;
 };
 
@@ -85,7 +92,11 @@ struct input_plugin_ops {
 			struct keyval **comments);
 	int (*duration)(struct input_plugin_data *ip_data);
 	long (*bitrate)(struct input_plugin_data *ip_data);
+	long (*bitrate_current)(struct input_plugin_data *ip_data);
 	char *(*codec)(struct input_plugin_data *ip_data);
+	char *(*codec_profile)(struct input_plugin_data *ip_data);
+	int (*set_option)(int key, const char *val);
+	int (*get_option)(int key, char **val);
 };
 
 /* symbols exported by plugin */
@@ -93,5 +104,6 @@ extern const struct input_plugin_ops ip_ops;
 extern const int ip_priority;
 extern const char * const ip_extensions[];
 extern const char * const ip_mime_types[];
+extern const char * const ip_options[];
 
 #endif
