@@ -1,9 +1,39 @@
+/*
+ * Copyright 2008-2011 Various Authors
+ * Copyright 2008 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "debug.h"
 #include "keyval.h"
 #include "xmalloc.h"
 
 #include <strings.h>
 
+struct keyval *keyvals_new(int num)
+{
+	struct keyval *c = xnew(struct keyval, num + 1);
+	int i;
+
+	for (i = 0; i <= num; i++) {
+		c[i].key = NULL;
+		c[i].val = NULL;
+	}
+	return c;
+}
+
 struct keyval *keyvals_dup(const struct keyval *keyvals)
 {
 	struct keyval *c;
