@@ -1,5 +1,19 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2006 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "lib.h"
@@ -90,21 +104,10 @@ struct fh_entry {
 #define FH_SIZE (1024)
 static struct fh_entry *ti_hash[FH_SIZE] = { NULL, };
 
-/* this is from glib */
-static unsigned int str_hash(const char *str)
-{
-	unsigned int hash = 0;
-	int i;
-
-	for (i = 0; str[i]; i++)
-		hash = (hash << 5) - hash + str[i];
-	return hash;
-}
-
 static int hash_insert(struct track_info *ti)
 {
 	const char *filename = ti->filename;
-	unsigned int pos = str_hash(filename) % FH_SIZE;
+	unsigned int pos = hash_str(filename) % FH_SIZE;
 	struct fh_entry **entryp;
 	struct fh_entry *e;
 
@@ -129,7 +132,7 @@ static int hash_insert(struct track_info *ti)
 static void hash_remove(struct track_info *ti)
 {
 	const char *filename = ti->filename;
-	unsigned int pos = str_hash(filename) % FH_SIZE;
+	unsigned int pos = hash_str(filename) % FH_SIZE;
 	struct fh_entry **entryp;
 
 	entryp = &ti_hash[pos];
