@@ -1,5 +1,19 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2006 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef LIB_H
@@ -11,8 +25,6 @@
 #include "expr.h"
 #include "rbtree.h"
 
-#include <sys/time.h>
-
 struct tree_track {
 	struct shuffle_track shuffle_track;
 
@@ -42,7 +54,9 @@ struct album {
 
 	struct artist *artist;
 	char *name;
+	char *sort_name;
 	char *collkey_name;
+	char *collkey_sort_name;
 	/* date of the first track added to this album */
 	int date;
 };
