@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "ip.h"
@@ -24,6 +23,8 @@
 #include "xmalloc.h"
 #include "read_wrapper.h"
 #include "debug.h"
+#include "utils.h"
+#include "comment.h"
 
 #include <stdio.h>
 #include <math.h>
@@ -68,11 +69,10 @@ static struct nomad_callbacks callbacks = {
 static int mad_open(struct input_plugin_data *ip_data)
 {
 	struct nomad *nomad;
-	struct nomad_info info;
-	int rc, fast;
+	const struct nomad_info *info;
+	int rc;
 
-	fast = 1;
-	rc = nomad_open_callbacks(&nomad, ip_data, fast, &callbacks);
+	rc = nomad_open_callbacks(&nomad, ip_data, &callbacks);
 	switch (rc) {
 	case -NOMAD_ERROR_ERRNO:
 		return -1;
@@ -81,11 +81,12 @@ static int mad_open(struct input_plugin_data *ip_data)
 	}
 	ip_data->private = nomad;
 
-	nomad_info(nomad, &info);
+	info = nomad_info(nomad);
 
 	/* always 16-bit signed little-endian */
-	ip_data->sf = sf_rate(info.sample_rate) | sf_channels(info.channels) |
+	ip_data->sf = sf_rate(info->sample_rate) | sf_channels(info->channels) |
 		sf_bits(16) | sf_signed(1);
+	channel_map_init_waveex(info->channels, 0, ip_data->channel_map);
 	return 0;
 }
 
@@ -119,11 +120,12 @@ static int mad_seek(struct input_plugin_data *ip_data, double offset)
 static int mad_read_comments(struct input_plugin_data *ip_data,
 		struct keyval **comments)
 {
+	struct nomad *nomad = ip_data->private;
+	const struct nomad_lame *lame = nomad_lame(nomad);
 	struct id3tag id3;
 	int fd, rc, save, i;
 	APETAG(ape);
 	GROWING_KEYVALS(c);
-	float rg_track_peak, rg_track_gain;
 
 	fd = open(ip_data->filename, O_RDONLY);
 	if (fd == -1) {
@@ -172,14 +174,14 @@ out:
 	ape_free(&ape);
 
 	/* add last so the other tags get preference */
-	if (nomad_lame_replaygain(ip_data->private, &rg_track_peak, &rg_track_gain) == 0) {
+	if (lame && !isnan(lame->trackGain)) {
 		char buf[64];
 
-		if (!isnan(rg_track_peak)) {
-			sprintf(buf, "%f", rg_track_peak);
+		if (!isnan(lame->peak)) {
+			sprintf(buf, "%f", lame->peak);
 			comments_add_const(&c, "replaygain_track_peak", buf);
 		}
-		sprintf(buf, "%+.1f dB", rg_track_gain);
+		sprintf(buf, "%+.1f dB", lame->trackGain);
 		comments_add_const(&c, "replaygain_track_gain", buf);
 	}
 
@@ -190,23 +192,30 @@ out:
 
 static int mad_duration(struct input_plugin_data *ip_data)
 {
-	struct nomad *nomad;
+	struct nomad *nomad = ip_data->private;
 
-	nomad = ip_data->private;
-	return nomad_time_total(nomad);
+	return nomad_info(nomad)->duration;
 }
 
 static long mad_bitrate(struct input_plugin_data *ip_data)
 {
 	struct nomad *nomad = ip_data->private;
-	long bitrate = nomad_bitrate(nomad);
+	long bitrate = nomad_info(nomad)->avg_bitrate;
+
 	return bitrate != -1 ? bitrate : -IP_ERROR_FUNCTION_NOT_SUPPORTED;
 }
 
+static long mad_current_bitrate(struct input_plugin_data *ip_data)
+{
+	struct nomad *nomad = ip_data->private;
+	return nomad_current_bitrate(nomad);
+}
+
 static char *mad_codec(struct input_plugin_data *ip_data)
 {
 	struct nomad *nomad = ip_data->private;
-	switch (nomad_layer(nomad)) {
+
+	switch (nomad_info(nomad)->layer) {
 	case 3:
 		return xstrdup("mp3");
 	case 2:
@@ -217,6 +226,42 @@ static char *mad_codec(struct input_plugin_data *ip_data)
 	return NULL;
 }
 
+static char *mad_codec_profile(struct input_plugin_data *ip_data)
+{
+	struct nomad *nomad = ip_data->private;
+	const struct nomad_lame *lame = nomad_lame(nomad);
+	const char *mode = nomad_info(nomad)->vbr ? "VBR" : "CBR";
+
+	if (lame) {
+		/* LAME:
+		 * 0: unknown
+		 * 1: cbr
+		 * 2: abr
+		 * 3: vbr rh (--vbr-old)
+		 * 4: vbr mtrh (--vbr-new)
+		 * 5: vbr mt (obsolete)
+		 */
+		int method = lame->vbr_method;
+		if (method == 2)
+			mode = "ABR";
+		else if (method >= 3 && method <= 5) {
+			const struct nomad_xing *xing = nomad_xing(nomad);
+
+			if (xing && xing->flags & XING_SCALE && xing->scale && xing->scale <= 100) {
+				char buf[16];
+				int v = 10 - (xing->scale + 9) / 10;
+				/* quality (-q): 10 - (xing->scale - ((9 - v) * 10)) */
+
+				sprintf(buf, "VBR V%d", v);
+				return xstrdup(buf);
+
+			}
+		}
+	}
+
+	return xstrdup(mode);
+}
+
 const struct input_plugin_ops ip_ops = {
 	.open = mad_open,
 	.close = mad_close,
@@ -225,7 +270,9 @@ const struct input_plugin_ops ip_ops = {
 	.read_comments = mad_read_comments,
 	.duration = mad_duration,
 	.bitrate = mad_bitrate,
-	.codec = mad_codec
+	.bitrate_current = mad_current_bitrate,
+	.codec = mad_codec,
+	.codec_profile = mad_codec_profile
 };
 
 const int ip_priority = 55;
@@ -233,3 +280,4 @@ const char * const ip_extensions[] = { "mp3", "mp2", NULL };
 const char * const ip_mime_types[] = {
 	"audio/mpeg", "audio/x-mp3", "audio/x-mpeg", NULL
 };
+const char * const ip_options[] = { NULL };
