@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mixer.h"
@@ -59,9 +58,9 @@ static int alsa_mixer_exit(void)
 static snd_mixer_elem_t *find_mixer_elem_by_name(const char *goal_name)
 {
 	snd_mixer_elem_t *elem;
-	snd_mixer_selem_id_t *sid;
+	snd_mixer_selem_id_t *sid = NULL;
 
-	snd_mixer_selem_id_alloca(&sid);
+	snd_mixer_selem_id_malloc(&sid);
 
 	for (elem = snd_mixer_first_elem(alsa_mixer_handle); elem;
 		 elem = snd_mixer_elem_next(elem)) {
@@ -75,16 +74,16 @@ static snd_mixer_elem_t *find_mixer_elem_by_name(const char *goal_name)
 		d_print("has playback switch = %d\n", snd_mixer_selem_has_playback_switch(elem));
 
 		if (strcasecmp(name, goal_name) == 0) {
-			if (snd_mixer_selem_has_playback_volume(elem)) {
-				return elem;
-			} else {
+			if (!snd_mixer_selem_has_playback_volume(elem)) {
 				d_print("mixer element `%s' does not have playback volume\n", name);
-				return NULL;
+				elem = NULL;
 			}
+			break;
 		}
 	}
 
-	return NULL;
+	snd_mixer_selem_id_free(sid);
+	return elem;
 }
 
 static int alsa_mixer_open(int *volume_max)
