@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,16 +11,18 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "ip.h"
 #include "file.h"
 #include "xmalloc.h"
+#include "comment.h"
+#ifdef HAVE_CONFIG
+#include "config/modplug.h"
+#endif
 
 #include <modplug.h>
 #include <sys/types.h>
@@ -35,7 +38,8 @@ static int mod_open(struct input_plugin_data *ip_data)
 {
 	struct mod_private *priv;
 	char *contents;
-	int size, rc;
+	off_t size;
+	ssize_t rc;
 	ModPlugFile *file;
 	ModPlug_Settings settings;
 
@@ -92,6 +96,7 @@ static int mod_open(struct input_plugin_data *ip_data)
 #ifdef WORDS_BIGENDIAN
 	ip_data->sf |= sf_bigendian(1);
 #endif
+	channel_map_init_stereo(ip_data->channel_map);
 	return 0;
 }
 
@@ -133,16 +138,21 @@ static int mod_seek(struct input_plugin_data *ip_data, double offset)
 static int mod_read_comments(struct input_plugin_data *ip_data, struct keyval **comments)
 {
 	struct mod_private *priv = ip_data->private;
-	struct keyval *c;
-	const char *name;
+	GROWING_KEYVALS(c);
+	const char *val;
 
-	c = xnew0(struct keyval, 2);
-	name = ModPlug_GetName(priv->file);
-	if (name != NULL && *name != 0) {
-		c[0].key = xstrdup("title");
-		c[0].val = xstrdup(name);
-	}
-	*comments = c;
+	val = ModPlug_GetName(priv->file);
+	if (val && val[0])
+		comments_add_const(&c, "title", val);
+
+#if MODPLUG_API_8
+	val = ModPlug_GetMessage(priv->file);
+	if (val && val[0])
+		comments_add_const(&c, "comment", val);
+#endif
+
+	keyvals_terminate(&c);
+	*comments = c.keyvals;
 	return 0;
 }
 
@@ -158,6 +168,7 @@ static long mod_bitrate(struct input_plugin_data *ip_data)
 	return -IP_ERROR_FUNCTION_NOT_SUPPORTED;
 }
 
+#if MODPLUG_API_8
 static const char *mod_type_to_string(int type)
 {
 	/* from <libmodplug/sndfile.h>, which is C++ */
@@ -188,9 +199,11 @@ static const char *mod_type_to_string(int type)
 	}
 	return NULL;
 }
+#endif
 
 static char *mod_codec(struct input_plugin_data *ip_data)
 {
+#if MODPLUG_API_8
 	struct mod_private *priv = ip_data->private;
 	const char *codec;
 	int type;
@@ -199,6 +212,14 @@ static char *mod_codec(struct input_plugin_data *ip_data)
 	codec = mod_type_to_string(type);
 
 	return codec ? xstrdup(codec) : NULL;
+#else
+	return NULL;
+#endif
+}
+
+static char *mod_codec_profile(struct input_plugin_data *ip_data)
+{
+	return NULL;
 }
 
 const struct input_plugin_ops ip_ops = {
@@ -209,7 +230,9 @@ const struct input_plugin_ops ip_ops = {
 	.read_comments = mod_read_comments,
 	.duration = mod_duration,
 	.bitrate = mod_bitrate,
-	.codec = mod_codec
+	.bitrate_current = mod_bitrate,
+	.codec = mod_codec,
+	.codec_profile = mod_codec_profile
 };
 
 const int ip_priority = 50;
@@ -219,3 +242,4 @@ const char * const ip_extensions[] = {
 	"psm", NULL
 };
 const char * const ip_mime_types[] = { NULL };
+const char * const ip_options[] = { NULL };
