@@ -1,4 +1,5 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2006 dnk <dnk@bjum.net>
  *
  * This program is free software; you can redistribute it and/or
@@ -12,16 +13,18 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "ip.h"
 #include "xmalloc.h"
 #include "debug.h"
 #include "file.h"
+#ifdef HAVE_CONFIG
 #include "config/mp4.h"
+#endif
+#include "comment.h"
+#include "aac.h"
 
 #if USE_MPEG4IP
 #include <mp4.h>
@@ -53,6 +56,11 @@ struct mp4_private {
 		MP4SampleId sample;
 		MP4SampleId num_samples;
 	} mp4;
+
+	struct {
+		unsigned long samples;
+		unsigned long bytes;
+	} current;
 };
 
 
@@ -92,6 +100,34 @@ static MP4TrackId mp4_get_track(MP4FileHandle *handle)
 	return MP4_INVALID_TRACK_ID;
 }
 
+static void mp4_get_channel_map(struct input_plugin_data *ip_data)
+{
+	struct mp4_private *priv = ip_data->private;
+	unsigned char *aac_data = NULL;
+	unsigned int aac_data_len = 0;
+	NeAACDecFrameInfo frame_info;
+	int i;
+
+	ip_data->channel_map[0] = CHANNEL_POSITION_INVALID;
+
+	if (MP4ReadSample(priv->mp4.handle, priv->mp4.track, priv->mp4.sample,
+			&aac_data, &aac_data_len, NULL, NULL, NULL, NULL) == 0)
+		return;
+
+	if (!aac_data)
+		return;
+
+	NeAACDecDecode(priv->decoder, &frame_info, aac_data, aac_data_len);
+	free(aac_data);
+
+	if (frame_info.error != 0 || frame_info.bytesconsumed <= 0
+			|| frame_info.channels > CHANNELS_MAX)
+		return;
+
+	for (i = 0; i < frame_info.channels; i++)
+		ip_data->channel_map[i] = channel_position_aac(frame_info.channel_position[i]);
+}
+
 static void mp4_close_handle(MP4FileHandle handle)
 {
 #ifdef MP4_CLOSE_DO_NOT_COMPUTE_BITRATE
@@ -109,13 +145,17 @@ static int mp4_open(struct input_plugin_data *ip_data)
 	unsigned int buf_size;
 	int rc = -IP_ERROR_FILE_FORMAT;
 
+	const struct mp4_private priv_init = {
+		.decoder = NULL
+	};
 
 	/* http://sourceforge.net/forum/message.php?msg_id=3578887 */
 	if (ip_data->remote)
 		return -IP_ERROR_FUNCTION_NOT_SUPPORTED;
 
 	/* init private struct */
-	priv = xnew0(struct mp4_private, 1);
+	priv = xnew(struct mp4_private, 1);
+	*priv = priv_init;
 	ip_data->private = priv;
 
 	priv->decoder = NeAACDecOpen();
@@ -123,7 +163,7 @@ static int mp4_open(struct input_plugin_data *ip_data)
 	/* set decoder config */
 	neaac_cfg = NeAACDecGetCurrentConfiguration(priv->decoder);
 	neaac_cfg->outputFormat = FAAD_FMT_16BIT;	/* force 16 bit audio */
-	neaac_cfg->downMatrix = 1;			/* 5.1 -> stereo */
+	neaac_cfg->downMatrix = 0;			/* NOT 5.1 -> stereo */
 	NeAACDecSetConfiguration(priv->decoder, neaac_cfg);
 
 	/* open mpeg-4 file */
@@ -174,6 +214,7 @@ static int mp4_open(struct input_plugin_data *ip_data)
 #if defined(WORDS_BIGENDIAN)
 	ip_data->sf |= sf_bigendian(1);
 #endif
+	mp4_get_channel_map(ip_data);
 
 	return 0;
 
@@ -240,6 +281,10 @@ static int decode_one_frame(struct input_plugin_data *ip_data, void *buffer, int
 	}
 
 	sample_buf = NeAACDecDecode(priv->decoder, &frame_info, aac_data, aac_data_len);
+	if (frame_info.error == 0 && frame_info.samples > 0) {
+		priv->current.samples += frame_info.samples;
+		priv->current.bytes += frame_info.bytesconsumed;
+	}
 
 	free(aac_data);
 
@@ -411,6 +456,8 @@ static int mp4_read_comments(struct input_plugin_data *ip_data,
 		comments_add_const(&c, "artistsort", tags->sortArtist);
 	if (tags->sortAlbumArtist)
 		comments_add_const(&c, "albumartistsort", tags->sortAlbumArtist);
+	if (tags->sortAlbum)
+		comments_add_const(&c, "albumsort", tags->sortAlbum);
 	if (tags->album)
 		comments_add_const(&c, "album", tags->album);
 	if (tags->name)
@@ -482,11 +529,54 @@ static long mp4_bitrate(struct input_plugin_data *ip_data)
 	return bitrate ? bitrate : -IP_ERROR_FUNCTION_NOT_SUPPORTED;
 }
 
+static long mp4_current_bitrate(struct input_plugin_data *ip_data)
+{
+	struct mp4_private *priv = ip_data->private;
+	long bitrate = -1;
+	if (priv->current.samples > 0) {
+		priv->current.samples /= priv->channels;
+		bitrate = (8 * priv->current.bytes * priv->sample_rate) / priv->current.samples;
+		priv->current.samples = 0;
+		priv->current.bytes = 0;
+	}
+	return bitrate;
+}
+
 static char *mp4_codec(struct input_plugin_data *ip_data)
 {
 	return xstrdup("aac");
 }
 
+static const char *object_type_to_str(uint8_t obj_type)
+{
+	switch (obj_type) {
+	case MP4_MPEG4_AAC_MAIN_AUDIO_TYPE:	return "Main";
+	case MP4_MPEG4_AAC_LC_AUDIO_TYPE:	return "LC";
+	case MP4_MPEG4_AAC_SSR_AUDIO_TYPE:	return "SSR";
+	case MP4_MPEG4_AAC_LTP_AUDIO_TYPE:	return "LTP";
+#ifdef MP4_MPEG4_AAC_HE_AUDIO_TYPE
+	case MP4_MPEG4_AAC_HE_AUDIO_TYPE:	return "HE";
+#endif
+	case MP4_MPEG4_AAC_SCALABLE_AUDIO_TYPE:	return "Scalable";
+	}
+	return NULL;
+}
+
+static char *mp4_codec_profile(struct input_plugin_data *ip_data)
+{
+	struct mp4_private *priv = ip_data->private;
+	const char *profile;
+	uint8_t obj_type;
+
+	obj_type = MP4GetTrackEsdsObjectTypeId(priv->mp4.handle, priv->mp4.track);
+	if (obj_type == MP4_MPEG4_AUDIO_TYPE)
+		obj_type = MP4GetTrackAudioMpeg4Type(priv->mp4.handle, priv->mp4.track);
+
+	profile = object_type_to_str(obj_type);
+
+	return profile ? xstrdup(profile) : NULL;
+}
+
 const struct input_plugin_ops ip_ops = {
 	.open = mp4_open,
 	.close = mp4_close,
@@ -495,9 +585,12 @@ const struct input_plugin_ops ip_ops = {
 	.read_comments = mp4_read_comments,
 	.duration = mp4_duration,
 	.bitrate = mp4_bitrate,
-	.codec = mp4_codec
+	.bitrate_current = mp4_current_bitrate,
+	.codec = mp4_codec,
+	.codec_profile = mp4_codec_profile
 };
 
 const int ip_priority = 50;
 const char * const ip_extensions[] = { "mp4", "m4a", "m4b", NULL };
 const char * const ip_mime_types[] = { /*"audio/mp4", "audio/mp4a-latm",*/ NULL };
+const char * const ip_options[] = { NULL };
