@@ -1,7 +1,21 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2006 Chun-Yu Shei <cshei AT cs.indiana.edu>
  *
  * Cleaned up by Timo Hirvonen <tihirvon@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "ip.h"
@@ -11,7 +25,9 @@
 #include "xmalloc.h"
 #include "read_wrapper.h"
 
+#ifdef HAVE_CONFIG
 #include "config/mpc.h"
+#endif
 
 #if MPC_SV8
 #include <mpc/mpcdec.h>
@@ -25,7 +41,9 @@
 #define get_ip_data(d) (d)
 #endif
 
-#include <inttypes.h>
+#include <stdint.h>
+#include <sys/types.h>
+#include <unistd.h>
 #include <errno.h>
 
 struct mpc_private {
@@ -54,6 +72,11 @@ struct mpc_private {
 	 * use MPC_DECODER_BUFFER_LENGTH just to be sure it works
 	 */
 	MPC_SAMPLE_FORMAT samples[MPC_DECODER_BUFFER_LENGTH];
+
+	struct {
+		unsigned long samples;
+		unsigned long bits;
+	} current;
 };
 
 /* callbacks */
@@ -75,10 +98,8 @@ static mpc_int32_t read_impl(callback_t *data, void *ptr, mpc_int32_t size)
 static mpc_bool_t seek_impl(callback_t *data, mpc_int32_t offset)
 {
 	struct input_plugin_data *ip_data = get_ip_data(data);
-	int rc;
 
-	rc = lseek(ip_data->fd, offset, SEEK_SET);
-	if (rc == -1)
+	if (lseek(ip_data->fd, offset, SEEK_SET) == -1)
 		return MPC_FALSE;
 	return MPC_TRUE;
 }
@@ -109,22 +130,27 @@ static int mpc_open(struct input_plugin_data *ip_data)
 {
 	struct mpc_private *priv;
 
-	priv = xnew0(struct mpc_private, 1);
+	const struct mpc_private priv_init = {
+		.file_size = -1,
+		/* set up an mpc_reader linked to our function implementations */
+		.reader = {
+			.read     = read_impl,
+			.seek     = seek_impl,
+			.tell     = tell_impl,
+			.get_size = get_size_impl,
+			.canseek  = canseek_impl,
+			.data     = ip_data
+		}
+	};
+
+	priv = xnew(struct mpc_private, 1);
+	*priv = priv_init;
 
-	priv->file_size = -1;
 	if (!ip_data->remote) {
 		priv->file_size = lseek(ip_data->fd, 0, SEEK_END);
 		lseek(ip_data->fd, 0, SEEK_SET);
 	}
 
-	/* set up an mpc_reader linked to our function implementations */
-	priv->reader.read = read_impl;
-	priv->reader.seek = seek_impl;
-	priv->reader.tell = tell_impl;
-	priv->reader.get_size = get_size_impl;
-	priv->reader.canseek = canseek_impl;
-	priv->reader.data = ip_data;
-
 	/* must be before mpc_streaminfo_read() */
 	ip_data->private = priv;
 
@@ -156,6 +182,7 @@ static int mpc_open(struct input_plugin_data *ip_data)
 
 	ip_data->sf = sf_rate(priv->info.sample_freq) | sf_channels(priv->info.channels) |
 		sf_bits(16) | sf_signed(1);
+	channel_map_init_waveex(priv->info.channels, 0, ip_data->channel_map);
 	return 0;
 }
 
@@ -234,11 +261,15 @@ static int mpc_read(struct input_plugin_data *ip_data, char *buffer, int count)
 
 		samples = frame.samples;
 		priv->samples_avail = samples * priv->info.channels;
+
+		priv->current.samples += frame.samples;
+		priv->current.bits += frame.bits;
 	}
 #else
 
 	if (priv->samples_avail == 0) {
-		uint32_t status = mpc_decoder_decode(&priv->decoder, priv->samples, NULL, NULL);
+		uint32_t acc = 0, bits = 0;
+		uint32_t status = mpc_decoder_decode(&priv->decoder, priv->samples, &acc, &bits);
 
 		if (status == (uint32_t)(-1)) {
 			/* right ret val? */
@@ -253,6 +284,9 @@ static int mpc_read(struct input_plugin_data *ip_data, char *buffer, int count)
 		 * the api documentation is wrong
 		 */
 		priv->samples_avail = status * priv->info.channels;
+
+		priv->current.samples += status;
+		priv->current.bits += bits;
 	}
 #endif
 
@@ -365,6 +399,19 @@ static long mpc_bitrate(struct input_plugin_data *ip_data)
 	return -IP_ERROR_FUNCTION_NOT_SUPPORTED;
 }
 
+static long mpc_current_bitrate(struct input_plugin_data *ip_data)
+{
+	struct mpc_private *priv = ip_data->private;
+	long bitrate = -1;
+	if (priv->current.samples > 0) {
+		bitrate = (priv->info.sample_freq * priv->current.bits) / priv->current.samples;
+		priv->current.samples = 0;
+		priv->current.bits = 0;
+	}
+	return bitrate;
+
+}
+
 static char *mpc_codec(struct input_plugin_data *ip_data)
 {
 	struct mpc_private *priv = ip_data->private;
@@ -377,6 +424,26 @@ static char *mpc_codec(struct input_plugin_data *ip_data)
 	return NULL;
 }
 
+static char *mpc_codec_profile(struct input_plugin_data *ip_data)
+{
+	struct mpc_private *priv = ip_data->private;
+	const char *profile = priv->info.profile_name;
+	char *s = NULL;
+
+	if (profile && profile[0]) {
+		int i;
+
+		/* remove single quotes */
+		while (*profile == '\'')
+			profile++;
+		s = xstrdup(profile);
+		for (i = strlen(s) - 1; s[i] == '\'' && i >= 0; i--)
+			s[i] = '\0';
+	}
+
+	return s;
+}
+
 const struct input_plugin_ops ip_ops = {
 	.open = mpc_open,
 	.close = mpc_close,
@@ -385,9 +452,12 @@ const struct input_plugin_ops ip_ops = {
 	.read_comments = mpc_read_comments,
 	.duration = mpc_duration,
 	.bitrate = mpc_bitrate,
-	.codec = mpc_codec
+	.bitrate_current = mpc_current_bitrate,
+	.codec = mpc_codec,
+	.codec_profile = mpc_codec_profile
 };
 
 const int ip_priority = 50;
 const char *const ip_extensions[] = { "mpc", "mpp", "mp+", NULL };
 const char *const ip_mime_types[] = { "audio/x-musepack", NULL };
+const char *const ip_options[] = { NULL };
