@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -40,9 +39,11 @@
 #include <string.h>
 #include <errno.h>
 
-/* the number of samples of silence the decoder inserts at start */
-#define DECODERDELAY 529
+#define INPUT_BUFFER_SIZE	(5 * 8192)
+#define SEEK_IDX_INTERVAL	15
 
+/* the number of samples of silence the decoder inserts at start */
+#define DECODERDELAY		529
 
 #define XING_MAGIC (('X' << 24) | ('i' << 16) | ('n' << 8) | 'g')
 #define INFO_MAGIC (('I' << 24) | ('n' << 16) | ('f' << 8) | 'o')
@@ -63,7 +64,6 @@ struct nomad {
 	   ref: http://www.mars.org/mailman/public/mad-dev/2001-May/000262.html */
 	unsigned char input_buffer[INPUT_BUFFER_SIZE + MAD_BUFFER_GUARD];
 	int i;
-	unsigned int fast : 1;
 	unsigned int has_xing : 1;
 	unsigned int has_lame : 1;
 	unsigned int seen_first_frame : 1;
@@ -73,60 +73,25 @@ struct nomad {
 	int end_drop_samples;
 	int end_drop_frames;
 
-	struct {
-		unsigned int is_info : 1;
-		unsigned int flags;
-		unsigned int nr_frames;
-		unsigned int bytes;
-		unsigned int scale;
-		unsigned char toc[100];
-	} xing;
-
-	struct {
-		char encoder[10];   /* 9 byte encoder name/version ("LAME3.97b") */
-		float peak;         /* replaygain peak */
-		float trackGain;    /* replaygain track gain */
-		float albumGain;    /* replaygain album gain */
-		int encoderDelay;   /* # of added samples at start of mp3 */
-		int encoderPadding; /* # of added samples at end of mp3 */
-	} lame;
+	struct nomad_xing xing;
+	struct nomad_lame lame;
 
 	struct {
 		int size;
 		struct seek_idx_entry *table;
 	} seek_idx;
 
+	struct {
+		unsigned long long int bitrate_sum;
+		unsigned long nr_frames;
+	} current;
+
 	struct nomad_info info;
 	void *datasource;
 	int datasource_fd;
 	struct nomad_callbacks cbs;
 };
 
-/* ------------------------------------------------------------------------- */
-
-static ssize_t default_read(void *datasource, void *buffer, size_t count)
-{
-	int fd = *(int *)datasource;
-
-	return read(fd, buffer, count);
-}
-
-static off_t default_lseek(void *datasource, off_t offset, int whence)
-{
-	int fd = *(int *)datasource;
-
-	return lseek(fd, offset, whence);
-}
-
-static int default_close(void *datasource)
-{
-	int fd = *(int *)datasource;
-
-	return close(fd);
-}
-
-/* ------------------------------------------------------------------------- */
-
 static inline int scale(mad_fixed_t sample)
 {
 	sample += 1L << (MAD_F_FRACBITS - 16);
@@ -171,9 +136,16 @@ static int parse_lame(struct nomad *nomad, struct mad_bitptr ptr, int bitlen)
 	d_print("detected LAME version %s\n", nomad->lame.encoder + 4);
 #endif
 
+	i = mad_bit_read(&ptr, 4);
+#if defined(DEBUG_LAME)
+	d_print("LAME tag revision: %d\n", i);
+#endif
+	nomad->lame.vbr_method = mad_bit_read(&ptr, 4);
+
 	/* ReplayGain in LAME tag was added in 3.94 */
 	if (version_major > 3 || (version_major == 3 && version_minor >= 94)) {
-		mad_bit_read(&ptr, 16);
+		/* lowpass */
+		mad_bit_read(&ptr, 8);
 
 		/* The reference volume was changed from the 83dB used in the
 		 * ReplayGain spec to 89dB in lame 3.95.1.  Bump the gain for older
@@ -202,9 +174,14 @@ static int parse_lame(struct nomad *nomad, struct mad_bitptr ptr, int bitlen)
 			*/
 		}
 
+		/*
+		 * 4 encoding flags
+		 * 4 ATH type
+		 * 8 minimal bitrate (if ABR -> specified bitrate)
+		 */
 		mad_bit_read(&ptr, 16);
 	} else
-		mad_bit_read(&ptr, 96);
+		mad_bit_read(&ptr, 88);
 
 	nomad->lame.encoderDelay = mad_bit_read(&ptr, 12);
 	nomad->lame.encoderPadding = mad_bit_read(&ptr, 12);
@@ -222,8 +199,6 @@ static int parse_lame(struct nomad *nomad, struct mad_bitptr ptr, int bitlen)
 
 	mad_bit_read(&ptr, 96);
 
-	bitlen -= 288;
-
 	nomad->start_drop_frames = 1;	/* XING/LAME header is an empty frame */
 	nomad->start_drop_samples = nomad->lame.encoderDelay + DECODERDELAY;
 	nomad->end_drop_samples = nomad->lame.encoderPadding - DECODERDELAY;
@@ -233,13 +208,6 @@ static int parse_lame(struct nomad *nomad, struct mad_bitptr ptr, int bitlen)
 	return 1;
 }
 
-enum {
-	XING_FRAMES = 0x00000001L,
-	XING_BYTES  = 0x00000002L,
-	XING_TOC    = 0x00000004L,
-	XING_SCALE  = 0x00000008L
-};
-
 /*
  * format:
  *
@@ -447,37 +415,27 @@ static void calc_frames_fast(struct nomad *nomad)
 
 static void calc_bitrate_fast(struct nomad *nomad)
 {
-	if (nomad->has_xing && (nomad->xing.flags & XING_BYTES) && !nomad->xing.is_info) {
-		nomad->info.avg_bitrate = (nomad->xing.bytes * 8.0) / nomad->info.duration;
-		nomad->info.vbr = 1;
-#if defined(DEBUG_XING)
-		d_print("seems to be VBR, bitrate from Xing Header: %d\n", nomad->info.avg_bitrate);
-#endif
-	} else {
-		nomad->info.avg_bitrate = nomad->frame.header.bitrate;
+	nomad->info.vbr = nomad->has_xing ? !nomad->xing.is_info : 0;
+
+	if (nomad->has_lame && nomad->lame.vbr_method == 1)
 		nomad->info.vbr = 0;
-#if defined(DEBUG_XING)
-		if (nomad->xing.is_info)
-			d_print("Xing header == Info, ");
-		d_print("taking bitrate from first frame: %d\n", nomad->info.avg_bitrate);
-#endif
-	}
+
+	if (nomad->has_xing && (nomad->xing.flags & XING_BYTES) && nomad->xing.bytes)
+		nomad->info.avg_bitrate = (nomad->xing.bytes * 8.0) / nomad->info.duration;
+	else
+		nomad->info.avg_bitrate = nomad->frame.header.bitrate;
 }
 
 /*
  * fields
  *     nomad->info.avg_bitrate and
  *     nomad->info.vbr
- * are only estimated if fast = 1
+ * are only estimated
  */
 static int scan(struct nomad *nomad)
 {
 	struct mad_header *header = &nomad->frame.header;
-	int old_bitrate = 0;
-	unsigned long long int bitrate_sum = 0;
 
-	nomad->info.nr_frames = 0;
-	nomad->info.vbr = 0;
 	while (1) {
 		int rc;
 
@@ -500,39 +458,24 @@ static int scan(struct nomad *nomad)
 		}
 
 		build_seek_index(nomad);
-		bitrate_sum += header->bitrate;
-		nomad->info.nr_frames++;
 
-		if (nomad->info.nr_frames == 1) {
-			// first valid frame
-			nomad->info.sample_rate = header->samplerate;
-			nomad->info.channels = MAD_NCHANNELS(header);
-			nomad->info.layer = header->layer;
-			nomad->info.dual_channel = header->mode == MAD_MODE_DUAL_CHANNEL;
-			nomad->info.joint_stereo = header->mode == MAD_MODE_JOINT_STEREO;
+		// first valid frame
+		nomad->info.sample_rate = header->samplerate;
+		nomad->info.channels = MAD_NCHANNELS(header);
+		nomad->info.layer = header->layer;
+		nomad->info.dual_channel = header->mode == MAD_MODE_DUAL_CHANNEL;
+		nomad->info.joint_stereo = header->mode == MAD_MODE_JOINT_STEREO;
 
-			xing_parse(nomad);
-
-			if (nomad->fast) {
-				calc_frames_fast(nomad);
-				break;
-			}
-		} else {
-			if (old_bitrate != header->bitrate)
-				nomad->info.vbr = 1;
-		}
-		old_bitrate = header->bitrate;
+		xing_parse(nomad);
+		calc_frames_fast(nomad);
+		break;
 	}
 	if (nomad->info.nr_frames == 0) {
 		d_print("error: not an mp3 file!\n");
 		return -NOMAD_ERROR_FILE_FORMAT;
 	}
 	nomad->info.duration = timer_to_seconds(nomad->timer);
-	if (!nomad->fast)
-		nomad->info.avg_bitrate = bitrate_sum / nomad->info.nr_frames;
-	else {
-		calc_bitrate_fast(nomad);
-	}
+	calc_bitrate_fast(nomad);
 	nomad->cur_frame = 0;
 	nomad->cbs.lseek(nomad->datasource, 0, SEEK_SET);
 	nomad->input_offset = 0;
@@ -562,7 +505,9 @@ start:
 		goto start;
 	}
 	nomad->cur_frame++;
-	if (nomad->info.filesize > 0) {
+	nomad->current.bitrate_sum += nomad->frame.header.bitrate;
+	nomad->current.nr_frames++;
+	if (nomad->info.filesize != -1) {
 		build_seek_index(nomad);
 	} else {
 		mad_timer_add(&nomad->timer, nomad->frame.header.duration);
@@ -592,18 +537,14 @@ static void free_mad(struct nomad *nomad)
 	mad_synth_finish(nomad->synth);
 }
 
-static int do_open(struct nomad *nomad, int fast)
+static int do_open(struct nomad *nomad)
 {
 	int rc;
 
 	init_mad(nomad);
 	nomad->info.filesize = nomad->cbs.lseek(nomad->datasource, 0, SEEK_END);
-	if (nomad->info.filesize == -1) {
-		nomad->fast = 1;
-	} else {
-		nomad->fast = fast != 0;
+	if (nomad->info.filesize != -1)
 		nomad->cbs.lseek(nomad->datasource, 0, SEEK_SET);
-	}
 	if (nomad->info.filesize == -1) {
 		rc = decode(nomad);
 		if (rc < 0)
@@ -655,35 +596,25 @@ eof:
 	return -NOMAD_ERROR_FILE_FORMAT;
 }
 
-int nomad_open(struct nomad **nomadp, int fd, int fast)
+int nomad_open_callbacks(struct nomad **nomadp, void *datasource, struct nomad_callbacks *cbs)
 {
 	struct nomad *nomad;
 
-	nomad = xnew0(struct nomad, 1);
-	nomad->datasource = &nomad->datasource_fd;
-	nomad->datasource_fd = fd;
-	nomad->cbs.read = default_read;
-	nomad->cbs.lseek = default_lseek;
-	nomad->cbs.close = default_close;
-	nomad->start_drop_samples = 0;
-	nomad->end_drop_samples = 0;
+	const struct nomad nomad_init = {
+		.datasource = datasource,
+		.cbs = {
+			.read  = cbs->read,
+			.lseek = cbs->lseek,
+			.close = cbs->close
+		}
+	};
+
+	nomad = xnew(struct nomad, 1);
+	*nomad = nomad_init;
 	nomad->lame.peak = nomad->lame.trackGain = nomad->lame.albumGain = strtof("NAN", NULL);
 	*nomadp = nomad;
 	/* on error do_open calls nomad_close */
-	return do_open(nomad, fast);
-}
-
-int nomad_open_callbacks(struct nomad **nomadp, void *datasource, int fast, struct nomad_callbacks *cbs)
-{
-	struct nomad *nomad;
-
-	nomad = xnew0(struct nomad, 1);
-	nomad->datasource = datasource;
-	nomad->cbs = *cbs;
-	nomad->lame.peak = nomad->lame.trackGain = nomad->lame.albumGain = strtof("NAN", NULL);
-	*nomadp = nomad;
-	/* on error do_open calls nomad_close */
-	return do_open(nomad, fast);
+	return do_open(nomad);
 }
 
 void nomad_close(struct nomad *nomad)
@@ -694,11 +625,6 @@ void nomad_close(struct nomad *nomad)
 	free(nomad);
 }
 
-void nomad_info(struct nomad *nomad, struct nomad_info *info)
-{
-	*info = nomad->info;
-}
-
 int nomad_read(struct nomad *nomad, char *buffer, int count)
 {
 	int i, j, size, psize, to;
@@ -807,7 +733,7 @@ static int nomad_time_seek_accurate(struct nomad *nomad, double pos)
 	int rc;
 
 	/* seek to beginning of file and search frame-by-frame */
-	if (nomad->cbs.lseek(nomad->datasource, 0, SEEK_SET) < 0)
+	if (nomad->cbs.lseek(nomad->datasource, 0, SEEK_SET) == -1)
 		return -1;
 
 	/* XING header should NOT be counted - if we're here, we know it's present */
@@ -887,7 +813,7 @@ int nomad_time_seek(struct nomad *nomad, double pos)
 			nomad->timer = nomad->seek_idx.table[idx].timer;
 		}
 	}
-	if (nomad->cbs.lseek(nomad->datasource, offset, SEEK_SET) < 0)
+	if (nomad->cbs.lseek(nomad->datasource, offset, SEEK_SET) == -1)
 		return -1;
 
 	nomad->input_offset = offset;
@@ -918,31 +844,28 @@ int nomad_time_seek(struct nomad *nomad, double pos)
 	return 0;
 }
 
-int nomad_lame_replaygain(struct nomad *nomad, float *peak, float *trackGain)
+const struct nomad_xing *nomad_xing(struct nomad *nomad)
 {
-	if (isnan(nomad->lame.trackGain))
-		return -1;
-	*peak = nomad->lame.peak;
-	*trackGain = nomad->lame.trackGain;
-	return 0;
+	return nomad->has_xing ? &nomad->xing : NULL;
 }
 
-double nomad_time_tell(struct nomad *nomad)
+const struct nomad_lame *nomad_lame(struct nomad *nomad)
 {
-	return timer_to_seconds(nomad->timer);
+	return nomad->has_lame ? &nomad->lame : NULL;
 }
 
-double nomad_time_total(struct nomad *nomad)
+const struct nomad_info *nomad_info(struct nomad *nomad)
 {
-	return nomad->info.duration;
+	return &nomad->info;
 }
 
-int nomad_bitrate(struct nomad *nomad)
+long nomad_current_bitrate(struct nomad *nomad)
 {
-	return nomad->info.avg_bitrate;
-}
-
-int nomad_layer(struct nomad *nomad)
-{
-	return nomad->info.layer;
+	long bitrate = -1;
+	if (nomad->current.nr_frames > 0) {
+		bitrate = nomad->current.bitrate_sum / nomad->current.nr_frames;
+		nomad->current.bitrate_sum = 0;
+		nomad->current.nr_frames = 0;
+	}
+	return bitrate;
 }
