@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,27 +11,20 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _NOMAD_H
 #define _NOMAD_H
 
-#include <mad.h>
 #include <sys/types.h>
 
 #ifndef __GNUC__
 #include <unistd.h>
 #endif
 
-#define INPUT_BUFFER_SIZE	(5 * 8192)
-
-#define SEEK_IDX_INTERVAL  15
-
 /* default callbacks use read, lseek, close */
 struct nomad_callbacks {
 	ssize_t (*read)(void *datasource, void *buffer, size_t count);
@@ -38,6 +32,32 @@ struct nomad_callbacks {
 	int (*close)(void *datasource);
 };
 
+enum {
+	XING_FRAMES = 0x00000001L,
+	XING_BYTES  = 0x00000002L,
+	XING_TOC    = 0x00000004L,
+	XING_SCALE  = 0x00000008L
+};
+
+struct nomad_xing {
+	unsigned int is_info : 1;
+	unsigned int flags;
+	unsigned int nr_frames;
+	unsigned int bytes;
+	unsigned int scale;
+	unsigned char toc[100];
+};
+
+struct nomad_lame {
+	char encoder[10];   /* 9 byte encoder name/version ("LAME3.97b") */
+	int vbr_method;     /* VBR method */
+	float peak;         /* replaygain peak */
+	float trackGain;    /* replaygain track gain */
+	float albumGain;    /* replaygain album gain */
+	int encoderDelay;   /* # of added samples at start of mp3 */
+	int encoderPadding; /* # of added samples at end of mp3 */
+};
+
 /* always 16-bit signed little-endian */
 struct nomad_info {
 	double duration;
@@ -45,12 +65,12 @@ struct nomad_info {
 	int channels;
 	int nr_frames;
 	int layer;
-	/* -1 if fast = 1 */
+	/* guessed */
 	int vbr;
-	/* -1 if fast = 1 */
+	/* guessed */
 	int avg_bitrate;
 	/* -1 if file not seekable */
-	int filesize;
+	off_t filesize;
 	unsigned int joint_stereo : 1;
 	unsigned int dual_channel : 1;
 };
@@ -64,12 +84,10 @@ enum {
 struct nomad;
 
 /* -NOMAD_ERROR_ERRNO -NOMAD_ERROR_FILE_FORMAT */
-int nomad_open(struct nomad **nomadp, int fd, int fast);
-int nomad_open_callbacks(struct nomad **nomadp, void *datasource, int fast,
+int nomad_open_callbacks(struct nomad **nomadp, void *datasource,
 		struct nomad_callbacks *cbs);
 
 void nomad_close(struct nomad *nomad);
-void nomad_info(struct nomad *nomad, struct nomad_info *info);
 
 /* -NOMAD_ERROR_ERRNO */
 int nomad_read(struct nomad *nomad, char *buffer, int count);
@@ -77,10 +95,9 @@ int nomad_read(struct nomad *nomad, char *buffer, int count);
 /* -NOMAD_ERROR_ERRNO */
 int nomad_time_seek(struct nomad *nomad, double pos);
 
-int nomad_lame_replaygain(struct nomad *nomad, float *peak, float *trackGain);
-double nomad_time_tell(struct nomad *nomad);
-double nomad_time_total(struct nomad *nomad);
-int nomad_bitrate(struct nomad *nomad);
-int nomad_layer(struct nomad *nomad);
+const struct nomad_xing *nomad_xing(struct nomad *nomad);
+const struct nomad_lame *nomad_lame(struct nomad *nomad);
+const struct nomad_info *nomad_info(struct nomad *nomad);
+long nomad_current_bitrate(struct nomad *nomad);
 
 #endif
