@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,17 +11,16 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _OP_H
 #define _OP_H
 
 #include "sf.h"
+#include "channelmap.h"
 
 #ifndef __GNUC__
 #include <fcntl.h>
@@ -50,7 +50,7 @@ enum {
 struct output_plugin_ops {
 	int (*init)(void);
 	int (*exit)(void);
-	int (*open)(sample_format_t sf);
+	int (*open)(sample_format_t sf, const channel_position_t *channel_map);
 	int (*close)(void);
 	int (*drop)(void);
 	int (*write)(const char *buffer, int count);
