@@ -1,5 +1,19 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2006 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "options.h"
@@ -9,7 +23,6 @@
 #include "player.h"
 #include "buffer.h"
 #include "ui_curses.h"
-#include "format_print.h"
 #include "cmus.h"
 #include "misc.h"
 #include "lib.h"
@@ -21,10 +34,14 @@
 #include "file.h"
 #include "prog.h"
 #include "output.h"
+#include "input.h"
+#ifdef HAVE_CONFIG
 #include "config/datadir.h"
+#endif
 #include "track_info.h"
 #include "cache.h"
 #include "debug.h"
+#include "discid.h"
 
 #include <stdio.h>
 #include <errno.h>
@@ -38,6 +55,7 @@
 
 /* initialized option variables */
 
+char *cdda_device = NULL;
 char *output_plugin = NULL;
 char *status_display_program = NULL;
 char *server_password;
@@ -45,13 +63,17 @@ int auto_reshuffle = 1;
 int confirm_run = 1;
 int resume_cmus = 0;
 int show_hidden = 0;
+int show_current_bitrate = 0;
 int show_remaining_time = 0;
 int set_term_title = 1;
+int wrap_search = 1;
 int play_library = 1;
 int repeat = 0;
 int shuffle = 0;
 int display_artist_sort_name;
 int smart_artist_sort = 1;
+int scroll_offset = 2;
+int skip_track_info = 0;
 
 int colors[NR_COLORS] = {
 	-1,
@@ -84,6 +106,18 @@ int colors[NR_COLORS] = {
 	COLOR_WHITE | BRIGHT
 };
 
+int attrs[NR_ATTRS] = {
+	A_NORMAL,
+	A_NORMAL,
+	A_NORMAL,
+	A_NORMAL,
+	A_NORMAL,
+	A_NORMAL,
+	A_NORMAL,
+	A_NORMAL,
+	A_NORMAL
+};
+
 /* uninitialized option variables */
 char *track_win_format = NULL;
 char *track_win_format_va = NULL;
@@ -96,6 +130,7 @@ char *current_alt_format = NULL;
 char *window_title_format = NULL;
 char *window_title_alt_format = NULL;
 char *id3_default_charset = NULL;
+char *icecast_default_charset = NULL;
 
 static void buf_int(char *buf, int val)
 {
@@ -164,6 +199,17 @@ enum format_id {
 
 /* callbacks for normal options {{{ */
 
+static void get_device(unsigned int id, char *buf)
+{
+	strcpy(buf, cdda_device);
+}
+
+static void set_device(unsigned int id, const char *buf)
+{
+	free(cdda_device);
+	cdda_device = expand_filename(buf);
+}
+
 #define SECOND_SIZE (44100 * 16 / 8 * 2)
 static void get_buffer_seconds(unsigned int id, char *buf)
 {
@@ -174,21 +220,45 @@ static void set_buffer_seconds(unsigned int id, const char *buf)
 {
 	int sec;
 
-	if (parse_int(buf, 1, 20, &sec))
+	if (parse_int(buf, 1, 300, &sec))
 		player_set_buffer_chunks((sec * SECOND_SIZE + CHUNK_SIZE / 2) / CHUNK_SIZE);
 }
 
+static void get_scroll_offset(unsigned int id, char *buf)
+{
+	buf_int(buf, scroll_offset);
+}
+
+static void set_scroll_offset(unsigned int id, const char *buf)
+{
+	int offset;
+
+	if (parse_int(buf, 0, 9999, &offset))
+		scroll_offset = offset;
+}
+
 static void get_id3_default_charset(unsigned int id, char *buf)
 {
 	strcpy(buf, id3_default_charset);
 }
 
+static void get_icecast_default_charset(unsigned int id, char *buf)
+{
+	strcpy(buf, icecast_default_charset);
+}
+
 static void set_id3_default_charset(unsigned int id, const char *buf)
 {
 	free(id3_default_charset);
 	id3_default_charset = xstrdup(buf);
 }
 
+static void set_icecast_default_charset(unsigned int id, const char *buf)
+{
+	free(icecast_default_charset);
+	icecast_default_charset = xstrdup(buf);
+}
+
 static const struct {
 	const char *str;
 	sort_key_t key;
@@ -199,6 +269,7 @@ static const struct {
 	{ "tracknumber",	SORT_TRACKNUMBER	},
 	{ "discnumber",		SORT_DISCNUMBER		},
 	{ "date",		SORT_DATE		},
+	{ "originaldate",	SORT_ORIGINALDATE	},
 	{ "genre",		SORT_GENRE		},
 	{ "comment",		SORT_COMMENT		},
 	{ "albumartist",	SORT_ALBUMARTIST	},
@@ -210,6 +281,8 @@ static const struct {
 	{ "rg_album_peak",	SORT_RG_ALBUM_PEAK	},
 	{ "bitrate",		SORT_BITRATE		},
 	{ "codec",		SORT_CODEC		},
+	{ "codec_profile",	SORT_CODEC_PROFILE	},
+	{ "media",		SORT_MEDIA		},
 	{ NULL,                 SORT_INVALID            }
 };
 
@@ -430,7 +503,7 @@ static void set_status_display_program(unsigned int id, const char *buf)
 	free(status_display_program);
 	status_display_program = NULL;
 	if (buf[0])
-		status_display_program = xstrdup(buf);
+		status_display_program = expand_filename(buf);
 }
 
 /* }}} */
@@ -717,6 +790,23 @@ static void toggle_show_hidden(unsigned int id)
 	browser_reload();
 }
 
+static void get_show_current_bitrate(unsigned int id, char *buf)
+{
+	strcpy(buf, bool_names[show_current_bitrate]);
+}
+
+static void set_show_current_bitrate(unsigned int id, const char *buf)
+{
+	if (parse_bool(buf, &show_current_bitrate))
+		update_statusline();
+}
+
+static void toggle_show_current_bitrate(unsigned int id)
+{
+	show_current_bitrate ^= 1;
+	update_statusline();
+}
+
 static void get_show_remaining_time(unsigned int id, char *buf)
 {
 	strcpy(buf, bool_names[show_remaining_time]);
@@ -797,6 +887,37 @@ static void toggle_softvol(unsigned int id)
 	do_set_softvol(soft_vol ^ 1);
 }
 
+static void get_wrap_search(unsigned int id, char *buf)
+{
+	strcpy(buf, bool_names[wrap_search]);
+}
+
+static void set_wrap_search(unsigned int id, const char *buf)
+{
+	parse_bool(buf, &wrap_search);
+}
+
+static void toggle_wrap_search(unsigned int id)
+{
+	wrap_search ^= 1;
+}
+
+static void get_skip_track_info(unsigned int id, char *buf)
+{
+	strcpy(buf, bool_names[skip_track_info]);
+}
+
+static void set_skip_track_info(unsigned int id, const char *buf)
+{
+	parse_bool(buf, &skip_track_info);
+}
+
+static void toggle_skip_track_info(unsigned int id)
+{
+	skip_track_info ^= 1;
+}
+
+
 /* }}} */
 
 /* special callbacks (id set) {{{ */
@@ -832,6 +953,79 @@ static void set_color(unsigned int id, const char *buf)
 	update_full();
 }
 
+static const char * const attr_enum_names[6 + 1] = {
+	"default",
+	"standout", "underline", "reverse", "blink", "bold",
+	NULL
+};
+
+static void get_attr(unsigned int id, char *buf)
+{
+	int attr = attrs[id];
+
+	if (attr == 0) {
+		strcpy(buf, "default");
+		return;
+	}
+
+	if (attr & A_STANDOUT)
+		strcat(buf, "standout|");
+
+	if (attr & A_UNDERLINE)
+		strcat(buf, "underline|");
+
+	if (attr & A_REVERSE)
+		strcat(buf, "reverse|");
+
+	if (attr & A_BLINK)
+		strcat(buf, "blink|");
+
+	if (attr & A_BOLD)
+		strcat(buf, "bold|");
+
+	buf[strlen(buf) - 1] = '\0';
+}
+
+static void set_attr(unsigned int id, const char *buf)
+{
+	int attr = 0;
+	size_t i = 0;
+	size_t offset = 0;
+	size_t length = 0;
+	char*  current;
+
+	do {
+		if (buf[i] == '|' || buf[i] == '\0') {
+			current = xstrndup(&buf[offset], length);
+
+			if (strcmp(current, "default") == 0)
+				attr |= A_NORMAL;
+			else if (strcmp(current, "standout") == 0)
+				attr |= A_STANDOUT;
+			else if (strcmp(current, "underline") == 0)
+				attr |= A_UNDERLINE;
+			else if (strcmp(current, "reverse") == 0)
+				attr |= A_REVERSE;
+			else if (strcmp(current, "blink") == 0)
+				attr |= A_BLINK;
+			else if (strcmp(current, "bold") == 0)
+				attr |= A_BOLD;
+
+			free(current);
+
+			offset = i;
+			length = -1;
+		}
+
+		i++;
+		length++;
+	} while (buf[i - 1] != '\0');
+
+	attrs[id] = attr;
+	update_colors();
+	update_full();
+}
+
 static char **id_to_fmt(enum format_id id)
 {
 	switch (id) {
@@ -884,22 +1078,27 @@ static void set_format(unsigned int id, const char *buf)
 
 /* }}} */
 
-#define DN(name) { #name, get_ ## name, set_ ## name, NULL },
-#define DT(name) { #name, get_ ## name, set_ ## name, toggle_ ## name },
+#define DN(name) { #name, get_ ## name, set_ ## name, NULL, 0 },
+#define DN_FLAGS(name, flags) { #name, get_ ## name, set_ ## name, NULL, flags },
+#define DT(name) { #name, get_ ## name, set_ ## name, toggle_ ## name, 0 },
 
 static const struct {
 	const char *name;
 	opt_get_cb get;
 	opt_set_cb set;
 	opt_toggle_cb toggle;
+	unsigned int flags;
 } simple_options[] = {
 	DT(aaa_mode)
 	DT(auto_reshuffle)
+	DN_FLAGS(device, OPT_PROGRAM_PATH)
 	DN(buffer_seconds)
+	DN(scroll_offset)
 	DT(confirm_run)
 	DT(continue)
 	DT(smart_artist_sort)
 	DN(id3_default_charset)
+	DN(icecast_default_charset)
 	DN(lib_sort)
 	DN(output_plugin)
 	DN(passwd)
@@ -914,13 +1113,16 @@ static const struct {
 	DN(replaygain_preamp)
 	DT(resume)
 	DT(show_hidden)
+	DT(show_current_bitrate)
 	DT(show_remaining_time)
 	DT(set_term_title)
 	DT(shuffle)
 	DT(softvol)
 	DN(softvol_state)
-	DN(status_display_program)
-	{ NULL, NULL, NULL, NULL }
+	DN_FLAGS(status_display_program, OPT_PROGRAM_PATH)
+	DT(wrap_search)
+	DT(skip_track_info)
+	{ NULL, NULL, NULL, NULL, 0 }
 };
 
 static const char * const color_names[NR_COLORS] = {
@@ -949,6 +1151,19 @@ static const char * const color_names[NR_COLORS] = {
 	"color_win_title_fg"
 };
 
+static const char * const attr_names[NR_ATTRS] = {
+	"color_cmdline_attr",
+	"color_statusline_attr",
+	"color_titleline_attr",
+	"color_win_attr",
+	"color_win_cur_sel_attr",
+	"color_cur_sel_attr",
+	"color_win_inactive_cur_sel_attr",
+	"color_win_inactive_sel_attr",
+	"color_win_sel_attr",
+	"color_win_title_attr"
+};
+
 /* default values for the variables which we must initialize but
  * can't do it statically */
 static const struct {
@@ -969,6 +1184,7 @@ static const struct {
 	{ "lib_sort"	,	"albumartist date album discnumber tracknumber title filename" },
 	{ "pl_sort",		"" },
 	{ "id3_default_charset","ISO-8859-1" },
+	{ "icecast_default_charset","ISO-8859-1" },
 	{ NULL, NULL }
 };
 
@@ -976,7 +1192,7 @@ LIST_HEAD(option_head);
 int nr_options = 0;
 
 void option_add(const char *name, unsigned int id, opt_get_cb get,
-		opt_set_cb set, opt_toggle_cb toggle)
+		opt_set_cb set, opt_toggle_cb toggle, unsigned int flags)
 {
 	struct cmus_opt *opt = xnew(struct cmus_opt, 1);
 	struct list_head *item;
@@ -986,6 +1202,7 @@ void option_add(const char *name, unsigned int id, opt_get_cb get,
 	opt->get = get;
 	opt->set = set;
 	opt->toggle = toggle;
+	opt->flags = flags;
 
 	item = option_head.next;
 	while (item != &option_head) {
@@ -1026,14 +1243,19 @@ void options_add(void)
 
 	for (i = 0; simple_options[i].name; i++)
 		option_add(simple_options[i].name, 0, simple_options[i].get,
-				simple_options[i].set, simple_options[i].toggle);
+				simple_options[i].set, simple_options[i].toggle,
+				simple_options[i].flags);
 
 	for (i = 0; i < NR_FMTS; i++)
-		option_add(str_defaults[i].name, i, get_format, set_format, NULL);
+		option_add(str_defaults[i].name, i, get_format, set_format, NULL, 0);
 
 	for (i = 0; i < NR_COLORS; i++)
-		option_add(color_names[i], i, get_color, set_color, NULL);
+		option_add(color_names[i], i, get_color, set_color, NULL, 0);
 
+	for (i = 0; i < NR_ATTRS; i++)
+		option_add(attr_names[i], i, get_attr, set_attr, NULL, 0);
+
+	ip_add_options();
 	op_add_options();
 }
 
@@ -1054,6 +1276,7 @@ void options_load(void)
 	int i;
 
 	/* initialize those that can't be statically initialized */
+	cdda_device = get_default_cdda_device();
 	for (i = 0; str_defaults[i].name; i++)
 		option_set(str_defaults[i].name, str_defaults[i].value);
 
@@ -1082,14 +1305,15 @@ void options_exit(void)
 {
 	struct cmus_opt *opt;
 	struct filter_entry *filt;
+	char filename_tmp[512];
 	char filename[512];
 	FILE *f;
 	int i;
 
-	snprintf(filename, sizeof(filename), "%s/autosave", cmus_config_dir);
-	f = fopen(filename, "w");
+	snprintf(filename_tmp, sizeof(filename_tmp), "%s/autosave.tmp", cmus_config_dir);
+	f = fopen(filename_tmp, "w");
 	if (f == NULL) {
-		warn_errno("creating %s", filename);
+		warn_errno("creating %s", filename_tmp);
 		return;
 	}
 
@@ -1129,6 +1353,11 @@ void options_exit(void)
 	fprintf(f, "\n");
 
 	fclose(f);
+
+	snprintf(filename, sizeof(filename), "%s/autosave", cmus_config_dir);
+	i = rename(filename_tmp, filename);
+	if (i)
+		warn_errno("renaming %s to %s", filename_tmp, filename);
 }
 
 struct resume {
@@ -1138,6 +1367,7 @@ struct resume {
 	char *lib_filename;
 	int view;
 	char *live_filter;
+	char *browser_dir;
 };
 
 static int handle_resume_line(void *data, const char *line)
@@ -1165,6 +1395,9 @@ static int handle_resume_line(void *data, const char *line)
 	} else if (strcmp(cmd, "live-filter") == 0) {
 		free(resume->live_filter);
 		resume->live_filter = xstrdup(unescape(arg));
+	} else if (strcmp(cmd, "browser-dir") == 0) {
+		free(resume->browser_dir);
+		resume->browser_dir = xstrdup(unescape(arg));
 	}
 
 	free(arg);
@@ -1189,7 +1422,7 @@ void resume_load(void)
 		set_view(resume.view);
 	if (resume.lib_filename) {
 		cache_lock();
-		ti = old = cache_get_ti(resume.lib_filename);
+		ti = old = cache_get_ti(resume.lib_filename, 0);
 		cache_unlock();
 		if (ti) {
 			editable_lock();
@@ -1210,7 +1443,7 @@ void resume_load(void)
 	}
 	if (resume.filename) {
 		cache_lock();
-		ti = cache_get_ti(resume.filename);
+		ti = cache_get_ti(resume.filename, 0);
 		cache_unlock();
 		if (ti) {
 			player_set_file(ti);
@@ -1225,18 +1458,24 @@ void resume_load(void)
 		editable_unlock();
 		free(resume.live_filter);
 	}
+	if (resume.browser_dir) {
+		browser_chdir(resume.browser_dir);
+		free(resume.browser_dir);
+	}
 }
 
 void resume_exit(void)
 {
+	char filename_tmp[512];
 	char filename[512];
 	struct track_info *ti;
 	FILE *f;
+	int rc;
 
-	snprintf(filename, sizeof(filename), "%s/resume", cmus_config_dir);
-	f = fopen(filename, "w");
+	snprintf(filename_tmp, sizeof(filename_tmp), "%s/resume.tmp", cmus_config_dir);
+	f = fopen(filename_tmp, "w");
 	if (!f) {
-		warn_errno("creating %s", filename);
+		warn_errno("creating %s", filename_tmp);
 		return;
 	}
 
@@ -1257,6 +1496,12 @@ void resume_exit(void)
 	fprintf(f, "view %s\n", view_names[cur_view]);
 	if (lib_live_filter)
 		fprintf(f, "live-filter %s\n", escape(lib_live_filter));
+	fprintf(f, "browser-dir %s\n", escape(browser_dir));
 
 	fclose(f);
+
+	snprintf(filename, sizeof(filename), "%s/resume", cmus_config_dir);
+	rc = rename(filename_tmp, filename);
+	if (rc)
+		warn_errno("renaming %s to %s", filename_tmp, filename);
 }
