@@ -1,5 +1,19 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2005 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef OPTIONS_H
@@ -13,6 +27,10 @@ typedef void (*opt_get_cb)(unsigned int id, char *buf);
 typedef void (*opt_set_cb)(unsigned int id, const char *buf);
 typedef void (*opt_toggle_cb)(unsigned int id);
 
+enum {
+	OPT_PROGRAM_PATH = 1 << 0,
+};
+
 struct cmus_opt {
 	struct list_head node;
 
@@ -31,6 +49,8 @@ struct cmus_opt {
 
 	/* NULL if not toggle-able */
 	opt_toggle_cb toggle;
+
+	unsigned int flags;
 };
 
 extern struct list_head option_head;
@@ -79,8 +99,23 @@ enum {
 	NR_COLORS
 };
 
+enum {
+	COLOR_CMDLINE_ATTR,
+	COLOR_STATUSLINE_ATTR,
+	COLOR_TITLELINE_ATTR,
+	COLOR_WIN_ATTR,
+	COLOR_WIN_CUR_SEL_ATTR,
+	COLOR_CUR_SEL_ATTR,
+	COLOR_WIN_INACTIVE_CUR_SEL_ATTR,
+	COLOR_WIN_INACTIVE_SEL_ATTR,
+	COLOR_WIN_SEL_ATTR,
+	COLOR_WIN_TITLE_ATTR,
+	NR_ATTRS
+};
+
 #define BRIGHT (1 << 3)
 
+extern char *cdda_device;
 extern char *output_plugin;
 extern char *status_display_program;
 extern char *server_password;
@@ -88,18 +123,23 @@ extern int auto_reshuffle;
 extern int confirm_run;
 extern int resume_cmus;
 extern int show_hidden;
+extern int show_current_bitrate;
 extern int show_remaining_time;
 extern int set_term_title;
+extern int wrap_search;
 extern int play_library;
 extern int repeat;
 extern int shuffle;
 extern int display_artist_sort_name;
 extern int smart_artist_sort;
+extern int scroll_offset;
+extern int skip_track_info;
 
 extern const char * const aaa_mode_names[];
 extern const char * const view_names[NR_VIEWS + 1];
 
 extern int colors[NR_COLORS];
+extern int attrs[NR_ATTRS];
 
 /* format string for track window (tree view) */
 extern char *track_win_format;
@@ -120,6 +160,7 @@ extern char *window_title_format;
 extern char *window_title_alt_format;
 
 extern char *id3_default_charset;
+extern char *icecast_default_charset;
 
 /* build option list */
 void options_add(void);
@@ -138,7 +179,7 @@ void resume_load(void);
 void resume_exit(void);
 
 void option_add(const char *name, unsigned int id, opt_get_cb get,
-		opt_set_cb set, opt_toggle_cb toggle);
+		opt_set_cb set, opt_toggle_cb toggle, unsigned int flags);
 struct cmus_opt *option_find(const char *name);
 void option_set(const char *name, const char *value);
 int parse_enum(const char *buf, int minval, int maxval, const char * const names[], int *val);
