@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2006 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "output.h"
@@ -27,7 +26,9 @@
 #include "debug.h"
 #include "ui_curses.h"
 #include "options.h"
+#ifdef HAVE_CONFIG
 #include "config/libdir.h"
+#endif
 
 #include <string.h>
 #include <strings.h>
@@ -234,7 +235,7 @@ int op_select_any(void)
 		rc = select_plugin(o);
 		if (rc != 0)
 			continue;
-		rc = o->pcm_ops->open(sf);
+		rc = o->pcm_ops->open(sf, NULL);
 		if (rc == 0) {
 			o->pcm_ops->close();
 			break;
@@ -243,11 +244,11 @@ int op_select_any(void)
 	return rc;
 }
 
-int op_open(sample_format_t sf)
+int op_open(sample_format_t sf, const channel_position_t *channel_map)
 {
 	if (op == NULL)
 		return -OP_ERROR_NOT_INITIALIZED;
-	return op->pcm_ops->open(sf);
+	return op->pcm_ops->open(sf, channel_map);
 }
 
 int op_drop(void)
@@ -400,13 +401,13 @@ void op_add_options(void)
 			snprintf(key, sizeof(key), "dsp.%s.%s",
 					o->name,
 					o->pcm_options[oid]);
-			option_add(xstrdup(key), (pid << 16) | oid, get_dsp_option, set_dsp_option, NULL);
+			option_add(xstrdup(key), (pid << 16) | oid, get_dsp_option, set_dsp_option, NULL, 0);
 		}
 		for (oid = 0; o->mixer_ops && o->mixer_options[oid]; oid++) {
 			snprintf(key, sizeof(key), "mixer.%s.%s",
 					o->name,
 					o->mixer_options[oid]);
-			option_add(xstrdup(key), (pid << 16) | oid, get_mixer_option, set_mixer_option, NULL);
+			option_add(xstrdup(key), (pid << 16) | oid, get_mixer_option, set_mixer_option, NULL, 0);
 		}
 		pid++;
 	}
