@@ -1,13 +1,26 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2005 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _OUTPUT_H
 #define _OUTPUT_H
 
-#include "mixer.h"
-#include "op.h"
 #include "sf.h"
+#include "channelmap.h"
 
 extern int volume_max;
 extern int volume_l;
@@ -29,7 +42,7 @@ int op_select_any(void);
  *
  * errors: OP_ERROR_{}
  */
-int op_open(sample_format_t sf);
+int op_open(sample_format_t sf, const channel_position_t *channel_map);
 
 /*
  * drop pcm data
