@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2006 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "player.h"
@@ -26,6 +25,7 @@
 #include "xmalloc.h"
 #include "debug.h"
 #include "compiler.h"
+#include "options.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -57,6 +57,7 @@ struct player_info player_info = {
 	.metadata = { 0, },
 	.status = PLAYER_STATUS_STOPPED,
 	.pos = 0,
+	.current_bitrate = -1,
 	.buffer_fill = 0,
 	.buffer_size = 0,
 	.error_msg = NULL,
@@ -84,6 +85,7 @@ int soft_vol_r;
 static const struct player_callbacks *player_cbs = NULL;
 
 static sample_format_t buffer_sf;
+static CHANNEL_MAP(buffer_channel_map);
 
 static pthread_t producer_thread;
 static pthread_mutex_t producer_mutex = CMUS_MUTEX_INITIALIZER;
@@ -135,9 +137,10 @@ static void reset_buffer(void)
 	pthread_cond_broadcast(&producer_playing);
 }
 
-static void set_buffer_sf(sample_format_t sf)
+static void set_buffer_sf(void)
 {
-	buffer_sf = sf;
+	buffer_sf = ip_get_sf(ip);
+	ip_get_channel_map(ip, buffer_channel_map);
 
 	/* ip_read converts samples to this format */
 	if (sf_get_channels(buffer_sf) <= 2 && sf_get_bits(buffer_sf) <= 16) {
@@ -146,6 +149,7 @@ static void set_buffer_sf(sample_format_t sf)
 #ifdef WORDS_BIGENDIAN
 		buffer_sf |= sf_bigendian(1);
 #endif
+		channel_map_init_stereo(buffer_channel_map);
 	}
 }
 
@@ -394,21 +398,26 @@ static inline int get_next(struct track_info **ti)
 
 /* updating player status {{{ */
 
-static inline void file_changed(struct track_info *ti)
+static inline void __file_changed(struct track_info *ti)
 {
-	player_info_lock();
 	if (player_info.ti)
 		track_info_unref(player_info.ti);
 
 	player_info.ti = ti;
+	update_rg_scale();
+	player_info.metadata[0] = 0;
+	player_info.file_changed = 1;
+}
+
+static inline void file_changed(struct track_info *ti)
+{
+	player_info_lock();
 	if (ti) {
 		d_print("file: %s\n", ti->filename);
 	} else {
 		d_print("unloaded\n");
 	}
-	update_rg_scale();
-	player_info.metadata[0] = 0;
-	player_info.file_changed = 1;
+	__file_changed(ti);
 	player_info_unlock();
 }
 
@@ -438,8 +447,9 @@ static inline void metadata_changed(void)
 static void player_error(const char *msg)
 {
 	player_info_lock();
-	player_info.status = consumer_status;
+	player_info.status = (enum player_status)consumer_status;
 	player_info.pos = 0;
+	player_info.current_bitrate = -1;
 	player_info.buffer_fill = buffer_get_filled_chunks();
 	player_info.buffer_size = buffer_nr_chunks;
 	player_info.status_changed = 1;
@@ -509,15 +519,21 @@ static void __consumer_position_update(void)
 {
 	static unsigned int old_pos = -1;
 	unsigned int pos = 0;
-	
+	long bitrate;
+
 	if (consumer_status == CS_PLAYING || consumer_status == CS_PAUSED)
 		pos = consumer_pos / buffer_second_size();
 	if (pos != old_pos) {
-/* 		d_print("\n"); */
 		old_pos = pos;
 
 		player_info_lock();
 		player_info.pos = pos;
+
+		if (show_current_bitrate) {
+			bitrate = ip_current_bitrate(ip);
+			if (bitrate != -1)
+				player_info.current_bitrate = bitrate;
+		}
 		player_info.position_changed = 1;
 		player_info_unlock();
 	}
@@ -535,8 +551,9 @@ static void __player_status_changed(void)
 		pos = consumer_pos / buffer_second_size();
 
 	player_info_lock();
-	player_info.status = consumer_status;
+	player_info.status = (enum player_status)consumer_status;
 	player_info.pos = pos;
+	player_info.current_bitrate = -1;
 	player_info.buffer_fill = buffer_get_filled_chunks();
 	player_info.buffer_size = buffer_nr_chunks;
 	player_info.status_changed = 1;
@@ -699,8 +716,8 @@ static void __consumer_play(void)
 	} else if (consumer_status == CS_STOPPED) {
 		int rc;
 
-		set_buffer_sf(ip_get_sf(ip));
-		rc = op_open(buffer_sf);
+		set_buffer_sf();
+		rc = op_open(buffer_sf, buffer_channel_map);
 		if (rc) {
 			player_op_error(rc, "opening audio device");
 		} else {
@@ -742,19 +759,22 @@ static void __consumer_pause(void)
 
 /* setting consumer status }}} */
 
-static int change_sf(sample_format_t sf, int drop)
+
+static int change_sf(int drop)
 {
 	int old_sf = buffer_sf;
+	CHANNEL_MAP(old_channel_map);
+	channel_map_copy(old_channel_map, buffer_channel_map);
 
-	set_buffer_sf(sf);
-	if (buffer_sf != old_sf) {
+	set_buffer_sf();
+	if (buffer_sf != old_sf || !channel_map_equal(buffer_channel_map, old_channel_map, sf_get_channels(buffer_sf))) {
 		/* reopen */
 		int rc;
 
 		if (drop)
 			op_drop();
 		op_close();
-		rc = op_open(buffer_sf);
+		rc = op_open(buffer_sf, buffer_channel_map);
 		if (rc) {
 			player_op_error(rc, "opening audio device");
 			__consumer_status_update(CS_STOPPED);
@@ -806,7 +826,7 @@ static void __consumer_handle_eof(void)
 			} else {
 				/* PS_PLAYING */
 				file_changed(ti);
-				if (!change_sf(ip_get_sf(ip), 0))
+				if (!change_sf(0))
 					__prebuffer();
 			}
 		} else {
@@ -1133,7 +1153,7 @@ void player_set_file(struct track_info *ti)
 			__consumer_stop();
 			goto out;
 		}
-		change_sf(ip_get_sf(ip), 1);
+		change_sf(1);
 	}
 out:
 	__player_status_changed();
@@ -1167,7 +1187,7 @@ void player_play_file(struct track_info *ti)
 			__producer_stop();
 	} else {
 		op_drop();
-		change_sf(ip_get_sf(ip), 1);
+		change_sf(1);
 	}
 out:
 	__player_status_changed();
@@ -1176,6 +1196,11 @@ out:
 	player_unlock();
 }
 
+void player_file_changed(struct track_info *ti)
+{
+	__file_changed(ti);
+}
+
 void player_seek(double offset, int relative, int start_playing)
 {
 	int stopped = 0;
@@ -1294,8 +1319,8 @@ void player_set_op(const char *name)
 	}
 
 	if (consumer_status == CS_PLAYING || consumer_status == CS_PAUSED) {
-		set_buffer_sf(ip_get_sf(ip));
-		rc = op_open(buffer_sf);
+		set_buffer_sf();
+		rc = op_open(buffer_sf, buffer_channel_map);
 		if (rc) {
 			__consumer_status_update(CS_STOPPED);
 			__producer_stop();
@@ -1312,11 +1337,6 @@ void player_set_op(const char *name)
 
 void player_set_buffer_chunks(unsigned int nr_chunks)
 {
-	if (nr_chunks < 3)
-		nr_chunks = 3;
-	if (nr_chunks > 30)
-		nr_chunks = 30;
-
 	player_lock();
 	__producer_stop();
 	__consumer_stop();
