@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,18 +11,15 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _PLAYER_H
 #define _PLAYER_H
 
 #include "locking.h"
-#include "comment.h"
 #include "track_info.h"
 
 #include <pthread.h>
@@ -67,6 +65,7 @@ struct player_info {
 	/* status */
 	enum player_status status;
 	int pos;
+	int current_bitrate;
 
 	int buffer_fill;
 	int buffer_size;
@@ -100,6 +99,9 @@ void player_set_file(struct track_info *ti);
 /* set current file and start playing */
 void player_play_file(struct track_info *ti);
 
+/* update track info */
+void player_file_changed(struct track_info *ti);
+
 void player_play(void);
 void player_stop(void);
 void player_pause(void);
