@@ -639,6 +639,12 @@ check_x11()
 # adds PTHREAD_CFLAGS and PTHREAD_LIBS to config.mk
 check_pthread()
 {
+	case `uname -s` in
+	OpenBSD)
+		PTHREAD_LIBS="$PTHREAD_LIBS -pthread"
+		;;
+	esac
+
 	for __libs in "$PTHREAD_LIBS" -lpthread -lc_r -lkse
 	do
 		test -z "$__libs" && continue
