@@ -1,5 +1,19 @@
-/* 
- * Copyright Timo Hirvonen
+/*
+ * Copyright 2008-2011 Various Authors
+ * Copyright 2005 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "search.h"
@@ -7,6 +21,7 @@
 #include "xmalloc.h"
 #include "ui_curses.h"
 #include "convert.h"
+#include "options.h"
 
 struct searchable {
 	void *data;
@@ -38,6 +53,8 @@ static int do_u_search(struct searchable *s, struct iter *iter, const char *text
 		}
 		if (direction == SEARCH_FORWARD) {
 			if (!s->ops.get_next(iter)) {
+				if (!wrap_search)
+					return 0;
 				*iter = s->head;
 				if (!s->ops.get_next(iter))
 					return 0;
@@ -45,6 +62,8 @@ static int do_u_search(struct searchable *s, struct iter *iter, const char *text
 			}
 		} else {
 			if (!s->ops.get_prev(iter)) {
+				if (!wrap_search)
+					return 0;
 				*iter = s->head;
 				if (!s->ops.get_prev(iter))
 					return 0;
