@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "track_info.h"
@@ -25,6 +24,7 @@
 #include "xmalloc.h"
 #include "utils.h"
 #include "debug.h"
+#include "path.h"
 
 #include <string.h>
 #include <math.h>
@@ -34,6 +34,7 @@ static void track_info_free(struct track_info *ti)
 	keyvals_free(ti->comments);
 	free(ti->filename);
 	free(ti->codec);
+	free(ti->codec_profile);
 	free(ti->collkey_artist);
 	free(ti->collkey_album);
 	free(ti->collkey_title);
@@ -51,6 +52,7 @@ struct track_info *track_info_new(const char *filename)
 	ti->ref = 1;
 	ti->comments = NULL;
 	ti->codec = NULL;
+	ti->codec_profile = NULL;
 	return ti;
 }
 
@@ -62,11 +64,24 @@ void track_info_set_comments(struct track_info *ti, struct keyval *comments) {
 	ti->tracknumber = comments_get_int(comments, "tracknumber");
 	ti->discnumber = comments_get_int(comments, "discnumber");
 	ti->date = comments_get_date(comments, "date");
+	ti->originaldate = comments_get_date(comments, "originaldate");
 	ti->genre = keyvals_get_val(comments, "genre");
 	ti->comment = keyvals_get_val(comments, "comment");
 	ti->albumartist = comments_get_albumartist(comments);
 	ti->artistsort = comments_get_artistsort(comments);
+	ti->albumsort = keyvals_get_val(comments, "albumsort");
 	ti->is_va_compilation = track_is_va_compilation(comments);
+	ti->media = keyvals_get_val(comments, "media");
+
+	if (ti->artist == NULL && ti->albumartist != NULL) {
+		/* best guess */
+		ti->artist = ti->albumartist;
+	}
+
+	if (track_info_has_tag(ti) && ti->title == NULL) {
+		/* best guess */
+		ti->title = path_basename(ti->filename);
+	}
 
 	ti->rg_track_gain = comments_get_double(comments, "replaygain_track_gain");
 	ti->rg_track_peak = comments_get_double(comments, "replaygain_track_peak");
@@ -131,13 +146,11 @@ int track_info_matches_full(const struct track_info *ti, const char *text,
 			matched = 1;
 		} else {
 			/* compare with url or filename without path */
-			char *filename = ti->filename;
+			const char *filename = ti->filename;
+
+			if (!is_url(filename))
+				filename = path_basename(filename);
 
-			if (!is_url(filename)) {
-				char *slash = strrchr(ti->filename, '/');
-				if (slash)
-					filename = slash + 1;
-			}
 			if (u_strcasestr_filename(filename, word))
 				matched = 1;
 		}
@@ -182,6 +195,7 @@ int track_info_cmp(const struct track_info *a, const struct track_info *b, const
 		case SORT_TRACKNUMBER:
 		case SORT_DISCNUMBER:
 		case SORT_DATE:
+		case SORT_ORIGINALDATE:
 			res = getentry(a, key, int) - getentry(b, key, int);
 			break;
 		case SORT_FILEMTIME:
