@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _TRACK_INFO_H
@@ -33,12 +32,14 @@ struct track_info {
 	int duration;
 	long bitrate;
 	char *codec;
+	char *codec_profile;
 	int ref;
 	char *filename;
 
 	int tracknumber;
 	int discnumber;
 	int date;
+	int originaldate;
 	double rg_track_gain;
 	double rg_track_peak;
 	double rg_album_gain;
@@ -50,6 +51,8 @@ struct track_info {
 	const char *comment;
 	const char *albumartist;
 	const char *artistsort;
+	const char *albumsort;
+	const char *media;
 
 	char *collkey_artist;
 	char *collkey_album;
@@ -69,6 +72,7 @@ typedef size_t sort_key_t;
 #define SORT_TRACKNUMBER   offsetof(struct track_info, tracknumber)
 #define SORT_DISCNUMBER    offsetof(struct track_info, discnumber)
 #define SORT_DATE          offsetof(struct track_info, date)
+#define SORT_ORIGINALDATE  offsetof(struct track_info, originaldate)
 #define SORT_RG_TRACK_GAIN offsetof(struct track_info, rg_track_gain)
 #define SORT_RG_TRACK_PEAK offsetof(struct track_info, rg_track_peak)
 #define SORT_RG_ALBUM_GAIN offsetof(struct track_info, rg_album_gain)
@@ -80,6 +84,8 @@ typedef size_t sort_key_t;
 #define SORT_FILEMTIME     offsetof(struct track_info, mtime)
 #define SORT_BITRATE       offsetof(struct track_info, bitrate)
 #define SORT_CODEC         offsetof(struct track_info, codec)
+#define SORT_CODEC_PROFILE offsetof(struct track_info, codec_profile)
+#define SORT_MEDIA         offsetof(struct track_info, media)
 #define SORT_INVALID       ((sort_key_t) (-1))
 
 #define TI_MATCH_ARTIST       (1 << 0)
