@@ -1,11 +1,24 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2006 Timo Hirvonen
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "lib.h"
 #include "search_mode.h"
 #include "xmalloc.h"
-#include "comment.h"
 #include "utils.h"
 #include "debug.h"
 #include "mergesort.h"
@@ -15,6 +28,7 @@
 
 #include <ctype.h>
 #include <stdio.h>
+#include <string.h>
 #include <strings.h>
 
 struct searchable *tree_searchable;
@@ -72,7 +86,7 @@ static void tree_set_expand_artist(struct artist *artist, int expand)
 static int tree_search_get_prev(struct iter *iter)
 {
 	struct rb_root *root = iter->data0;
-	struct tree_track *track = iter->data1, *old = track;
+	struct tree_track *track = iter->data1;
 	struct artist *artist;
 	struct album *album;
 
@@ -108,17 +122,13 @@ static int tree_search_get_prev(struct iter *iter)
 		track = to_tree_track(rb_prev(&track->tree_node));
 	}
 	iter->data1 = track;
-	/* collapse old search result */
-	artist = old->album->artist;
-	if (artist != track->album->artist && artist->expanded)
-		tree_set_expand_artist(artist, 0);
 	return 1;
 }
 
 static int tree_search_get_next(struct iter *iter)
 {
 	struct rb_root *root = iter->data0;
-	struct tree_track *track = iter->data1, *old = track;
+	struct tree_track *track = iter->data1;
 	struct artist *artist;
 	struct album *album;
 
@@ -154,10 +164,6 @@ static int tree_search_get_next(struct iter *iter)
 		track = to_tree_track(rb_next(&track->tree_node));
 	}
 	iter->data1 = track;
-	/* collapse old search result */
-	artist = old->album->artist;
-	if (artist != track->album->artist && artist->expanded)
-		tree_set_expand_artist(artist, 0);
 	return 1;
 }
 /* }}} */
@@ -292,25 +298,7 @@ static inline struct tree_track *iter_to_tree_search_track(const struct iter *it
 	return iter->data1;
 }
 
-static int tree_search_matches(void *data, struct iter *iter, const char *text)
-{
-	struct tree_track *track;
-	struct iter tmpiter;
-	unsigned int flags = TI_MATCH_ARTIST | TI_MATCH_ALBUM | TI_MATCH_ALBUMARTIST;
-
-	if (!search_restricted)
-		flags |= TI_MATCH_TITLE;
-	track = iter_to_tree_search_track(iter);
-	if (!track_info_matches(tree_track_info(track), text, flags))
-		return 0;
-	track->album->artist->expanded = 1;
-	album_to_iter(track->album, &tmpiter);
-	window_set_sel(lib_tree_win, &tmpiter);
-
-	tree_track_to_iter(track, &tmpiter);
-	window_set_sel(lib_track_win, &tmpiter);
-	return 1;
-}
+static int tree_search_matches(void *data, struct iter *iter, const char *text);
 
 static const struct searchable_ops tree_search_ops = {
 	.get_prev = tree_search_get_prev,
@@ -394,6 +382,11 @@ static struct artist *artist_new(const char *name, const char *sort_name, int is
 	return a;
 }
 
+static struct artist *artist_copy(const struct artist *artist)
+{
+	return artist_new(artist->name, artist->sort_name, artist->is_compilation);
+}
+
 static void artist_free(struct artist *artist)
 {
 	free(artist->name);
@@ -405,12 +398,15 @@ static void artist_free(struct artist *artist)
 	free(artist);
 }
 
-static struct album *album_new(struct artist *artist, const char *name, int date)
+static struct album *album_new(struct artist *artist, const char *name,
+		const char *sort_name, int date)
 {
 	struct album *album = xnew(struct album, 1);
 
 	album->name = xstrdup(name);
+	album->sort_name = sort_name ? xstrdup(sort_name) : NULL;
 	album->collkey_name = u_strcasecoll_key(name);
+	album->collkey_sort_name = u_strcasecoll_key0(sort_name);
 	album->date = date;
 	rb_root_init(&album->track_root);
 	album->artist = artist;
@@ -421,7 +417,9 @@ static struct album *album_new(struct artist *artist, const char *name, int date
 static void album_free(struct album *album)
 {
 	free(album->name);
+	free(album->sort_name);
 	free(album->collkey_name);
+	free(album->collkey_sort_name);
 	free(album);
 }
 
@@ -498,9 +496,17 @@ static int special_name_cmp(const char *a, const char *collkey_a,
 	return strcmp(collkey_a, collkey_b);
 }
 
+static inline const char *album_sort_collkey(const struct album *a)
+{
+        if (a->sort_name)
+                return a->collkey_sort_name;
+
+        return a->collkey_name;
+}
+
 static int special_album_cmp(const struct album *a, const struct album *b)
 {
-	return special_name_cmp(a->name, a->collkey_name, b->name, b->collkey_name);
+	return special_name_cmp(a->name, album_sort_collkey(a), b->name, album_sort_collkey(b));
 }
 
 static int special_album_cmp_date(const struct album *a, const struct album *b)
@@ -514,7 +520,7 @@ static int special_album_cmp_date(const struct album *a, const struct album *b)
 	if (cmp)
 		return cmp;
 
-	return strcmp(a->collkey_name, b->collkey_name);
+	return strcmp(album_sort_collkey(a), album_sort_collkey(b));
 }
 
 /* has to follow the same logic as artist_sort_name() */
@@ -559,6 +565,43 @@ static struct artist *do_find_artist(const struct artist *artist,
 	return NULL;
 }
 
+/* search (tree) {{{ */
+static struct artist *collapse_artist;
+
+static int tree_search_matches(void *data, struct iter *iter, const char *text)
+{
+	struct tree_track *track;
+	struct iter tmpiter;
+	unsigned int flags = TI_MATCH_ARTIST | TI_MATCH_ALBUM | TI_MATCH_ALBUMARTIST;
+
+	if (!search_restricted)
+		flags |= TI_MATCH_TITLE;
+	track = iter_to_tree_search_track(iter);
+	if (!track_info_matches(tree_track_info(track), text, flags))
+		return 0;
+
+	/* collapse old search result */
+	if (collapse_artist) {
+		struct artist *artist = do_find_artist(collapse_artist, &lib_artist_root, NULL, NULL);
+		if (artist && artist != track->album->artist) {
+			if (artist->expanded)
+				tree_set_expand_artist(artist, 0);
+			artist_free(collapse_artist);
+			collapse_artist = (!track->album->artist->expanded) ? artist_copy(track->album->artist) : NULL;
+		}
+	} else if (!track->album->artist->expanded)
+		collapse_artist = artist_copy(track->album->artist);
+
+	track->album->artist->expanded = 1;
+	album_to_iter(track->album, &tmpiter);
+	window_set_sel(lib_tree_win, &tmpiter);
+
+	tree_track_to_iter(track, &tmpiter);
+	window_set_sel(lib_track_win, &tmpiter);
+	return 1;
+}
+/* search (tree) }}} */
+
 static void insert_artist(struct artist *artist, struct rb_root *root)
 {
 	struct rb_node **new = &(root->rb_node), *parent = NULL;
@@ -719,36 +762,40 @@ void tree_add_track(struct tree_track *track)
 {
 	const struct track_info *ti = tree_track_info(track);
 	const char *album_name, *artist_name, *artistsort_name = NULL;
+	const char *albumsort_name = NULL;
 	struct artist *artist, *new_artist;
 	struct album *album, *new_album;
 	int date;
 	int is_va_compilation = 0;
 
-	date = ti->date;
+	date = ti->originaldate;
+	if (date < 0)
+		date = ti->date;
 
-	if (is_url(ti->filename)) {
+	if (is_http_url(ti->filename)) {
 		artist_name = "<Stream>";
 		album_name = "<Stream>";
 	} else {
 		album_name	= tree_album_name(ti);
 		artist_name	= tree_artist_name(ti);
 		artistsort_name	= ti->artistsort;
+		albumsort_name	= ti->albumsort;
 
 		is_va_compilation = ti->is_va_compilation;
 	}
 
 	new_artist = artist_new(artist_name, artistsort_name, is_va_compilation);
-	new_album = album = NULL;
+	album = NULL;
 
 	artist = find_artist(new_artist);
 	if (artist) {
 		artist_free(new_artist);
-		new_album = album_new(artist, album_name, date);
+		new_album = album_new(artist, album_name, albumsort_name, date);
 		album = find_album(new_album);
 		if (album)
 			album_free(new_album);
 	} else
-		new_album = album_new(new_artist, album_name, date);
+		new_album = album_new(new_artist, album_name, albumsort_name, date);
 
 	if (artist) {
 		int changed = 0;
