@@ -1,4 +1,5 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
  *
  * This program is free software; you can redistribute it and/or
@@ -12,11 +13,10 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "convert.h"
 #include "ui_curses.h"
 #include "cmdline.h"
 #include "search_mode.h"
@@ -35,6 +35,7 @@
 #include "xstrjoin.h"
 #include "window.h"
 #include "format_print.h"
+#include "comment.h"
 #include "misc.h"
 #include "prog.h"
 #include "uchar.h"
@@ -46,14 +47,19 @@
 #include "worker.h"
 #include "input.h"
 #include "file.h"
+#include "path.h"
+#include "mixer.h"
+#ifdef HAVE_CONFIG
 #include "config/curses.h"
 #include "config/iconv.h"
+#endif
 
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
 #include <sys/ioctl.h>
+#include <sys/select.h>
 #include <ctype.h>
 #include <dirent.h>
 #include <locale.h>
@@ -113,7 +119,7 @@ static char *server_address = NULL;
 
 static char print_buffer[512];
 
-/* destination buffer for utf8_encode and utf8_decode */
+/* destination buffer for utf8_encode_to_buf and utf8_decode */
 static char conv_buffer[512];
 
 /* one character can take up to 4 bytes in UTF-8 */
@@ -138,6 +144,8 @@ static int show_cursor;
 static int cursor_x;
 static int cursor_y;
 
+static char *title_buf = NULL;
+
 enum {
 	CURSED_WIN,
 	CURSED_WIN_CUR,
@@ -206,6 +214,28 @@ static unsigned char cursed_to_fg_idx[NR_CURSED] = {
 	COLOR_INFO
 };
 
+static unsigned char cursed_to_attr_idx[NR_CURSED] = {
+	COLOR_WIN_ATTR,
+	COLOR_WIN_ATTR,
+	COLOR_WIN_INACTIVE_SEL_ATTR,
+	COLOR_WIN_INACTIVE_CUR_SEL_ATTR,
+
+	COLOR_WIN_ATTR,
+	COLOR_WIN_ATTR,
+	COLOR_WIN_SEL_ATTR,
+	COLOR_WIN_CUR_SEL_ATTR,
+
+	COLOR_WIN_ATTR,
+	COLOR_WIN_TITLE_ATTR,
+	COLOR_CMDLINE_ATTR,
+	COLOR_STATUSLINE_ATTR,
+
+	COLOR_TITLELINE_ATTR,
+	COLOR_WIN_ATTR,
+	COLOR_CMDLINE_ATTR,
+	COLOR_CMDLINE_ATTR
+};
+
 /* index is CURSED_*, value is fucking color pair */
 static int pairs[NR_CURSED];
 
@@ -217,17 +247,33 @@ enum {
 	TF_TRACK,
 	TF_TITLE,
 	TF_YEAR,
+	TF_ORIGINALYEAR,
 	TF_GENRE,
 	TF_COMMENT,
 	TF_DURATION,
 	TF_BITRATE,
 	TF_CODEC,
+	TF_CODEC_PROFILE,
 	TF_PATHFILE,
 	TF_FILE,
 	TF_RG_TRACK_GAIN,
 	TF_RG_TRACK_PEAK,
 	TF_RG_ALBUM_GAIN,
 	TF_RG_ALBUM_PEAK,
+	TF_ARRANGER,
+	TF_COMPOSER,
+	TF_CONDUCTOR,
+	TF_LYRICIST,
+	TF_PERFORMER,
+	TF_REMIXER,
+	TF_LABEL,
+	TF_PUBLISHER,
+	TF_WORK,
+	TF_OPUS,
+	TF_PARTNUMBER,
+	TF_PART,
+	TF_SUBTITLE,
+	TF_MEDIA,
 	NR_TFS
 };
 
@@ -239,17 +285,33 @@ static struct format_option track_fopts[NR_TFS + 1] = {
 	DEF_FO_INT('n', "tracknumber", 1),
 	DEF_FO_STR('t', "title", 0),
 	DEF_FO_STR('y', "date", 1),
+	DEF_FO_STR('\0', "originaldate", 1),
 	DEF_FO_STR('g', "genre", 0),
 	DEF_FO_STR('c', "comment", 0),
 	DEF_FO_TIME('d', "duration", 0),
 	DEF_FO_INT('\0', "bitrate", 0),
 	DEF_FO_STR('\0', "codec", 0),
+	DEF_FO_STR('\0', "codec_profile", 0),
 	DEF_FO_STR('f', "path", 0),
 	DEF_FO_STR('F', "filename", 0),
 	DEF_FO_DOUBLE('\0', "rg_track_gain", 0),
 	DEF_FO_DOUBLE('\0', "rg_track_peak", 0),
 	DEF_FO_DOUBLE('\0', "rg_album_gain", 0),
 	DEF_FO_DOUBLE('\0', "rg_album_peak", 0),
+	DEF_FO_STR('\0', "arranger", 0),
+	DEF_FO_STR('\0', "composer", 0),
+	DEF_FO_STR('\0', "conductor", 0),
+	DEF_FO_STR('\0', "lyricist", 0),
+	DEF_FO_STR('\0', "performer", 0),
+	DEF_FO_STR('\0', "remixer", 0),
+	DEF_FO_STR('\0', "label", 0),
+	DEF_FO_STR('\0', "publisher", 0),
+	DEF_FO_STR('\0', "work", 0),
+	DEF_FO_STR('\0', "opus", 0),
+	DEF_FO_STR('\0', "partnumber", 0),
+	DEF_FO_STR('\0', "part", 0),
+	DEF_FO_STR('\0', "subtitle", 0),
+	DEF_FO_STR('\0', "media", 0),
 	DEF_FO_END
 };
 
@@ -266,6 +328,7 @@ enum {
 	SF_CONTINUE,
 	SF_SHUFFLE,
 	SF_PLAYLISTMODE,
+	SF_BITRATE,
 	NR_SFS
 };
 
@@ -282,6 +345,7 @@ static struct format_option status_fopts[NR_SFS + 1] = {
 	DEF_FO_STR('C', NULL, 0),
 	DEF_FO_STR('S', NULL, 0),
 	DEF_FO_STR('L', NULL, 0),
+	DEF_FO_INT('B', NULL, 0),
 	DEF_FO_END
 };
 
@@ -290,7 +354,7 @@ int track_format_valid(const char *format)
 	return format_valid(format, track_fopts);
 }
 
-static void utf8_encode(const char *buffer)
+static void utf8_encode_to_buf(const char *buffer)
 {
 	int n;
 #ifdef HAVE_ICONV
@@ -538,7 +602,7 @@ static void fill_track_fopts_track_info(struct track_info *info)
 	if (using_utf8) {
 		filename = info->filename;
 	} else {
-		utf8_encode(info->filename);
+		utf8_encode_to_buf(info->filename);
 		filename = conv_buffer;
 	}
 
@@ -556,20 +620,29 @@ static void fill_track_fopts_track_info(struct track_info *info)
 	fopt_set_double(&track_fopts[TF_RG_TRACK_PEAK], info->rg_track_peak, isnan(info->rg_track_peak));
 	fopt_set_double(&track_fopts[TF_RG_ALBUM_GAIN], info->rg_album_gain, isnan(info->rg_album_gain));
 	fopt_set_double(&track_fopts[TF_RG_ALBUM_PEAK], info->rg_album_peak, isnan(info->rg_album_peak));
+	fopt_set_str(&track_fopts[TF_ORIGINALYEAR], keyvals_get_val(info->comments, "originaldate"));
 	fopt_set_int(&track_fopts[TF_BITRATE], (int) (info->bitrate / 1000. + 0.5), info->bitrate == -1);
 	fopt_set_str(&track_fopts[TF_CODEC], info->codec);
+	fopt_set_str(&track_fopts[TF_CODEC_PROFILE], info->codec_profile);
 	fopt_set_str(&track_fopts[TF_PATHFILE], filename);
-	if (is_url(info->filename)) {
+	fopt_set_str(&track_fopts[TF_ARRANGER], keyvals_get_val(info->comments, "arranger"));
+	fopt_set_str(&track_fopts[TF_COMPOSER], keyvals_get_val(info->comments, "composer"));
+	fopt_set_str(&track_fopts[TF_CONDUCTOR], keyvals_get_val(info->comments, "conductor"));
+	fopt_set_str(&track_fopts[TF_LYRICIST], keyvals_get_val(info->comments, "lyricist"));
+	fopt_set_str(&track_fopts[TF_PERFORMER], keyvals_get_val(info->comments, "performer"));
+	fopt_set_str(&track_fopts[TF_REMIXER], keyvals_get_val(info->comments, "remixer"));
+	fopt_set_str(&track_fopts[TF_LABEL], keyvals_get_val(info->comments, "label"));
+	fopt_set_str(&track_fopts[TF_PUBLISHER], keyvals_get_val(info->comments, "publisher"));
+	fopt_set_str(&track_fopts[TF_WORK], keyvals_get_val(info->comments, "work"));
+	fopt_set_str(&track_fopts[TF_OPUS], keyvals_get_val(info->comments, "opus"));
+	fopt_set_str(&track_fopts[TF_PARTNUMBER], keyvals_get_val(info->comments, "partnumber"));
+	fopt_set_str(&track_fopts[TF_PART], keyvals_get_val(info->comments, "part"));
+	fopt_set_str(&track_fopts[TF_SUBTITLE], keyvals_get_val(info->comments, "subtitle"));
+	fopt_set_str(&track_fopts[TF_MEDIA], keyvals_get_val(info->comments, "media"));
+	if (is_http_url(info->filename)) {
 		fopt_set_str(&track_fopts[TF_FILE], filename);
 	} else {
-		const char *f;
-
-		f = strrchr(filename, '/');
-		if (f) {
-			fopt_set_str(&track_fopts[TF_FILE], f + 1);
-		} else {
-			fopt_set_str(&track_fopts[TF_FILE], filename);
-		}
+		fopt_set_str(&track_fopts[TF_FILE], path_basename(filename));
 	}
 }
 
@@ -721,7 +794,7 @@ static void print_filter(struct window *win, int row, struct iter *iter)
 
 	e_filter = e->filter;
 	if (!using_utf8) {
-		utf8_encode(e_filter);
+		utf8_encode_to_buf(e_filter);
 		e_filter = conv_buffer;
 	}
 
@@ -856,7 +929,7 @@ static void update_editable_window(struct editable *e, const char *title, const
 		if (using_utf8) {
 			/* already UTF-8 */
 		} else {
-			utf8_encode(filename);
+			utf8_encode_to_buf(filename);
 			filename = conv_buffer;
 		}
 		snprintf(buf, sizeof(buf), "%s %s - %d tracks", title,
@@ -903,7 +976,7 @@ static void update_browser_window(void)
 		/* already UTF-8 */
 		dirname = browser_dir;
 	} else {
-		utf8_encode(browser_dir);
+		utf8_encode_to_buf(browser_dir);
 		dirname = conv_buffer;
 	}
 	snprintf(title, sizeof(title), "Browser - %s", dirname);
@@ -1026,6 +1099,7 @@ static void do_update_statusline(void)
 	fopt_set_int(&status_fopts[SF_RVOLUME], vol_right, 0);
 	fopt_set_int(&status_fopts[SF_BUFFER], buffer_fill, 0);
 	fopt_set_str(&status_fopts[SF_CONTINUE], cont_strs[player_cont]);
+	fopt_set_int(&status_fopts[SF_BITRATE], player_info.current_bitrate / 1000. + 0.5, 0);
 
 	strcpy(format, " %s %p ");
 	if (duration != -1)
@@ -1038,8 +1112,12 @@ static void do_update_statusline(void)
 			strcat(format, "vol: %v ");
 		}
 	}
-	if (player_info.ti && is_url(player_info.ti->filename))
-		strcat(format, "buf: %b ");
+	if (player_info.ti) {
+		if (is_http_url(player_info.ti->filename))
+			strcat(format, "buf: %b ");
+		if (show_current_bitrate && player_info.current_bitrate >= 0)
+			strcat(format, " %B kbps ");
+	}
 	strcat(format, "%=");
 	if (player_repeat_current) {
 		strcat(format, "repeat current");
@@ -1121,7 +1199,7 @@ static void do_update_commandline(void)
 		 * (displayed as <xx>) there would be no problem because bpos
 		 * still equals to cpos, I think.
 		 */
-		utf8_encode(cmdline.line);
+		utf8_encode_to_buf(cmdline.line);
 		str = conv_buffer;
 	}
 
@@ -1227,15 +1305,17 @@ static void do_update_titleline(void)
 
 		use_alt_format = !track_info_has_tag(player_info.ti);
 
-		if (is_url(player_info.ti->filename)) {
+		if (is_http_url(player_info.ti->filename)) {
 			const char *title = get_stream_title();
 
 			if (title != NULL) {
+				free(title_buf);
+				title_buf = to_utf8(title, icecast_default_charset);
 				/*
 				 * StreamTitle overrides radio station name
 				 */
 				use_alt_format = 0;
-				fopt_set_str(&track_fopts[TF_TITLE], title);
+				fopt_set_str(&track_fopts[TF_TITLE], title_buf);
 			}
 		}
 
@@ -1286,7 +1366,7 @@ static int cmdline_cursor_column(void)
 	str = cmdline.line;
 	if (!using_utf8) {
 		/* see do_update_commandline */
-		utf8_encode(cmdline.line);
+		utf8_encode_to_buf(cmdline.line);
 		str = conv_buffer;
 	}
 
@@ -1604,15 +1684,16 @@ void update_colors(void)
 	for (i = 0; i < NR_CURSED; i++) {
 		int bg = colors[cursed_to_bg_idx[i]];
 		int fg = colors[cursed_to_fg_idx[i]];
+		int attr = attrs[cursed_to_attr_idx[i]];
 		int pair = i + 1;
 
 		if (fg >= 8 && fg <= 15) {
 			/* fg colors 8..15 are special (0..7 + bold) */
 			init_pair(pair, fg & 7, bg);
-			pairs[i] = COLOR_PAIR(pair) | (fg & BRIGHT ? A_BOLD : 0);
+			pairs[i] = COLOR_PAIR(pair) | (fg & BRIGHT ? A_BOLD : 0) | attr;
 		} else {
 			init_pair(pair, fg, bg);
-			pairs[i] = COLOR_PAIR(pair);
+			pairs[i] = COLOR_PAIR(pair) | attr;
 		}
 	}
 }
@@ -1646,7 +1727,7 @@ static void spawn_status_program(void)
 
 	player_info_lock();
 	status = player_info.status;
-	if (status == PLAYER_STATUS_PLAYING && player_info.ti && is_url(player_info.ti->filename))
+	if (status == PLAYER_STATUS_PLAYING && player_info.ti && is_http_url(player_info.ti->filename))
 		stream_title = get_stream_title();
 
 	i = 0;
@@ -1656,11 +1737,12 @@ static void spawn_status_program(void)
 	argv[i++] = xstrdup(player_status_names[status]);
 	if (player_info.ti) {
 		static const char *keys[] = {
-			"artist", "album", "discnumber", "tracknumber", "title", "date", NULL
+			"artist", "album", "discnumber", "tracknumber", "title", "date",
+			"musicbrainz_trackid", NULL
 		};
 		int j;
 
-		if (is_url(player_info.ti->filename)) {
+		if (is_http_url(player_info.ti->filename)) {
 			argv[i++] = xstrdup("url");
 		} else {
 			argv[i++] = xstrdup("file");
@@ -2234,11 +2316,11 @@ static void init_all(void)
 
 	if (resume_cmus) {
 		resume_load();
-		cmus_add(play_queue_append, play_queue_autosave_filename, FILE_TYPE_PL, JOB_TYPE_QUEUE);
+		cmus_add(play_queue_append, play_queue_autosave_filename, FILE_TYPE_PL, JOB_TYPE_QUEUE, 0);
 	}
 
-	cmus_add(pl_add_track, pl_autosave_filename, FILE_TYPE_PL, JOB_TYPE_PL);
-	cmus_add(lib_add_track, lib_autosave_filename, FILE_TYPE_PL, JOB_TYPE_LIB);
+	cmus_add(pl_add_track, pl_autosave_filename, FILE_TYPE_PL, JOB_TYPE_PL, 0);
+	cmus_add(lib_add_track, lib_autosave_filename, FILE_TYPE_PL, JOB_TYPE_LIB, 0);
 }
 
 static void exit_all(void)
