@@ -1,4 +1,5 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004 Timo Hirvonen
  *
  * This program is free software; you can redistribute it and/or
@@ -12,15 +13,15 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef _UTILS_H
 #define _UTILS_H
 
+#ifdef HAVE_CONFIG
 #include "config/utils.h"
+#endif
 
 #include <stdlib.h>
 #include <string.h>
@@ -28,13 +29,16 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <time.h>
-#include <inttypes.h>
+#include <stdint.h>
 #ifdef HAVE_BYTESWAP_H
 #include <byteswap.h>
 #endif
 
 #define N_ELEMENTS(array) (sizeof(array) / sizeof((array)[0]))
 
+#define STRINGIZE_HELPER(x) #x
+#define STRINGIZE(x) STRINGIZE_HELPER(x)
+
 #define getentry(ptr, offset, type) (*((type *) ((char *) (ptr) + (offset))))
 
 static inline int min(int a, int b)
@@ -95,6 +99,24 @@ static inline int strcmp0(const char *str1, const char *str2)
 	return strcmp(str1, str2);
 }
 
+static inline int ends_with(const char *str, const char *suffix)
+{
+	return strstr(str, suffix) + strlen(suffix) == str + strlen(str);
+}
+
+static inline uint32_t hash_str(const char *s)
+{
+	const unsigned char *p = (const unsigned char *)s;
+	uint32_t h = 5381;
+
+	while (*p) {
+		h *= 33;
+		h ^= *p++;
+	}
+
+	return h ^ (h >> 16);
+}
+
 static inline time_t file_get_mtime(const char *filename)
 {
 	struct stat s;
@@ -124,11 +146,26 @@ static inline void ms_sleep(int ms)
 	ns_sleep(ms * 1e6);
 }
 
-static inline int is_url(const char *name)
+static inline int is_http_url(const char *name)
 {
 	return strncmp(name, "http://", 7) == 0;
 }
 
+static inline int is_cdda_url(const char *name)
+{
+	return strncmp(name, "cdda://", 7) == 0;
+}
+
+static inline int is_cue_url(const char *name)
+{
+	return strncmp(name, "cue://", 6) == 0;
+}
+
+static inline int is_url(const char *name)
+{
+	return is_http_url(name) || is_cdda_url(name) || is_cue_url(name);
+}
+
 static inline int is_freeform_true(const char *c)
 {
 	return	c[0] == '1' ||
