@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,18 +11,19 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "ip.h"
 #include "xmalloc.h"
 #include "read_wrapper.h"
 #include "debug.h"
+#ifdef HAVE_CONFIG
 #include "config/tremor.h"
+#endif
+#include "comment.h"
 
 #ifdef CONFIG_TREMOR
 #include <tremor/ivorbisfile.h>
@@ -33,6 +35,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <math.h>
 
 struct vorbis_private {
 	OggVorbis_File vf;
@@ -61,10 +64,8 @@ static size_t read_func(void *ptr, size_t size, size_t nmemb, void *datasource)
 static int seek_func(void *datasource, ogg_int64_t offset, int whence)
 {
 	struct input_plugin_data *ip_data = datasource;
-	int rc;
 
-	rc = lseek(ip_data->fd, offset, whence);
-	if (rc == -1)
+	if (lseek(ip_data->fd, offset, whence) == -1)
 		return -1;
 	return 0;
 }
@@ -82,10 +83,10 @@ static int close_func(void *datasource)
 static long tell_func(void *datasource)
 {
 	struct input_plugin_data *ip_data = datasource;
-	int rc;
+	off_t off;
 
-	rc = lseek(ip_data->fd, 0, SEEK_CUR);
-	return rc;
+	off = lseek(ip_data->fd, 0, SEEK_CUR);
+	return (off == -1) ? -1 : off;
 }
 
 /*
@@ -103,6 +104,52 @@ static ov_callbacks callbacks = {
 	.tell_func = tell_func
 };
 
+/* http://xiph.org/vorbis/doc/Vorbis_I_spec.html#x1-800004.3.9 */
+static void channel_map_init_vorbis(int channels, channel_position_t *map)
+{
+	switch (channels) {
+	case 8:
+		channel_map_init_vorbis(7, map);
+		map[5] = CHANNEL_POSITION_REAR_LEFT;
+		map[6] = CHANNEL_POSITION_REAR_RIGHT;
+		map[7] = CHANNEL_POSITION_LFE;
+		break;
+	case 7:
+		channel_map_init_vorbis(3, map);
+		map[3] = CHANNEL_POSITION_SIDE_LEFT;
+		map[4] = CHANNEL_POSITION_SIDE_RIGHT;
+		map[5] = CHANNEL_POSITION_REAR_CENTER;
+		map[6] = CHANNEL_POSITION_LFE;
+		break;
+	case 6:
+		map[5] = CHANNEL_POSITION_LFE;
+		/* Fall through */
+	case 5:
+		map[3] = CHANNEL_POSITION_REAR_LEFT;
+		map[4] = CHANNEL_POSITION_REAR_RIGHT;
+		/* Fall through */
+	case 3:
+		map[0] = CHANNEL_POSITION_FRONT_LEFT;
+		map[1] = CHANNEL_POSITION_CENTER;
+		map[2] = CHANNEL_POSITION_FRONT_RIGHT;
+		break;
+	case 4:
+		map[2] = CHANNEL_POSITION_REAR_LEFT;
+		map[3] = CHANNEL_POSITION_REAR_RIGHT;
+		/* Fall through */
+	case 2:
+		map[0] = CHANNEL_POSITION_FRONT_LEFT;
+		map[1] = CHANNEL_POSITION_FRONT_RIGHT;
+		break;
+	case 1:
+		map[0] = CHANNEL_POSITION_MONO;
+		break;
+	default:
+		map[0] = CHANNEL_POSITION_INVALID;
+		break;
+	}
+}
+
 static int vorbis_open(struct input_plugin_data *ip_data)
 {
 	struct vorbis_private *priv;
@@ -128,6 +175,7 @@ static int vorbis_open(struct input_plugin_data *ip_data)
 #ifdef WORDS_BIGENDIAN
 	ip_data->sf |= sf_bigendian(1);
 #endif
+	channel_map_init_vorbis(vi->channels, ip_data->channel_map);
 	return 0;
 }
 
@@ -256,7 +304,7 @@ static int vorbis_read_comments(struct input_plugin_data *ip_data,
 	vc = ov_comment(&priv->vf, -1);
 	if (vc == NULL) {
 		d_print("vc == NULL\n");
-		*comments = xnew0(struct keyval, 1);
+		*comments = keyvals_new(0);
 		return 0;
 	}
 	for (i = 0; i < vc->comments; i++) {
@@ -302,11 +350,52 @@ static long vorbis_bitrate(struct input_plugin_data *ip_data)
 	return bitrate;
 }
 
+static long vorbis_current_bitrate(struct input_plugin_data *ip_data)
+{
+	struct vorbis_private *priv = ip_data->private;
+	return ov_bitrate_instant(&priv->vf);
+}
+
 static char *vorbis_codec(struct input_plugin_data *ip_data)
 {
 	return xstrdup("vorbis");
 }
 
+static const long rate_mapping_44[2][12] = {
+	{ 32000, 48000, 60000, 70000,  80000,  86000,  96000, 110000, 120000, 140000, 160000, 239920 },
+	{ 45000, 64000, 80000, 96000, 112000, 128000, 160000, 192000, 224000, 256000, 320000, 499821 }
+};
+
+static char *vorbis_codec_profile(struct input_plugin_data *ip_data)
+{
+	struct vorbis_private *priv = ip_data->private;
+	vorbis_info *vi = ov_info(&priv->vf, -1);
+	long b = vi->bitrate_nominal;
+	char buf[64];
+
+	if (b <= 0)
+		return NULL;
+
+	if (vi->channels > 2 || vi->rate < 44100) {
+		sprintf(buf, "%ldkbps", b / 1000);
+	} else {
+		const long *map = rate_mapping_44[vi->channels - 1];
+		float q;
+		int i;
+
+		for (i = 0; i < 12-1; i++) {
+			if (b >= map[i] && b < map[i+1])
+				break;
+		}
+		/* This is used even if upper / lower bitrate are set
+		 * because it gives a good approximation. */
+		q = (i - 1) + (float) (b - map[i]) / (map[i+1] - map[i]);
+		sprintf(buf, "q%g", roundf(q * 100.f) / 100.f);
+	}
+
+	return xstrdup(buf);
+}
+
 const struct input_plugin_ops ip_ops = {
 	.open = vorbis_open,
 	.close = vorbis_close,
@@ -315,9 +404,12 @@ const struct input_plugin_ops ip_ops = {
 	.read_comments = vorbis_read_comments,
 	.duration = vorbis_duration,
 	.bitrate = vorbis_bitrate,
-	.codec = vorbis_codec
+	.bitrate_current = vorbis_current_bitrate,
+	.codec = vorbis_codec,
+	.codec_profile = vorbis_codec_profile
 };
 
 const int ip_priority = 50;
 const char * const ip_extensions[] = { "ogg", NULL };
 const char * const ip_mime_types[] = { "application/ogg", "audio/x-ogg", NULL };
+const char * const ip_options[] = { NULL };
