@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,11 +11,9 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "ip.h"
@@ -22,9 +21,11 @@
 #include "xmalloc.h"
 #include "debug.h"
 #include "utils.h"
+#include "comment.h"
 
 #include <stdio.h>
 #include <string.h>
+#include <strings.h>
 #include <errno.h>
 
 #include <sys/types.h>
@@ -140,7 +141,7 @@ static int wav_open(struct input_plugin_data *ip_data)
 		goto error_exit;
 	}
 	{
-		unsigned int format_tag, channels, rate, bits;
+		unsigned int format_tag, channels, rate, bits, channel_mask = 0;
 
 		format_tag = read_le16(fmt + 0);
 		channels = read_le16(fmt + 2);
@@ -170,8 +171,7 @@ static int wav_open(struct input_plugin_data *ip_data)
 				rc = -IP_ERROR_FILE_FORMAT;
 				goto error_exit;
 			}
-			/* speaker position mask */
-			read_le32(fmt + 20);
+			channel_mask = read_le32(fmt + 20);
 			format_tag = read_le16(fmt + 24);
 			/* ignore rest of extension tag */
 		}
@@ -182,19 +182,19 @@ static int wav_open(struct input_plugin_data *ip_data)
 			rc = -IP_ERROR_UNSUPPORTED_FILE_TYPE;
 			goto error_exit;
 		}
-		if ((bits != 8 && bits != 16 && bits != 24 && bits != 32) || channels < 1 || channels > 2) {
+		if ((bits != 8 && bits != 16 && bits != 24 && bits != 32) || channels < 1) {
 			rc = -IP_ERROR_SAMPLE_FORMAT;
 			goto error_exit;
 		}
 		ip_data->sf = sf_channels(channels) | sf_rate(rate) | sf_bits(bits) |
 			sf_signed(bits > 8);
+		channel_map_init_waveex(channels, channel_mask, ip_data->channel_map);
 	}
 
 	rc = find_chunk(ip_data->fd, "data", &priv->pcm_size);
 	if (rc)
 		goto error_exit;
-	rc = lseek(ip_data->fd, 0, SEEK_CUR);
-	if (rc == -1) {
+	if (lseek(ip_data->fd, 0, SEEK_CUR) == -1) {
 		rc = -IP_ERROR_ERRNO;
 		goto error_exit;
 	}
@@ -260,14 +260,12 @@ static int wav_seek(struct input_plugin_data *ip_data, double _offset)
 {
 	struct wav_private *priv = ip_data->private;
 	unsigned int offset;
-	off_t rc;
 
 	offset = (unsigned int)(_offset * (double)priv->sec_size + 0.5);
 	/* align to frame size */
 	offset -= offset % priv->frame_size;
 	priv->pos = offset;
-	rc = lseek(ip_data->fd, priv->pcm_start + offset, SEEK_SET);
-	if (rc == (off_t)-1)
+	if (lseek(ip_data->fd, priv->pcm_start + offset, SEEK_SET) == -1)
 		return -1;
 	return 0;
 }
@@ -310,9 +308,10 @@ static int wav_read_comments(struct input_plugin_data *ip_data,
 	priv = ip_data->private;
 	id[4] = '\0';
 
-	rc = lseek(ip_data->fd, 12, SEEK_SET);
-	if (rc == -1)
+	if (lseek(ip_data->fd, 12, SEEK_SET) == -1) {
+		rc = -1;
 		goto out;
+	}
 
 	while (1) {
 		rc = read_chunk_header(ip_data->fd, id, &size);
@@ -344,9 +343,10 @@ static int wav_read_comments(struct input_plugin_data *ip_data,
 			}
 		}
 
-		rc = lseek(ip_data->fd, size, SEEK_CUR);
-		if (rc == -1)
+		if (lseek(ip_data->fd, size, SEEK_CUR) == -1) {
+			rc = -1;
 			break;
+		}
 	}
 
 out:
@@ -390,6 +390,11 @@ static char *wav_codec(struct input_plugin_data *ip_data)
 	return xstrdup(buf);
 }
 
+static char *wav_codec_profile(struct input_plugin_data *ip_data)
+{
+	return NULL;
+}
+
 const struct input_plugin_ops ip_ops = {
 	.open = wav_open,
 	.close = wav_close,
@@ -398,9 +403,12 @@ const struct input_plugin_ops ip_ops = {
 	.read_comments = wav_read_comments,
 	.duration = wav_duration,
 	.bitrate = wav_bitrate,
-	.codec = wav_codec
+	.bitrate_current = wav_bitrate,
+	.codec = wav_codec,
+	.codec_profile = wav_codec_profile
 };
 
 const int ip_priority = 50;
 const char * const ip_extensions[] = { "wav", NULL };
 const char * const ip_mime_types[] = { NULL };
+const char * const ip_options[] = { NULL };
