@@ -1,4 +1,5 @@
 /*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2007 Johannes Weißl
  *
  * This program is free software; you can redistribute it and/or
@@ -12,9 +13,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "ip.h"
@@ -24,6 +23,7 @@
 #include "read_wrapper.h"
 #include "debug.h"
 #include "buffer.h"
+#include "comment.h"
 
 #include <wavpack/wavpack.h>
 
@@ -37,20 +37,37 @@
 
 #define WV_CHANNEL_MAX 2
 
+struct wavpack_file {
+	int fd;
+	off_t len;
+	int push_back_byte;
+};
+
 struct wavpack_private {
 	WavpackContext *wpc;
-	off_t len;
 	int32_t samples[CHUNK_SIZE * WV_CHANNEL_MAX];
+	struct wavpack_file wv_file;
+	struct wavpack_file wvc_file;
+	unsigned int has_wvc : 1;
 };
 
 /* http://www.wavpack.com/lib_use.txt */
 
 static int32_t read_bytes(void *data, void *ptr, int32_t count)
 {
-	struct input_plugin_data *ip_data = data;
-	int rc;
+	struct wavpack_file *file = data;
+	int32_t rc, n = 0;
 
-	rc = read_wrapper(ip_data, ptr, count);
+	if (file->push_back_byte != EOF) {
+		char *p = ptr;
+		*p = (char) file->push_back_byte;
+		ptr = p + 1;
+		file->push_back_byte = EOF;
+		count--;
+		n++;
+	}
+
+	rc = read(file->fd, ptr, count);
 	if (rc == -1) {
 		d_print("error: %s\n", strerror(errno));
 		return 0;
@@ -59,50 +76,54 @@ static int32_t read_bytes(void *data, void *ptr, int32_t count)
 		errno = 0;
 		return 0;
 	}
-	return rc;
+	return rc + n;
 }
 
 static uint32_t get_pos(void *data)
 {
-	struct input_plugin_data *ip_data = data;
+	struct wavpack_file *file = data;
 
-	return lseek(ip_data->fd, 0, SEEK_CUR);
-}
-
-static int set_pos_abs(void *data, uint32_t pos)
-{
-	struct input_plugin_data *ip_data = data;
-
-	lseek(ip_data->fd, pos, SEEK_SET);
-	return errno;
+	return lseek(file->fd, 0, SEEK_CUR);
 }
 
 static int set_pos_rel(void *data, int32_t delta, int mode)
 {
-	struct input_plugin_data *ip_data = data;
+	struct wavpack_file *file = data;
 
-	lseek(ip_data->fd, delta, mode);
-	return errno;
+	if (lseek(file->fd, delta, mode) == -1)
+		return -1;
+
+	file->push_back_byte = EOF;
+	return 0;
+}
+
+static int set_pos_abs(void *data, uint32_t pos)
+{
+	return set_pos_rel(data, pos, SEEK_SET);
 }
 
 static int push_back_byte(void *data, int c)
 {
-	/* not possible? */
-	d_print("NOT POSSIBLE\n");
-	return EOF;
+	struct wavpack_file *file = data;
+
+	if (file->push_back_byte != EOF) {
+		d_print("error: only one byte push back possible!\n");
+		return EOF;
+	}
+	file->push_back_byte = c;
+	return c;
 }
 
 static uint32_t get_length(void *data)
 {
-	struct input_plugin_data *ip_data = data;
-	struct wavpack_private *priv = ip_data->private;
-	return priv->len;
+	struct wavpack_file *file = data;
+	return file->len;
 }
 
 static int can_seek(void *data)
 {
-	struct input_plugin_data *ip_data = data;
-	return !ip_data->remote;
+	struct wavpack_file *file = data;
+	return file->len != -1;
 }
 
 static int32_t write_bytes(void *data, void *ptr, int32_t count)
@@ -142,18 +163,43 @@ static int wavpack_open(struct input_plugin_data *ip_data)
 	struct wavpack_private *priv;
 	struct stat st;
 	char msg[80];
+	int channel_mask = 0;
+
+	const struct wavpack_private priv_init = {
+		.wv_file = {
+			.fd = ip_data->fd,
+			.push_back_byte = EOF
+		}
+	};
 
 	priv = xnew(struct wavpack_private, 1);
-	priv->wpc = NULL;
-	priv->len = 0;
-	if (!ip_data->remote && !fstat(ip_data->fd, &st))
-		priv->len = st.st_size;
+	*priv = priv_init;
+	if (!ip_data->remote && fstat(ip_data->fd, &st) == 0) {
+		char *filename_wvc;
+
+		priv->wv_file.len = st.st_size;
+
+		filename_wvc = xnew(char, strlen(ip_data->filename) + 2);
+		sprintf(filename_wvc, "%sc", ip_data->filename);
+		if (stat(filename_wvc, &st) == 0) {
+			priv->wvc_file.fd = open(filename_wvc, O_RDONLY);
+			if (priv->wvc_file.fd != -1) {
+				priv->wvc_file.len = st.st_size;
+				priv->wvc_file.push_back_byte = EOF;
+				priv->has_wvc = 1;
+				d_print("use correction file: %s\n", filename_wvc);
+			}
+		}
+		free(filename_wvc);
+	} else
+		priv->wv_file.len = -1;
 	ip_data->private = priv;
 
 	*msg = '\0';
 
-	priv->wpc = WavpackOpenFileInputEx(&callbacks, ip_data, NULL, msg,
-			OPEN_2CH_MAX | OPEN_NORMALIZE, 0);
+	priv->wpc = WavpackOpenFileInputEx(&callbacks, &priv->wv_file,
+			priv->has_wvc ? &priv->wvc_file : NULL, msg,
+			OPEN_NORMALIZE, 0);
 
 	if (!priv->wpc) {
 		d_print("WavpackOpenFileInputEx failed: %s\n", msg);
@@ -165,6 +211,10 @@ static int wavpack_open(struct input_plugin_data *ip_data)
 		| sf_channels(WavpackGetReducedChannels(priv->wpc))
 		| sf_bits(WavpackGetBitsPerSample(priv->wpc))
 		| sf_signed(1);
+#if CUR_STREAM_VERS > 0x404
+	channel_mask = WavpackGetChannelMask(priv->wpc);
+#endif
+	channel_map_init_waveex(sf_get_channels(ip_data->sf), channel_mask, ip_data->channel_map);
 	return 0;
 }
 
@@ -174,6 +224,8 @@ static int wavpack_close(struct input_plugin_data *ip_data)
 
 	priv = ip_data->private;
 	priv->wpc = WavpackCloseFile(priv->wpc);
+	if (priv->has_wvc)
+		close(priv->wvc_file.fd);
 	free(priv);
 	ip_data->private = NULL;
 	return 0;
@@ -318,11 +370,50 @@ static long wavpack_bitrate(struct input_plugin_data *ip_data)
 	return (long) (bitrate + 0.5);
 }
 
+static long wavpack_current_bitrate(struct input_plugin_data *ip_data)
+{
+	struct wavpack_private *priv = ip_data->private;
+	return WavpackGetInstantBitrate(priv->wpc);
+}
+
 static char *wavpack_codec(struct input_plugin_data *ip_data)
 {
 	return xstrdup("wavpack");
 }
 
+static char *wavpack_codec_profile(struct input_plugin_data *ip_data)
+{
+	struct wavpack_private *priv = ip_data->private;
+	int m = WavpackGetMode(priv->wpc);
+	char buf[32];
+
+	buf[0] = '\0';
+
+	if (m & MODE_FAST)
+		strcat(buf, "fast");
+#ifdef MODE_VERY_HIGH
+	else if (m & MODE_VERY_HIGH)
+		strcat(buf, "very high");
+#endif
+	else if (m & MODE_HIGH)
+		strcat(buf, "high");
+	else
+		strcat(buf, "normal");
+
+	if (m & MODE_HYBRID)
+		strcat(buf, " hybrid");
+
+#ifdef MODE_XMODE
+	if ((m & MODE_EXTRA) && (m & MODE_XMODE)) {
+		char xmode[] = " x0";
+		xmode[2] = ((m & MODE_XMODE) >> 12) + '0';
+		strcat(buf, xmode);
+	}
+#endif
+
+	return xstrdup(buf);
+}
+
 const struct input_plugin_ops ip_ops = {
 	.open = wavpack_open,
 	.close = wavpack_close,
@@ -331,9 +422,12 @@ const struct input_plugin_ops ip_ops = {
 	.read_comments = wavpack_read_comments,
 	.duration = wavpack_duration,
 	.bitrate = wavpack_bitrate,
-	.codec = wavpack_codec
+	.bitrate_current = wavpack_current_bitrate,
+	.codec = wavpack_codec,
+	.codec_profile = wavpack_codec_profile
 };
 
 const int ip_priority = 50;
 const char * const ip_extensions[] = { "wv", NULL };
 const char * const ip_mime_types[] = { "audio/x-wavpack", NULL };
+const char * const ip_options[] = { NULL };
