@@ -1,6 +1,7 @@
-/* 
+/*
+ * Copyright 2008-2011 Various Authors
  * Copyright 2004-2005 Timo Hirvonen
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -10,14 +11,13 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "window.h"
+#include "options.h"
 #include "xmalloc.h"
 #include "debug.h"
 
@@ -84,25 +84,51 @@ void window_set_nr_rows(struct window *win, int nr_rows)
 
 void window_up(struct window *win, int rows)
 {
-	int i;
+	struct iter iter;
+	int delta, sel_up, top_up, actual_offset = -1;
 
-	for (i = 0; i < rows; i++) {
-		struct iter prev = win->sel;
-
-		if (!win->get_prev(&prev))
-			break;
-		if (iters_equal(&win->sel, &win->top))
-			win->top = prev;
-		win->sel = prev;
+	/* distance between top and sel */
+	delta = 0;
+	iter = win->top;
+	while (!iters_equal(&iter, &win->sel)) {
+		win->get_next(&iter);
+		delta++;
 	}
-	if (i)
+
+	for (sel_up = 0; sel_up < rows; sel_up++) {
+		iter = win->sel;
+		if (!win->get_prev(&iter)) {
+			actual_offset = 0;
+			break;
+		}
+		win->sel = iter;
+	}
+
+	if (actual_offset == -1) {
+		int upper_bound = win->nr_rows / 2;
+		if (scroll_offset < upper_bound)
+			upper_bound = scroll_offset;
+		for (actual_offset = 0; actual_offset < upper_bound; actual_offset++) {
+			if (!win->get_prev(&iter)) {
+				break;
+			}
+		}
+	}
+
+	top_up = actual_offset + sel_up - delta;
+	while (top_up > 0) {
+		win->get_prev(&win->top);
+		top_up--;
+	}
+
+	if (sel_up)
 		sel_changed(win);
 }
 
 void window_down(struct window *win, int rows)
 {
 	struct iter iter;
-	int delta, sel_down, top_down;
+	int delta, sel_down, top_down, actual_offset = -1;
 
 	/* distance between top and sel */
 	delta = 0;
@@ -114,16 +140,30 @@ void window_down(struct window *win, int rows)
 
 	for (sel_down = 0; sel_down < rows; sel_down++) {
 		iter = win->sel;
-		if (!win->get_next(&iter))
+		if (!win->get_next(&iter)) {
+			actual_offset = 0;
 			break;
+		}
 		win->sel = iter;
 	}
 
-	top_down = sel_down - (win->nr_rows - delta - 1);
+	if (actual_offset == -1) {
+		int upper_bound = (win->nr_rows - 1) / 2;
+		if (scroll_offset < upper_bound)
+			upper_bound = scroll_offset;
+		for (actual_offset = 0; actual_offset < upper_bound; actual_offset++) {
+			if (!win->get_next(&iter)) {
+				break;
+			}
+		}
+	}
+
+	top_down = actual_offset + sel_down - (win->nr_rows - delta - 1);
 	while (top_down > 0) {
 		win->get_next(&win->top);
 		top_down--;
 	}
+
 	if (sel_down)
 		sel_changed(win);
 }
@@ -234,7 +274,8 @@ int window_get_next(struct window *win, struct iter *iter)
 
 void window_set_sel(struct window *win, struct iter *iter)
 {
-	int sel_nr, top_nr;
+	int sel_nr, top_nr, bottom_nr;
+	int upper_bound;
 	struct iter tmp;
 
 	BUG_ON(iter_is_empty(&win->top));
@@ -261,11 +302,50 @@ void window_set_sel(struct window *win, struct iter *iter)
 		sel_nr++;
 	}
 
-	if (sel_nr < top_nr)
-		win->top = win->sel;
-	while (sel_nr - top_nr >= win->nr_rows) {
-		win->get_next(&win->top);
-		top_nr++;
+	upper_bound = win->nr_rows / 2;
+	if (scroll_offset < upper_bound)
+		upper_bound = scroll_offset;
+
+	if (sel_nr < top_nr + upper_bound) { // scroll up
+		tmp = win->head;
+		win->get_next(&tmp);
+		if (sel_nr < upper_bound) { // no space above
+			win->top = tmp;
+		} else {
+			win->top = win->sel;
+			while (upper_bound > 0) {
+				win->get_prev(&win->top);
+				upper_bound--;
+			}
+		}
+	} else { //scroll down
+		upper_bound = (win->nr_rows - 1) / 2;
+		if (scroll_offset < upper_bound)
+			upper_bound = scroll_offset;
+
+		tmp = win->sel;
+		bottom_nr = sel_nr;
+		if (sel_nr >= top_nr + win->nr_rows) { //  seleced element not visible
+			while (sel_nr >= top_nr + win->nr_rows) {
+				win->get_next(&win->top);
+				top_nr++;
+			}
+		} else { // selected element visible
+			while (bottom_nr + 1 < top_nr + win->nr_rows) {
+				if (!win->get_next(&tmp)) { //no space below
+					bottom_nr = sel_nr + upper_bound;
+					break;
+				}
+				bottom_nr++;
+			}
+		}
+
+		while (bottom_nr < sel_nr + upper_bound) {
+			if (!win->get_next(&tmp))
+				break;
+			bottom_nr++;
+			win->get_next(&win->top);
+		}
 	}
 	sel_changed(win);
 }
@@ -314,6 +394,34 @@ void window_page_down(struct window *win)
 	window_down(win, win->nr_rows - 1);
 }
 
+static void window_goto_pos(struct window *win, int pos)
+{
+	struct iter old_sel;
+	int i;
+
+	old_sel = win->sel;
+	win->sel = win->top;
+	for (i = 0; i < pos; i++)
+		win->get_next(&win->sel);
+	if (!iters_equal(&old_sel, &win->sel))
+		sel_changed(win);
+}
+
+void window_page_top(struct window *win)
+{
+	window_goto_pos(win, 0);
+}
+
+void window_page_bottom(struct window *win)
+{
+	window_goto_pos(win, win->nr_rows - 1);
+}
+
+void window_page_middle(struct window *win)
+{
+	window_goto_pos(win, win->nr_rows / 2);
+}
+
 int window_get_nr_rows(struct window *win)
 {
 	return win->nr_rows;
